@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012-2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -21,26 +19,20 @@ import ldap as ldap
 
 from keystone import assignment
 from keystone import clean
-from keystone.common import dependency
+from keystone.common import driver_hints
 from keystone.common import ldap as common_ldap
 from keystone.common import models
 from keystone import config
 from keystone import exception
 from keystone.identity.backends import ldap as ldap_identity
-from keystone.openstack.common import log as logging
+from keystone.openstack.common.gettextutils import _
+from keystone.openstack.common import log
 
 
 CONF = config.CONF
-LOG = logging.getLogger(__name__)
-
-DEFAULT_DOMAIN = {
-    'id': CONF.identity.default_domain_id,
-    'name': 'Default',
-    'enabled': True
-}
+LOG = log.getLogger(__name__)
 
 
-@dependency.requires('identity_api')
 class Assignment(assignment.Driver):
     def __init__(self):
         super(Assignment, self).__init__()
@@ -57,21 +49,24 @@ class Assignment(assignment.Driver):
 
         self.project = ProjectApi(CONF)
         self.role = RoleApi(CONF)
-        self._identity_api = None
 
     def get_project(self, tenant_id):
         return self._set_default_domain(self.project.get(tenant_id))
 
-    def list_projects(self, domain_id=None):
-        # We don't support multiple domains within this driver, so ignore
-        # any domain passed.
+    def list_projects(self, hints):
         return self._set_default_domain(self.project.get_all())
 
+    def list_projects_in_domain(self, domain_id):
+        # We don't support multiple domains within this driver, so ignore
+        # any domain specified
+        return self.list_projects(driver_hints.Hints())
+
     def get_project_by_name(self, tenant_name, domain_id):
         self._validate_default_domain_id(domain_id)
         return self._set_default_domain(self.project.get_by_name(tenant_name))
 
     def create_project(self, tenant_id, tenant):
+        self.project.check_allow_create()
         tenant = self._validate_default_domain(tenant)
         tenant['name'] = clean.project_name(tenant['name'])
         data = tenant.copy()
@@ -82,6 +77,7 @@ class Assignment(assignment.Driver):
         return self._set_default_domain(self.project.create(data))
 
     def update_project(self, tenant_id, tenant):
+        self.project.check_allow_update()
         tenant = self._validate_default_domain(tenant)
         if 'name' in tenant:
             tenant['name'] = clean.project_name(tenant['name'])
@@ -91,20 +87,40 @@ class Assignment(assignment.Driver):
                       domain_id=None, group_id=None):
 
         def _get_roles_for_just_user_and_project(user_id, tenant_id):
-            self.identity_api.get_user(user_id)
             self.get_project(tenant_id)
             return [self.role._dn_to_id(a.role_dn)
                     for a in self.role.get_role_assignments
                     (self.project._id_to_dn(tenant_id))
                     if self.user._dn_to_id(a.user_dn) == user_id]
 
+        def _get_roles_for_group_and_project(group_id, project_id):
+            self.get_project(project_id)
+            group_dn = self.group._id_to_dn(group_id)
+            # NOTE(marcos-fermin-lobo): In Active Directory, for functions
+            # such as "self.role.get_role_assignments", it returns
+            # the key "CN" or "OU" in uppercase.
+            # The group_dn var has "CN" and "OU" in lowercase.
+            # For this reason, it is necessary to use the "upper()"
+            # function so both are consistent.
+            return [self.role._dn_to_id(a.role_dn)
+                    for a in self.role.get_role_assignments
+                    (self.project._id_to_dn(project_id))
+                    if a.user_dn.upper() == group_dn.upper()]
+
         if domain_id is not None:
-            msg = 'Domain metadata not supported by LDAP'
+            msg = _('Domain metadata not supported by LDAP')
             raise exception.NotImplemented(message=msg)
-        if tenant_id is None or user_id is None:
+        if group_id is None and user_id is None:
             return {}
 
-        metadata_ref = _get_roles_for_just_user_and_project(user_id, tenant_id)
+        if tenant_id is None:
+            return {}
+        if user_id is None:
+            metadata_ref = _get_roles_for_group_and_project(group_id,
+                                                            tenant_id)
+        else:
+            metadata_ref = _get_roles_for_just_user_and_project(user_id,
+                                                                tenant_id)
         if not metadata_ref:
             return {}
         return {'roles': [self._role_to_dict(r, False) for r in metadata_ref]}
@@ -112,14 +128,13 @@ class Assignment(assignment.Driver):
     def get_role(self, role_id):
         return self.role.get(role_id)
 
-    def list_roles(self):
+    def list_roles(self, hints):
         return self.role.get_all()
 
-    def list_projects_for_user(self, user_id, group_ids):
+    def list_projects_for_user(self, user_id, group_ids, hints):
         # NOTE(henry-nash): The LDAP backend is being deprecated, so no
         # support is provided for projects that the user has a role on solely
         # by virtue of group membership.
-        self.identity_api.get_user(user_id)
         user_dn = self.user._id_to_dn(user_id)
         associations = (self.role.list_project_roles_for_user
                         (user_dn, self.project.tree_dn))
@@ -129,6 +144,15 @@ class Assignment(assignment.Driver):
         return [self._set_default_domain(x) for x in
                 self.project.get_user_projects(user_dn, associations)]
 
+    def get_roles_for_groups(self, group_ids, project_id=None, domain_id=None):
+        raise exception.NotImplemented()
+
+    def list_projects_for_groups(self, group_ids):
+        raise exception.NotImplemented()
+
+    def list_domains_for_groups(self, group_ids):
+        raise exception.NotImplemented()
+
     def list_user_ids_for_project(self, tenant_id):
         self.get_project(tenant_id)
         tenant_dn = self.project._id_to_dn(tenant_id)
@@ -145,28 +169,35 @@ class Assignment(assignment.Driver):
                                  self.project._id_to_dn(tenant_id))
 
     def add_role_to_user_and_project(self, user_id, tenant_id, role_id):
-        self.identity_api.get_user(user_id)
         self.get_project(tenant_id)
         self.get_role(role_id)
         user_dn = self.user._id_to_dn(user_id)
         role_dn = self._subrole_id_to_dn(role_id, tenant_id)
         self.role.add_user(role_id, role_dn, user_dn, user_id, tenant_id)
         tenant_dn = self.project._id_to_dn(tenant_id)
-        return UserRoleAssociation(
-            role_dn=role_dn,
-            user_dn=user_dn,
-            tenant_dn=tenant_dn)
+        return UserRoleAssociation(role_dn=role_dn,
+                                   user_dn=user_dn,
+                                   tenant_dn=tenant_dn)
 
-    def _create_metadata(self, user_id, tenant_id, metadata):
-        return {}
+    def _add_role_to_group_and_project(self, group_id, tenant_id, role_id):
+        self.get_project(tenant_id)
+        self.get_role(role_id)
+        group_dn = self.group._id_to_dn(group_id)
+        role_dn = self._subrole_id_to_dn(role_id, tenant_id)
+        self.role.add_user(role_id, role_dn, group_dn, group_id, tenant_id)
+        tenant_dn = self.project._id_to_dn(tenant_id)
+        return GroupRoleAssociation(group_dn=group_dn,
+                                    role_dn=role_dn,
+                                    tenant_dn=tenant_dn)
 
     def create_role(self, role_id, role):
+        self.role.check_allow_create()
         try:
             self.get_role(role_id)
         except exception.NotFound:
             pass
         else:
-            msg = 'Duplicate ID, %s.' % role_id
+            msg = _('Duplicate ID, %s.') % role_id
             raise exception.Conflict(type='role', details=msg)
 
         try:
@@ -174,17 +205,19 @@ class Assignment(assignment.Driver):
         except exception.NotFound:
             pass
         else:
-            msg = 'Duplicate name, %s.' % role['name']
+            msg = _('Duplicate name, %s.') % role['name']
             raise exception.Conflict(type='role', details=msg)
 
         return self.role.create(role)
 
     def delete_role(self, role_id):
+        self.role.check_allow_delete()
         return self.role.delete(role_id, self.project.tree_dn)
 
     def delete_project(self, tenant_id):
+        self.project.check_allow_delete()
         if self.project.subtree_delete_enabled:
-            self.project.deleteTree(id)
+            self.project.deleteTree(tenant_id)
         else:
             tenant_dn = self.project._id_to_dn(tenant_id)
             self.role.roles_delete_subtree_by_project(tenant_dn)
@@ -197,30 +230,39 @@ class Assignment(assignment.Driver):
                                      self.project._id_to_dn(tenant_id),
                                      user_id, role_id)
 
+    def _remove_role_from_group_and_project(self, group_id, tenant_id,
+                                            role_id):
+        role_dn = self._subrole_id_to_dn(role_id, tenant_id)
+        return self.role.delete_user(role_dn,
+                                     self.group._id_to_dn(group_id),
+                                     self.project._id_to_dn(tenant_id),
+                                     group_id, role_id)
+
     def update_role(self, role_id, role):
+        self.role.check_allow_update()
         self.get_role(role_id)
-        self.role.update(role_id, role)
+        return self.role.update(role_id, role)
 
     def create_domain(self, domain_id, domain):
         if domain_id == CONF.identity.default_domain_id:
-            msg = 'Duplicate ID, %s.' % domain_id
+            msg = _('Duplicate ID, %s.') % domain_id
             raise exception.Conflict(type='domain', details=msg)
-        raise exception.Forbidden('Domains are read-only against LDAP')
+        raise exception.Forbidden(_('Domains are read-only against LDAP'))
 
     def get_domain(self, domain_id):
         self._validate_default_domain_id(domain_id)
-        return DEFAULT_DOMAIN
+        return assignment.calc_default_domain()
 
     def update_domain(self, domain_id, domain):
         self._validate_default_domain_id(domain_id)
-        raise exception.Forbidden('Domains are read-only against LDAP')
+        raise exception.Forbidden(_('Domains are read-only against LDAP'))
 
     def delete_domain(self, domain_id):
         self._validate_default_domain_id(domain_id)
-        raise exception.Forbidden('Domains are read-only against LDAP')
+        raise exception.Forbidden(_('Domains are read-only against LDAP'))
 
-    def list_domains(self):
-        return [assignment.DEFAULT_DOMAIN]
+    def list_domains(self, hints):
+        return [assignment.calc_default_domain()]
 
 #Bulk actions on User From identity
     def delete_user(self, user_id):
@@ -246,19 +288,127 @@ class Assignment(assignment.Driver):
             # role support which will be added under bug 1101287
             query = '(objectClass=%s)' % self.group.object_class
             dn = None
-            dn = self.group._id_to_dn(id)
+            dn = self.group._id_to_dn(group_id)
             if dn:
                 try:
                     conn = self.group.get_connection()
                     roles = conn.search_s(dn, ldap.SCOPE_ONELEVEL,
                                           query, ['%s' % '1.1'])
-                    for role_dn, _ in roles:
+                    for role_dn, i in roles:
                         conn.delete_s(role_dn)
                 except ldap.NO_SUCH_OBJECT:
                     pass
                 finally:
                     conn.unbind_s()
 
+    def create_grant(self, role_id, user_id=None, group_id=None,
+                     domain_id=None, project_id=None,
+                     inherited_to_projects=False):
+        self.get_role(role_id)
+
+        if domain_id:
+            self.get_domain(domain_id)
+        if project_id:
+            self.get_project(project_id)
+
+        if project_id and inherited_to_projects:
+            msg = _('Inherited roles can only be assigned to domains')
+            raise exception.Conflict(type='role grant', details=msg)
+
+        try:
+            metadata_ref = self._get_metadata(user_id, project_id,
+                                              domain_id, group_id)
+        except exception.MetadataNotFound:
+            metadata_ref = {}
+
+        if user_id is None:
+            metadata_ref['roles'] = self._add_role_to_group_and_project(
+                group_id, project_id, role_id)
+        else:
+            metadata_ref['roles'] = self.add_role_to_user_and_project(
+                user_id, project_id, role_id)
+
+    def get_grant(self, role_id, user_id=None, group_id=None,
+                  domain_id=None, project_id=None,
+                  inherited_to_projects=False):
+        role_ref = self.get_role(role_id)
+
+        if domain_id:
+            self.get_domain(domain_id)
+        if project_id:
+            self.get_project(project_id)
+
+        try:
+            metadata_ref = self._get_metadata(user_id, project_id,
+                                              domain_id, group_id)
+        except exception.MetadataNotFound:
+            metadata_ref = {}
+        role_ids = set(self._roles_from_role_dicts(
+            metadata_ref.get('roles', []), inherited_to_projects))
+        if role_id not in role_ids:
+            raise exception.RoleNotFound(role_id=role_id)
+        return role_ref
+
+    def delete_grant(self, role_id, user_id=None, group_id=None,
+                     domain_id=None, project_id=None,
+                     inherited_to_projects=False):
+        self.get_role(role_id)
+
+        if domain_id:
+            self.get_domain(domain_id)
+        if project_id:
+            self.get_project(project_id)
+
+        try:
+            metadata_ref = self._get_metadata(user_id, project_id,
+                                              domain_id, group_id)
+        except exception.MetadataNotFound:
+            metadata_ref = {}
+
+        try:
+            if user_id is None:
+                metadata_ref['roles'] = (
+                    self._remove_role_from_group_and_project(
+                        group_id, project_id, role_id))
+            else:
+                metadata_ref['roles'] = self.remove_role_from_user_and_project(
+                    user_id, project_id, role_id)
+        except KeyError:
+            raise exception.RoleNotFound(role_id=role_id)
+
+    def list_grants(self, user_id=None, group_id=None,
+                    domain_id=None, project_id=None,
+                    inherited_to_projects=False):
+        if domain_id:
+            self.get_domain(domain_id)
+        if project_id:
+            self.get_project(project_id)
+
+        try:
+            metadata_ref = self._get_metadata(user_id, project_id,
+                                              domain_id, group_id)
+        except exception.MetadataNotFound:
+            metadata_ref = {}
+
+        return [self.get_role(x) for x in
+                self._roles_from_role_dicts(metadata_ref.get('roles', []),
+                                            inherited_to_projects)]
+
+    def get_domain_by_name(self, domain_name):
+        default_domain = assignment.calc_default_domain()
+        if domain_name != default_domain['name']:
+            raise exception.DomainNotFound(domain_id=domain_name)
+        return default_domain
+
+    def list_role_assignments(self):
+        role_assignments = []
+        for a in self.role.list_role_assignments(self.project.tree_dn):
+            assignment = {'role_id': self.role._dn_to_id(a.role_dn),
+                          'user_id': self.user._dn_to_id(a.user_dn),
+                          'project_id': self.project._dn_to_id(a.project_dn)}
+            role_assignments.append(assignment)
+        return role_assignments
+
 
 # TODO(termie): turn this into a data object and move logic to driver
 class ProjectApi(common_ldap.EnabledEmuMixIn, common_ldap.BaseLdap):
@@ -346,9 +496,9 @@ class ProjectApi(common_ldap.EnabledEmuMixIn, common_ldap.BaseLdap):
                 res.add(rolegrant.user_dn)
         return list(res)
 
-    def update(self, id, values):
-        old_obj = self.get(id)
-        return super(ProjectApi, self).update(id, values, old_obj)
+    def update(self, project_id, values):
+        old_obj = self.get(project_id)
+        return super(ProjectApi, self).update(project_id, values, old_obj)
 
 
 class UserRoleAssociation(object):
@@ -388,8 +538,8 @@ class RoleApi(common_ldap.BaseLdap):
         self.member_attribute = (getattr(conf.ldap, 'role_member_attribute')
                                  or self.DEFAULT_MEMBER_ATTRIBUTE)
 
-    def get(self, id, filter=None):
-        model = super(RoleApi, self).get(id, filter)
+    def get(self, role_id, role_filter=None):
+        model = super(RoleApi, self).get(role_id, role_filter)
         return model
 
     def create(self, values):
@@ -401,8 +551,9 @@ class RoleApi(common_ldap.BaseLdap):
             conn.modify_s(role_dn, [(ldap.MOD_ADD,
                                      self.member_attribute, user_dn)])
         except ldap.TYPE_OR_VALUE_EXISTS:
-            msg = ('User %s already has role %s in tenant %s'
-                   % (user_id, role_id, tenant_id))
+            msg = (_('User %(user_id)s already has role %(role_id)s in '
+                     'tenant %(tenant_id)s') %
+                   dict(user_id=user_id, role_id=role_id, tenant_id=tenant_id))
             raise exception.Conflict(type='role grant', details=msg)
         except ldap.NO_SUCH_OBJECT:
             if tenant_id is None or self.get(role_id) is None:
@@ -513,15 +664,15 @@ class RoleApi(common_ldap.BaseLdap):
     def update(self, role_id, role):
         try:
             old_name = self.get_by_name(role['name'])
-            raise exception.Conflict('Cannot duplicate name %s' % old_name)
+            raise exception.Conflict(_('Cannot duplicate name %s') % old_name)
         except exception.NotFound:
             pass
         return super(RoleApi, self).update(role_id, role)
 
-    def delete(self, id, tenant_dn):
+    def delete(self, role_id, tenant_dn):
         conn = self.get_connection()
         query = '(&(objectClass=%s)(%s=%s))' % (self.object_class,
-                                                self.id_attr, id)
+                                                self.id_attr, role_id)
         try:
             for role_dn, _ in conn.search_s(tenant_dn,
                                             ldap.SCOPE_SUBTREE,
@@ -531,4 +682,43 @@ class RoleApi(common_ldap.BaseLdap):
             pass
         finally:
             conn.unbind_s()
-        super(RoleApi, self).delete(id)
+        super(RoleApi, self).delete(role_id)
+
+    def list_role_assignments(self, project_tree_dn):
+        """Returns a list of all the role assignments linked to project_tree_dn
+        attribute.
+        """
+        conn = self.get_connection()
+        query = '(objectClass=%s)' % (self.object_class)
+        try:
+            roles = conn.search_s(project_tree_dn,
+                                  ldap.SCOPE_SUBTREE,
+                                  query)
+        except ldap.NO_SUCH_OBJECT:
+            return []
+        finally:
+            conn.unbind_s()
+
+        res = []
+        for role_dn, role in roles:
+            tenant = ldap.dn.str2dn(role_dn)
+            tenant.pop(0)
+            # It obtains the tenant DN to construct the UserRoleAssociation
+            # object.
+            tenant_dn = ldap.dn.dn2str(tenant)
+            for user_dn in role[self.member_attribute]:
+                # NOTE(nkinder): Ideally, this comparison would be aware of the
+                # Distinguished Name LDAP syntax. Since Keystone is responsible
+                # for setting the dumb member DN, we are relatively sure that
+                # it is returned in the same form. We still need to do a case
+                # insensitive comparison since attribute names will be upper
+                # case for AD. We already do this elsewhere in the LDAP
+                # driver, so it's OK until we decide to become syntax aware.
+                if (self.use_dumb_member and
+                        user_dn.lower() == self.dumb_member.lower()):
+                    continue
+                res.append(UserRoleAssociation(
+                           user_dn=user_dn,
+                           role_dn=role_dn,
+                           tenant_dn=tenant_dn))
+        return res
