@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,47 +12,94 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import json
+
+from keystoneclient.common import cms
+import six
 
 from keystone.common import controller
 from keystone.common import dependency
 from keystone.common import wsgi
 from keystone import config
+from keystone.contrib import federation
 from keystone import exception
-from keystone import identity
+from keystone.openstack.common.gettextutils import _
 from keystone.openstack.common import importutils
-from keystone.openstack.common import log as logging
-from keystone import token
-from keystone import trust
+from keystone.openstack.common import log
+from keystone.openstack.common import timeutils
 
 
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 CONF = config.CONF
 
 # registry of authentication methods
 AUTH_METHODS = {}
+AUTH_PLUGINS_LOADED = False
 
 
-def load_auth_method(method_name):
-    if method_name not in CONF.auth.methods:
-        raise exception.AuthMethodNotSupported()
-    driver = CONF.auth.get(method_name)
-    return importutils.import_object(driver)
+def load_auth_methods():
+    global AUTH_PLUGINS_LOADED
+
+    if AUTH_PLUGINS_LOADED:
+        # Only try and load methods a single time.
+        return
+    # config.setup_authentication should be idempotent, call it to ensure we
+    # have setup all the appropriate configuration options we may need.
+    config.setup_authentication()
+    for plugin in CONF.auth.methods:
+        if '.' in plugin:
+            # NOTE(morganfainberg): if '.' is in the plugin name, it should be
+            # imported rather than used as a plugin identifier.
+            plugin_class = plugin
+            driver = importutils.import_object(plugin)
+            if not hasattr(driver, 'method'):
+                raise ValueError(_('Cannot load an auth-plugin by class-name '
+                                   'without a "method" attribute defined: %s'),
+                                 plugin_class)
+        else:
+            plugin_class = CONF.auth.get(plugin)
+            driver = importutils.import_object(plugin_class)
+            if hasattr(driver, 'method'):
+                if driver.method != plugin:
+                    raise ValueError(_('Driver requested method %(req)s does '
+                                       'not match plugin name %(plugin)s.') %
+                                     {'req': driver.method,
+                                      'plugin': plugin})
+            else:
+                LOG.warning(_('Auth Plugin %s does not have a "method" '
+                              'attribute.'), plugin)
+                setattr(driver, 'method', plugin)
+        if driver.method in AUTH_METHODS:
+            raise ValueError(_('Auth plugin %(plugin)s is requesting '
+                               'previously registered method %(method)s') %
+                             {'plugin': plugin_class, 'method': driver.method})
+        AUTH_METHODS[driver.method] = driver
+    AUTH_PLUGINS_LOADED = True
 
 
 def get_auth_method(method_name):
     global AUTH_METHODS
     if method_name not in AUTH_METHODS:
-        AUTH_METHODS[method_name] = load_auth_method(method_name)
+        raise exception.AuthMethodNotSupported()
     return AUTH_METHODS[method_name]
 
 
+# TODO(blk-u): this class doesn't use identity_api directly, but makes it
+# available for consumers. Consumers should probably not be getting
+# identity_api from this since it's available in global registry, then
+# identity_api should be removed from this list.
+@dependency.requires('assignment_api', 'identity_api', 'trust_api')
 class AuthInfo(object):
     """Encapsulation of "auth" request."""
 
+    @staticmethod
+    def create(context, auth=None):
+        auth_info = AuthInfo(context, auth=auth)
+        auth_info._validate_and_normalize_auth_data()
+        return auth_info
+
     def __init__(self, context, auth=None):
-        self.identity_api = identity.Manager()
-        self.trust_api = trust.Manager()
         self.context = context
         self.auth = auth
         self._scope_data = (None, None, None)
@@ -63,7 +108,6 @@ class AuthInfo(object):
         # domain scope: (domain_id, None, None)
         # trust scope: (None, None, trust_ref)
         # unscoped: (None, None, None)
-        self._validate_and_normalize_auth_data()
 
     def _assert_project_is_enabled(self, project_ref):
         # ensure the project is enabled
@@ -78,12 +122,6 @@ class AuthInfo(object):
             LOG.warning(msg)
             raise exception.Unauthorized(msg)
 
-    def _assert_user_is_enabled(self, user_ref):
-        if not user_ref.get('enabled', True):
-            msg = _('User is disabled: %s') % (user_ref['id'])
-            LOG.warning(msg)
-            raise exception.Unauthorized(msg)
-
     def _lookup_domain(self, domain_info):
         domain_id = domain_info.get('id')
         domain_name = domain_info.get('name')
@@ -93,9 +131,10 @@ class AuthInfo(object):
                                             target='domain')
         try:
             if domain_name:
-                domain_ref = self.identity_api.get_domain_by_name(domain_name)
+                domain_ref = self.assignment_api.get_domain_by_name(
+                    domain_name)
             else:
-                domain_ref = self.identity_api.get_domain(domain_id)
+                domain_ref = self.assignment_api.get_domain(domain_id)
         except exception.DomainNotFound as e:
             LOG.exception(e)
             raise exception.Unauthorized(e)
@@ -115,10 +154,10 @@ class AuthInfo(object):
                     raise exception.ValidationError(attribute='domain',
                                                     target='project')
                 domain_ref = self._lookup_domain(project_info['domain'])
-                project_ref = self.identity_api.get_project_by_name(
+                project_ref = self.assignment_api.get_project_by_name(
                     project_name, domain_ref['id'])
             else:
-                project_ref = self.identity_api.get_project(project_id)
+                project_ref = self.assignment_api.get_project(project_id)
         except exception.ProjectNotFound as e:
             LOG.exception(e)
             raise exception.Unauthorized(e)
@@ -135,29 +174,6 @@ class AuthInfo(object):
             raise exception.TrustNotFound(trust_id=trust_id)
         return trust
 
-    def lookup_user(self, user_info):
-        user_id = user_info.get('id')
-        user_name = user_info.get('name')
-        user_ref = None
-        if not user_id and not user_name:
-            raise exception.ValidationError(attribute='id or name',
-                                            target='user')
-        try:
-            if user_name:
-                if 'domain' not in user_info:
-                    raise exception.ValidationError(attribute='domain',
-                                                    target='user')
-                domain_ref = self._lookup_domain(user_info['domain'])
-                user_ref = self.identity_api.get_user_by_name(
-                    user_name, domain_ref['id'])
-            else:
-                user_ref = self.identity_api.get_user(user_id)
-        except exception.UserNotFound as e:
-            LOG.exception(e)
-            raise exception.Unauthorized(e)
-        self._assert_user_is_enabled(user_ref)
-        return user_ref
-
     def _validate_and_normalize_scope_data(self):
         """Validate and normalize scope data."""
         if 'scope' not in self.auth:
@@ -206,7 +222,7 @@ class AuthInfo(object):
 
         # make sure auth method is supported
         for method_name in self.get_method_names():
-            if method_name not in CONF.auth.methods:
+            if method_name not in AUTH_METHODS:
                 raise exception.AuthMethodNotSupported()
 
     def _validate_and_normalize_auth_data(self):
@@ -276,11 +292,25 @@ class AuthInfo(object):
         self._scope_data = (domain_id, project_id, trust)
 
 
-@dependency.requires('token_provider_api')
+@dependency.requires('assignment_api', 'identity_api', 'token_api',
+                     'token_provider_api', 'trust_api')
 class Auth(controller.V3Controller):
+
+    # Note(atiwari): From V3 auth controller code we are
+    # calling protection() wrappers, so we need to setup
+    # the member_name and  collection_name attributes of
+    # auth controller code.
+    # In the absence of these attributes, default 'entity'
+    # string will be used to represent the target which is
+    # generic. Policy can be defined using 'entity' but it
+    # would not reflect the exact entity that is in context.
+    # We are defining collection_name = 'tokens' and
+    # member_name = 'token' to facilitate policy decisions.
+    collection_name = 'tokens'
+    member_name = 'token'
+
     def __init__(self, *args, **kw):
         super(Auth, self).__init__(*args, **kw)
-        self.token_controllers_ref = token.controllers.Auth()
         config.setup_authentication()
 
     def authenticate_for_token(self, context, auth=None):
@@ -288,13 +318,17 @@ class Auth(controller.V3Controller):
         include_catalog = 'nocatalog' not in context['query_string']
 
         try:
-            auth_info = AuthInfo(context, auth=auth)
+            auth_info = AuthInfo.create(context, auth=auth)
             auth_context = {'extras': {}, 'method_names': [], 'bind': {}}
             self.authenticate(context, auth_info, auth_context)
             if auth_context.get('access_token_id'):
                 auth_info.set_scope(None, auth_context['project_id'], None)
             self._check_and_set_default_scoping(auth_info, auth_context)
             (domain_id, project_id, trust) = auth_info.get_scope()
+
+            if trust:
+                self.trust_api.consume_use(trust['id'])
+
             method_names = auth_info.get_method_names()
             method_names += auth_context.get('method_names', [])
             # make sure the list is unique
@@ -321,22 +355,61 @@ class Auth(controller.V3Controller):
             # scope is specified
             return
 
+        # Skip scoping when unscoped federated token is being issued
+        if federation.IDENTITY_PROVIDER in auth_context:
+            return
+
         # fill in default_project_id if it is available
         try:
             user_ref = self.identity_api.get_user(auth_context['user_id'])
-            default_project_id = user_ref.get('default_project_id')
-            if default_project_id:
-                auth_info.set_scope(domain_id=None,
-                                    project_id=default_project_id)
         except exception.UserNotFound as e:
             LOG.exception(e)
             raise exception.Unauthorized(e)
 
+        default_project_id = user_ref.get('default_project_id')
+        if not default_project_id:
+            # User has no default project. He shall get an unscoped token.
+            return
+
+        # make sure user's default project is legit before scoping to it
+        try:
+            default_project_ref = self.assignment_api.get_project(
+                default_project_id)
+            default_project_domain_ref = self.assignment_api.get_domain(
+                default_project_ref['domain_id'])
+            if (default_project_ref.get('enabled', True) and
+                    default_project_domain_ref.get('enabled', True)):
+                if self.assignment_api.get_roles_for_user_and_project(
+                        user_ref['id'], default_project_id):
+                    auth_info.set_scope(project_id=default_project_id)
+                else:
+                    msg = _("User %(user_id)s doesn't have access to"
+                            " default project %(project_id)s. The token will"
+                            " be unscoped rather than scoped to the project.")
+                    LOG.warning(msg,
+                                {'user_id': user_ref['id'],
+                                 'project_id': default_project_id})
+            else:
+                msg = _("User %(user_id)s's default project %(project_id)s is"
+                        " disabled. The token will be unscoped rather than"
+                        " scoped to the project.")
+                LOG.warning(msg,
+                            {'user_id': user_ref['id'],
+                             'project_id': default_project_id})
+        except (exception.ProjectNotFound, exception.DomainNotFound):
+            # default project or default project domain doesn't exist,
+            # will issue unscoped token instead
+            msg = _("User %(user_id)s's default project %(project_id)s not"
+                    " found. The token will be unscoped rather than"
+                    " scoped to the project.")
+            LOG.warning(msg, {'user_id': user_ref['id'],
+                              'project_id': default_project_id})
+
     def authenticate(self, context, auth_info, auth_context):
         """Authenticate user."""
 
         # user has been authenticated externally
-        if 'REMOTE_USER' in context:
+        if 'REMOTE_USER' in context['environment']:
             external = get_auth_method('external')
             external.authenticate(context, auth_info, auth_context)
 
@@ -352,7 +425,7 @@ class Auth(controller.V3Controller):
                 auth_response['methods'].append(method_name)
                 auth_response[method_name] = resp
 
-        if len(auth_response["methods"]) > 0:
+        if auth_response["methods"]:
             # authentication continuation required
             raise exception.AdditionalAuthRequired(auth_response)
 
@@ -360,13 +433,6 @@ class Auth(controller.V3Controller):
             msg = _('User not found')
             raise exception.Unauthorized(msg)
 
-        # NOTE(nkinder): We only log this message in Havana.  In Icehouse and
-        # later, we emit a CADF notification upon successful token issuance
-        # instead.
-        msg = _('Issued token for user %(u_id)s')
-        msg = msg % {'u_id': auth_context['user_id']}
-        LOG.info(msg)
-
     @controller.protected()
     def check_token(self, context):
         token_id = context.get('subject_token_id')
@@ -380,12 +446,30 @@ class Auth(controller.V3Controller):
     @controller.protected()
     def validate_token(self, context):
         token_id = context.get('subject_token_id')
-        token_data = self.token_provider_api.validate_v3_token(token_id)
+        include_catalog = 'nocatalog' not in context['query_string']
+        token_data = self.token_provider_api.validate_v3_token(
+            token_id)
+        if not include_catalog and 'catalog' in token_data['token']:
+            del token_data['token']['catalog']
         return render_token_data_response(token_id, token_data)
 
     @controller.protected()
     def revocation_list(self, context, auth=None):
-        return self.token_controllers_ref.revocation_list(context, auth)
+        if not CONF.token.revoke_by_id:
+            raise exception.Gone()
+        tokens = self.token_api.list_revoked_tokens()
+
+        for t in tokens:
+            expires = t['expires']
+            if not (expires and isinstance(expires, six.text_type)):
+                    t['expires'] = timeutils.isotime(expires)
+        data = {'revoked': tokens}
+        json_data = json.dumps(data)
+        signed_text = cms.cms_sign_text(json_data,
+                                        CONF.signing.certfile,
+                                        CONF.signing.keyfile)
+
+        return {'signed': signed_text}
 
 
 #FIXME(gyee): not sure if it belongs here or keystone.common. Park it here
