@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2013 OpenStack LLC
+# Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -14,26 +12,29 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-
-from keystone.common import logging
 from keystone import auth
+from keystone.common import dependency
 from keystone import exception
-from keystone import identity
-
+from keystone.openstack.common.gettextutils import _
+from keystone.openstack.common import log
 
 METHOD_NAME = 'password'
 
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 
+@dependency.requires('assignment_api', 'identity_api')
 class UserAuthInfo(object):
-    def __init__(self, context, auth_payload):
-        self.identity_api = identity.Manager()
-        self.context = context
+    @staticmethod
+    def create(auth_payload):
+        user_auth_info = UserAuthInfo()
+        user_auth_info._validate_and_normalize_auth_data(auth_payload)
+        return user_auth_info
+
+    def __init__(self):
         self.user_id = None
         self.password = None
         self.user_ref = None
-        self._validate_and_normalize_auth_data(auth_payload)
 
     def _assert_domain_is_enabled(self, domain_ref):
         if not domain_ref.get('enabled'):
@@ -56,11 +57,10 @@ class UserAuthInfo(object):
                                             target='domain')
         try:
             if domain_name:
-                domain_ref = self.identity_api.get_domain_by_name(
-                    context=self.context, domain_name=domain_name)
+                domain_ref = self.assignment_api.get_domain_by_name(
+                    domain_name)
             else:
-                domain_ref = self.identity_api.get_domain(
-                    context=self.context, domain_id=domain_id)
+                domain_ref = self.assignment_api.get_domain(domain_id)
         except exception.DomainNotFound as e:
             LOG.exception(e)
             raise exception.Unauthorized(e)
@@ -78,7 +78,7 @@ class UserAuthInfo(object):
         if not user_id and not user_name:
             raise exception.ValidationError(attribute='id or name',
                                             target='user')
-        self.password = user_info.get('password', None)
+        self.password = user_info.get('password')
         try:
             if user_name:
                 if 'domain' not in user_info:
@@ -86,13 +86,11 @@ class UserAuthInfo(object):
                                                     target='user')
                 domain_ref = self._lookup_domain(user_info['domain'])
                 user_ref = self.identity_api.get_user_by_name(
-                    context=self.context, user_name=user_name,
-                    domain_id=domain_ref['id'])
+                    user_name, domain_ref['id'])
             else:
-                user_ref = self.identity_api.get_user(
-                    context=self.context, user_id=user_id)
-                domain_ref = self.identity_api.get_domain(
-                    context=self.context, domain_id=user_ref['domain_id'])
+                user_ref = self.identity_api.get_user(user_id)
+                domain_ref = self.assignment_api.get_domain(
+                    user_ref['domain_id'])
                 self._assert_domain_is_enabled(domain_ref)
         except exception.UserNotFound as e:
             LOG.exception(e)
@@ -100,18 +98,30 @@ class UserAuthInfo(object):
         self._assert_user_is_enabled(user_ref)
         self.user_ref = user_ref
         self.user_id = user_ref['id']
+        self.domain_id = domain_ref['id']
 
 
+@dependency.requires('identity_api')
 class Password(auth.AuthMethodHandler):
-    def authenticate(self, context, auth_payload, user_context):
-        """ Try to authenticate against the identity backend. """
-        user_info = UserAuthInfo(context, auth_payload)
 
-        # FIXME: identity.authenticate() can use some refactoring since
+    method = METHOD_NAME
+
+    def authenticate(self, context, auth_payload, user_context):
+        """Try to authenticate against the identity backend."""
+        user_info = UserAuthInfo.create(auth_payload)
+
+        # FIXME(gyee): identity.authenticate() can use some refactoring since
         # all we care is password matches
-        user_auth_data = self.identity_api.authenticate(
-            context=context,
-            user_id=user_info.user_id,
-            password=user_info.password)
+        try:
+            self.identity_api.authenticate(
+                context,
+                user_id=user_info.user_id,
+                password=user_info.password,
+                domain_scope=user_info.domain_id)
+        except AssertionError:
+            # authentication failed because of invalid username or password
+            msg = _('Invalid username or password')
+            raise exception.Unauthorized(msg)
+
         if 'user_id' not in user_context:
             user_context['user_id'] = user_info.user_id
