@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation
 # Copyright 2012 Canonical Ltd.
 #
@@ -17,15 +15,21 @@
 
 """Main entry point into the Catalog service."""
 
+import abc
+
+import six
+
 from keystone.common import dependency
+from keystone.common import driver_hints
 from keystone.common import manager
 from keystone import config
 from keystone import exception
-from keystone.openstack.common import log as logging
+from keystone.openstack.common.gettextutils import _
+from keystone.openstack.common import log
 
 
 CONF = config.CONF
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 
 def format_url(url, data):
@@ -35,19 +39,19 @@ def format_url(url, data):
     except AttributeError:
         return None
     except KeyError as e:
-        LOG.error(_("Malformed endpoint %(url)s - unknown key %(keyerror)s") %
+        LOG.error(_("Malformed endpoint %(url)s - unknown key %(keyerror)s"),
                   {"url": url,
-                   "keyerror": str(e)})
+                   "keyerror": e})
         raise exception.MalformedEndpoint(endpoint=url)
     except TypeError as e:
         LOG.error(_("Malformed endpoint %(url)s - unknown key %(keyerror)s"
-                    "(are you missing brackets ?)") %
+                    "(are you missing brackets ?)"),
                   {"url": url,
-                   "keyerror": str(e)})
+                   "keyerror": e})
         raise exception.MalformedEndpoint(endpoint=url)
     except ValueError as e:
-        LOG.error(_("Malformed endpoint %s - incomplete format \
-                  (are you missing a type notifier ?)") % url)
+        LOG.error(_("Malformed endpoint %s - incomplete format "
+                    "(are you missing a type notifier ?)"), url)
         raise exception.MalformedEndpoint(endpoint=url)
     return result
 
@@ -64,6 +68,38 @@ class Manager(manager.Manager):
     def __init__(self):
         super(Manager, self).__init__(CONF.catalog.driver)
 
+    def create_region(self, region_ref):
+        # Check duplicate ID
+        try:
+            self.get_region(region_ref['id'])
+        except exception.RegionNotFound:
+            pass
+        else:
+            msg = _('Duplicate ID, %s.') % region_ref['id']
+            raise exception.Conflict(type='region', details=msg)
+
+        try:
+            return self.driver.create_region(region_ref)
+        except exception.NotFound:
+            parent_region_id = region_ref.get('parent_region_id')
+            raise exception.RegionNotFound(region_id=parent_region_id)
+
+    def get_region(self, region_id):
+        try:
+            return self.driver.get_region(region_id)
+        except exception.NotFound:
+            raise exception.RegionNotFound(region_id=region_id)
+
+    def delete_region(self, region_id):
+        try:
+            return self.driver.delete_region(region_id)
+        except exception.NotFound:
+            raise exception.RegionNotFound(region_id=region_id)
+
+    def create_service(self, service_id, service_ref):
+        service_ref.setdefault('enabled', True)
+        return self.driver.create_service(service_id, service_ref)
+
     def get_service(self, service_id):
         try:
             return self.driver.get_service(service_id)
@@ -76,6 +112,10 @@ class Manager(manager.Manager):
         except exception.NotFound:
             raise exception.ServiceNotFound(service_id=service_id)
 
+    @manager.response_truncated
+    def list_services(self, hints=None):
+        return self.driver.list_services(hints or driver_hints.Hints())
+
     def create_endpoint(self, endpoint_id, endpoint_ref):
         try:
             return self.driver.create_endpoint(endpoint_id, endpoint_ref)
@@ -95,6 +135,10 @@ class Manager(manager.Manager):
         except exception.NotFound:
             raise exception.EndpointNotFound(endpoint_id=endpoint_id)
 
+    @manager.response_truncated
+    def list_endpoints(self, hints=None):
+        return self.driver.list_endpoints(hints or driver_hints.Hints())
+
     def get_catalog(self, user_id, tenant_id, metadata=None):
         try:
             return self.driver.get_catalog(user_id, tenant_id, metadata)
@@ -102,8 +146,62 @@ class Manager(manager.Manager):
             raise exception.NotFound('Catalog not found for user and tenant')
 
 
+@six.add_metaclass(abc.ABCMeta)
 class Driver(object):
     """Interface description for an Catalog driver."""
+
+    def _get_list_limit(self):
+        return CONF.catalog.list_limit or CONF.list_limit
+
+    @abc.abstractmethod
+    def create_region(self, region_ref):
+        """Creates a new region.
+
+        :raises: keystone.exception.Conflict
+        :raises: keystone.exception.RegionNotFound (if parent region invalid)
+
+        """
+        raise exception.NotImplemented()
+
+    @abc.abstractmethod
+    def list_regions(self):
+        """List all regions.
+
+        :returns: list of region_refs or an empty list.
+
+        """
+        raise exception.NotImplemented()
+
+    @abc.abstractmethod
+    def get_region(self, region_id):
+        """Get region by id.
+
+        :returns: region_ref dict
+        :raises: keystone.exception.RegionNotFound
+
+        """
+        raise exception.NotImplemented()
+
+    @abc.abstractmethod
+    def update_region(self, region_id):
+        """Update region by id.
+
+        :returns: region_ref dict
+        :raises: keystone.exception.RegionNotFound
+
+        """
+        raise exception.NotImplemented()
+
+    @abc.abstractmethod
+    def delete_region(self, region_id):
+        """Deletes an existing region.
+
+        :raises: keystone.exception.RegionNotFound
+
+        """
+        raise exception.NotImplemented()
+
+    @abc.abstractmethod
     def create_service(self, service_id, service_ref):
         """Creates a new service.
 
@@ -112,6 +210,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def list_services(self):
         """List all services.
 
@@ -120,6 +219,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def get_service(self, service_id):
         """Get service by id.
 
@@ -129,6 +229,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def update_service(self, service_id):
         """Update service by id.
 
@@ -138,6 +239,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def delete_service(self, service_id):
         """Deletes an existing service.
 
@@ -146,6 +248,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def create_endpoint(self, endpoint_id, endpoint_ref):
         """Creates a new endpoint for a service.
 
@@ -155,6 +258,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def get_endpoint(self, endpoint_id):
         """Get endpoint by id.
 
@@ -164,6 +268,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def list_endpoints(self):
         """List all endpoints.
 
@@ -172,6 +277,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def update_endpoint(self, endpoint_id, endpoint_ref):
         """Get endpoint by id.
 
@@ -182,6 +288,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def delete_endpoint(self, endpoint_id):
         """Deletes an endpoint for a service.
 
@@ -190,6 +297,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def get_catalog(self, user_id, tenant_id, metadata=None):
         """Retrieve and format the current service catalog.
 
@@ -214,6 +322,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def get_v3_catalog(self, user_id, tenant_id, metadata=None):
         """Retrieve and format the current V3 service catalog.
 
