@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2012 OpenStack LLC
+# Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -19,6 +17,17 @@ from keystone.common import router
 
 
 def append_v3_routers(mapper, routers):
+    regions_controller = controllers.RegionV3()
+    routers.append(router.Router(regions_controller,
+                                 'regions', 'region'))
+
+    # Need to add an additional route to support PUT /regions/{region_id}
+    mapper.connect(
+        '/regions/{region_id}',
+        controller=regions_controller,
+        action='create_region_with_id',
+        conditions=dict(method=['PUT']))
+
     routers.append(router.Router(controllers.ServiceV3(),
                                  'services', 'service'))
     routers.append(router.Router(controllers.EndpointV3(),
