@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -18,10 +16,11 @@ import functools
 import os
 
 from keystone.common import config
-from keystone.openstack.common import log as logging
+from keystone.openstack.common.gettextutils import _
+from keystone.openstack.common import log
 
 CONF = config.CONF
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 
 __all__ = ['Server', 'httplib', 'subprocess']
