@@ -1,3 +1,5 @@
+# flake8: noqa
+
 # Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -12,4 +14,4 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystone.common.sql.core import *  # noqa
+from keystone.common.sql.core import *
