@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,25 +12,25 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-
-from sqlalchemy import *
+import sqlalchemy as sql
 
 
 def upgrade(migrate_engine):
     # Upgrade operations go here. Don't create your own engine; bind
 
-    meta = MetaData()
+    meta = sql.MetaData()
     meta.bind = migrate_engine
-    token = Table('token', meta, autoload=True)
+    token = sql.Table('token', meta, autoload=True)
     # creating the column immediately with nullable=False fails with
     # PostgreSQL (LP 1068181), so do it in two steps instead
-    valid = Column("valid", Boolean(), ColumnDefault(True), nullable=True)
+    valid = sql.Column(
+        'valid', sql.Boolean(), sql.ColumnDefault(True), nullable=True)
     valid.create(token, populate_default=True)
-    valid.alter(type=Boolean(), default=True, nullable=False)
+    valid.alter(type=sql.Boolean(), default=True, nullable=False)
 
 
 def downgrade(migrate_engine):
-    meta = MetaData()
+    meta = sql.MetaData()
     meta.bind = migrate_engine
-    token = Table('token', meta, autoload=True)
+    token = sql.Table('token', meta, autoload=True)
     token.drop_column('valid')
