@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2012 OpenStack LLC
+# Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -18,11 +16,11 @@ import json
 
 import sqlalchemy as sql
 from sqlalchemy import orm
+
 from keystone import config
 
 
 CONF = config.CONF
-DEFAULT_DOMAIN_ID = CONF.identity.default_domain_id
 
 
 def upgrade(migrate_engine):
@@ -33,7 +31,7 @@ def upgrade(migrate_engine):
     domain_table = sql.Table('domain', meta, autoload=True)
 
     domain = {
-        'id': DEFAULT_DOMAIN_ID,
+        'id': CONF.identity.default_domain_id,
         'name': 'Default',
         'enabled': True,
         'extra': json.dumps({
@@ -54,5 +52,6 @@ def downgrade(migrate_engine):
     sql.Table('domain', meta, autoload=True)
     session = orm.sessionmaker(bind=migrate_engine)()
     session.execute(
-        'DELETE FROM domain WHERE id=:id', {'id': DEFAULT_DOMAIN_ID})
+        'DELETE FROM domain WHERE id=:id',
+        {'id': CONF.identity.default_domain_id})
     session.commit()
