@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
@@ -26,16 +24,20 @@ import os
 import pwd
 
 import passlib.hash
+import six
 
 from keystone.common import config
 from keystone.common import environment
 from keystone import exception
-from keystone.openstack.common import log as logging
+from keystone.openstack.common.gettextutils import _
+from keystone.openstack.common import log
+from keystone.openstack.common import strutils
+from six import moves
 
 
 CONF = config.CONF
 
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 
 def read_cached_file(filename, cache_info, reload_func=None):
@@ -72,7 +74,7 @@ def trunc_password(password):
     try:
         if len(password) > max_length:
             LOG.warning(
-                _('Truncating user password to %s characters.') % max_length)
+                _('Truncating user password to %s characters.'), max_length)
         return password[:max_length]
     except TypeError:
         raise exception.ValidationError(attribute='string', target='password')
@@ -86,32 +88,27 @@ def hash_access_key(access):
 
 def hash_user_password(user):
     """Hash a user dict's password without modifying the passed-in dict."""
-    try:
-        password = user['password']
-    except KeyError:
+    password = user.get('password')
+    if password is None:
         return user
-    else:
-        return dict(user, password=hash_password(password))
+
+    return dict(user, password=hash_password(password))
 
 
 def hash_ldap_user_password(user):
     """Hash a user dict's password without modifying the passed-in dict."""
-    try:
-        password = user['password']
-    except KeyError:
+    password = user.get('password')
+    if password is None:
         return user
-    else:
-        return dict(user, password=ldap_hash_password(password))
+
+    return dict(user, password=ldap_hash_password(password))
 
 
 def hash_password(password):
     """Hash a password. Hard."""
     password_utf8 = trunc_password(password).encode('utf-8')
-    if passlib.hash.sha512_crypt.identify(password_utf8):
-        return password_utf8
-    h = passlib.hash.sha512_crypt.encrypt(password_utf8,
-                                          rounds=CONF.crypt_strength)
-    return h
+    return passlib.hash.sha512_crypt.encrypt(
+        password_utf8, rounds=CONF.crypt_strength)
 
 
 def ldap_hash_password(password):
@@ -141,6 +138,18 @@ def check_password(password, hashed):
     return passlib.hash.sha512_crypt.verify(password_utf8, hashed)
 
 
+def attr_as_boolean(val_attr):
+    """Returns the boolean value, decoded from a string.
+
+    We test explicitly for a value meaning False, which can be one of
+    several formats as specified in oslo strutils.FALSE_STRINGS.
+    All other string values (including an empty string) are treated as
+    meaning True.
+
+    """
+    return strutils.bool_from_string(val_attr, default=True)
+
+
 # From python 2.7
 def check_output(*popenargs, **kwargs):
     r"""Run command with arguments and return its output as a byte string.
@@ -241,7 +250,7 @@ def auth_str_equal(provided, known):
     result = 0
     p_len = len(provided)
     k_len = len(known)
-    for i in xrange(p_len):
+    for i in moves.range(p_len):
         a = ord(provided[i]) if i < p_len else 0
         b = ord(known[i]) if i < k_len else 0
         result |= a ^ b
@@ -331,7 +340,7 @@ def get_unix_user(user=None):
     :return: tuple of (uid, name)
     '''
 
-    if isinstance(user, basestring):
+    if isinstance(user, six.string_types):
         try:
             user_info = pwd.getpwnam(user)
         except KeyError:
@@ -387,7 +396,7 @@ def get_unix_group(group=None):
     :return: tuple of (gid, name)
     '''
 
-    if isinstance(group, basestring):
+    if isinstance(group, six.string_types):
         try:
             group_info = grp.getgrnam(group)
         except KeyError:
@@ -516,15 +525,3 @@ def make_dirs(path, mode=None, user=None, group=None, log=None):
             raise EnvironmentError("makedirs('%s'): %s" % (path, exc.strerror))
 
     set_permissions(path, mode, user, group, log)
-
-
-class WhiteListedItemFilter(object):
-
-    def __init__(self, whitelist, data):
-        self._whitelist = set(whitelist or [])
-        self._data = data
-
-    def __getitem__(self, name):
-        if name not in self._whitelist:
-            raise KeyError
-        return self._data[name]
