@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
@@ -20,23 +18,24 @@
 
 """Utility methods for working with WSGI servers."""
 
-import re
-
 import routes.middleware
+import six
 import webob.dec
 import webob.exc
 
 from keystone.common import config
+from keystone.common import dependency
 from keystone.common import utils
 from keystone import exception
 from keystone.openstack.common import gettextutils
+from keystone.openstack.common.gettextutils import _
 from keystone.openstack.common import importutils
 from keystone.openstack.common import jsonutils
-from keystone.openstack.common import log as logging
+from keystone.openstack.common import log
 
 
 CONF = config.CONF
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 # Environment variable used to pass the request context
 CONTEXT_ENV = 'openstack.context'
@@ -46,34 +45,6 @@ CONTEXT_ENV = 'openstack.context'
 PARAMS_ENV = 'openstack.params'
 
 
-_RE_PASS = re.compile(r'([\'"].*?password[\'"]\s*:\s*u?[\'"]).*?([\'"])',
-                      re.DOTALL)
-
-
-def mask_password(message, is_unicode=False, secret="***"):
-    """Replace password with 'secret' in message.
-
-    :param message: The string which include security information.
-    :param is_unicode: Is unicode string ?
-    :param secret: substitution string default to "***".
-    :returns: The string
-
-    For example:
-       >>> mask_password('"password" : "aaaaa"')
-       '"password" : "***"'
-       >>> mask_password("'original_password' : 'aaaaa'")
-       "'original_password' : '***'"
-       >>> mask_password("u'original_password' :   u'aaaaa'")
-       "u'original_password' :   u'***'"
-    """
-    if is_unicode:
-        message = unicode(message)
-    # Match the group 1,2 and replace all others with 'secret'
-    secret = r"\g<1>" + secret + r"\g<2>"
-    result = _RE_PASS.sub(secret, message)
-    return result
-
-
 def validate_token_bind(context, token_ref):
     bind_mode = CONF.token.enforce_token_bind
 
@@ -100,13 +71,14 @@ def validate_token_bind(context, token_ref):
         LOG.info(_("Named bind mode %s not in bind information"), name)
         raise exception.Unauthorized()
 
-    for bind_type, identifier in bind.iteritems():
+    for bind_type, identifier in six.iteritems(bind):
         if bind_type == 'kerberos':
-            if not context.get('AUTH_TYPE', '').lower() == 'negotiate':
+            if not (context['environment'].get('AUTH_TYPE', '').lower()
+                    == 'negotiate'):
                 LOG.info(_("Kerberos credentials required and not present"))
                 raise exception.Unauthorized()
 
-            if not context.get('REMOTE_USER') == identifier:
+            if not context['environment'].get('REMOTE_USER') == identifier:
                 LOG.info(_("Kerberos credentials do not match those in bind"))
                 raise exception.Unauthorized()
 
@@ -123,16 +95,15 @@ def validate_token_bind(context, token_ref):
             raise exception.Unauthorized()
 
 
-class Request(webob.Request):
-    def best_match_language(self):
-        """Determines the best available locale from the Accept-Language
-        HTTP header passed in the request.
-        """
+def best_match_language(req):
+    """Determines the best available locale from the Accept-Language
+    HTTP header passed in the request.
+    """
 
-        if not self.accept_language:
-            return None
-        return self.accept_language.best_match(
-            gettextutils.get_available_languages('keystone'))
+    if not req.accept_language:
+        return None
+    return req.accept_language.best_match(
+        gettextutils.get_available_languages('keystone'))
 
 
 class BaseApplication(object):
@@ -166,7 +137,7 @@ class BaseApplication(object):
     def __call__(self, environ, start_response):
         r"""Subclasses will probably want to implement __call__ like this:
 
-        @webob.dec.wsgify(RequestClass=Request)
+        @webob.dec.wsgify()
         def __call__(self, req):
           # Any of the following objects work as responses:
 
@@ -174,7 +145,7 @@ class BaseApplication(object):
           res = 'message\n'
 
           # Option 2: a nicely formatted HTTP exception page
-          res = exc.HTTPForbidden(detail='Nice try')
+          res = exc.HTTPForbidden(explanation='Nice try')
 
           # Option 3: a webob Response object (in case you need to play with
           # headers, or you want to be treated like an iterable, or or or)
@@ -200,8 +171,9 @@ class BaseApplication(object):
         raise NotImplementedError('You must implement __call__')
 
 
+@dependency.requires('assignment_api', 'policy_api', 'token_api')
 class Application(BaseApplication):
-    @webob.dec.wsgify(RequestClass=Request)
+    @webob.dec.wsgify()
     def __call__(self, req):
         arg_dict = req.environ['wsgiorg.routing_args'][1]
         action = arg_dict.pop('action')
@@ -210,19 +182,16 @@ class Application(BaseApplication):
 
         # allow middleware up the stack to provide context, params and headers.
         context = req.environ.get(CONTEXT_ENV, {})
-        context['query_string'] = dict(req.params.iteritems())
-        context['headers'] = dict(req.headers.iteritems())
+        context['query_string'] = dict(six.iteritems(req.params))
+        context['headers'] = dict(six.iteritems(req.headers))
         context['path'] = req.environ['PATH_INFO']
+        context['host_url'] = req.host_url
         params = req.environ.get(PARAMS_ENV, {})
-
-        for name in ['REMOTE_USER', 'AUTH_TYPE']:
-            try:
-                context[name] = req.environ[name]
-            except KeyError:
-                try:
-                    del context[name]
-                except KeyError:
-                    pass
+        #authentication and authorization attributes are set as environment
+        #values by the container and processed by the pipeline.  the complete
+        #set is not yet know.
+        context['environment'] = req.environ
+        req.environ = None
 
         params.update(arg_dict)
 
@@ -238,24 +207,28 @@ class Application(BaseApplication):
             result = method(context, **params)
         except exception.Unauthorized as e:
             LOG.warning(
-                _('Authorization failed. %(exception)s from %(remote_addr)s') %
+                _('Authorization failed. %(exception)s from %(remote_addr)s'),
                 {'exception': e, 'remote_addr': req.environ['REMOTE_ADDR']})
-            return render_exception(e, user_locale=req.best_match_language())
+            return render_exception(e, context=context,
+                                    user_locale=best_match_language(req))
         except exception.Error as e:
             LOG.warning(e)
-            return render_exception(e, user_locale=req.best_match_language())
+            return render_exception(e, context=context,
+                                    user_locale=best_match_language(req))
         except TypeError as e:
             LOG.exception(e)
             return render_exception(exception.ValidationError(e),
-                                    user_locale=req.best_match_language())
+                                    context=context,
+                                    user_locale=best_match_language(req))
         except Exception as e:
             LOG.exception(e)
             return render_exception(exception.UnexpectedError(exception=e),
-                                    user_locale=req.best_match_language())
+                                    context=context,
+                                    user_locale=best_match_language(req))
 
         if result is None:
             return render_response(status=(204, 'No Content'))
-        elif isinstance(result, basestring):
+        elif isinstance(result, six.string_types):
             return result
         elif isinstance(result, webob.Response):
             return result
@@ -278,7 +251,7 @@ class Application(BaseApplication):
 
     def _normalize_dict(self, d):
         return dict([(self._normalize_arg(k), v)
-                     for (k, v) in d.iteritems()])
+                     for (k, v) in six.iteritems(d)])
 
     def assert_admin(self, context):
         if not context['is_admin']:
@@ -303,11 +276,53 @@ class Application(BaseApplication):
                 raise exception.Unauthorized()
 
             # NOTE(vish): this is pretty inefficient
-            creds['roles'] = [self.identity_api.get_role(role)['name']
+            creds['roles'] = [self.assignment_api.get_role(role)['name']
                               for role in creds.get('roles', [])]
             # Accept either is_admin or the admin role
             self.policy_api.enforce(creds, 'admin_required', {})
 
+    def _require_attribute(self, ref, attr):
+        """Ensures the reference contains the specified attribute."""
+        if ref.get(attr) is None or ref.get(attr) == '':
+            msg = _('%s field is required and cannot be empty') % attr
+            raise exception.ValidationError(message=msg)
+
+    def _get_trust_id_for_request(self, context):
+        """Get the trust_id for a call.
+
+        Retrieve the trust_id from the token
+        Returns None if token is is not trust scoped
+        """
+        if ('token_id' not in context or
+                context.get('token_id') == CONF.admin_token):
+            LOG.debug(_('will not lookup trust as the request auth token is '
+                        'either absent or it is the system admin token'))
+            return None
+
+        try:
+            token_ref = self.token_api.get_token(context['token_id'])
+        except exception.TokenNotFound:
+            LOG.warning(_('Invalid token in _get_trust_id_for_request'))
+            raise exception.Unauthorized()
+
+        return token_ref.get('trust_id')
+
+    @classmethod
+    def base_url(cls, context, endpoint_type):
+        url = CONF['%s_endpoint' % endpoint_type]
+
+        if url:
+            url = url % CONF
+        else:
+            # NOTE(jamielennox): if url is not set via the config file we
+            # should set it relative to the url that the user used to get here
+            # so as not to mess with version discovery. This is not perfect.
+            # host_url omits the path prefix, but there isn't another good
+            # solution that will work for all urls.
+            url = context['host_url']
+
+        return url.rstrip('/')
+
 
 class Middleware(Application):
     """Base WSGI middleware.
@@ -349,6 +364,7 @@ class Middleware(Application):
         return _factory
 
     def __init__(self, application):
+        super(Middleware, self).__init__()
         self.application = application
 
     def process_request(self, request):
@@ -365,7 +381,7 @@ class Middleware(Application):
         """Do whatever you'd like to the response, based on the request."""
         return response
 
-    @webob.dec.wsgify(RequestClass=Request)
+    @webob.dec.wsgify()
     def __call__(self, request):
         try:
             response = self.process_request(request)
@@ -375,16 +391,18 @@ class Middleware(Application):
             return self.process_response(request, response)
         except exception.Error as e:
             LOG.warning(e)
-            return render_exception(e,
-                                    user_locale=request.best_match_language())
+            return render_exception(e, request=request,
+                                    user_locale=best_match_language(request))
         except TypeError as e:
             LOG.exception(e)
             return render_exception(exception.ValidationError(e),
-                                    user_locale=request.best_match_language())
+                                    request=request,
+                                    user_locale=best_match_language(request))
         except Exception as e:
             LOG.exception(e)
             return render_exception(exception.UnexpectedError(exception=e),
-                                    user_locale=request.best_match_language())
+                                    request=request,
+                                    user_locale=best_match_language(request))
 
 
 class Debug(Middleware):
@@ -395,23 +413,23 @@ class Debug(Middleware):
 
     """
 
-    @webob.dec.wsgify(RequestClass=Request)
+    @webob.dec.wsgify()
     def __call__(self, req):
         if not hasattr(LOG, 'isEnabledFor') or LOG.isEnabledFor(LOG.debug):
             LOG.debug('%s %s %s', ('*' * 20), 'REQUEST ENVIRON', ('*' * 20))
             for key, value in req.environ.items():
-                LOG.debug('%s = %s', key, mask_password(value,
-                                                        is_unicode=True))
+                LOG.debug('%s = %s', key,
+                          log.mask_password(value))
             LOG.debug('')
             LOG.debug('%s %s %s', ('*' * 20), 'REQUEST BODY', ('*' * 20))
             for line in req.body_file:
-                LOG.debug(mask_password(line))
+                LOG.debug('%s', log.mask_password(line))
             LOG.debug('')
 
         resp = req.get_response(self.application)
         if not hasattr(LOG, 'isEnabledFor') or LOG.isEnabledFor(LOG.debug):
             LOG.debug('%s %s %s', ('*' * 20), 'RESPONSE HEADERS', ('*' * 20))
-            for (key, value) in resp.headers.iteritems():
+            for (key, value) in six.iteritems(resp.headers):
                 LOG.debug('%s = %s', key, value)
             LOG.debug('')
 
@@ -455,16 +473,11 @@ class Router(object):
           mapper.connect(None, '/v1.0/{path_info:.*}', controller=BlogApp())
 
         """
-        # if we're only running in debug, bump routes' internal logging up a
-        # notch, as it's very spammy
-        if CONF.debug:
-            logging.getLogger('routes.middleware')
-
         self.map = mapper
         self._router = routes.middleware.RoutesMiddleware(self._dispatch,
                                                           self.map)
 
-    @webob.dec.wsgify(RequestClass=Request)
+    @webob.dec.wsgify()
     def __call__(self, req):
         """Route the incoming request to a controller based on self.map.
 
@@ -474,7 +487,7 @@ class Router(object):
         return self._router
 
     @staticmethod
-    @webob.dec.wsgify(RequestClass=Request)
+    @webob.dec.wsgify()
     def _dispatch(req):
         """Dispatch the request to the appropriate controller.
 
@@ -485,9 +498,10 @@ class Router(object):
         """
         match = req.environ['wsgiorg.routing_args'][1]
         if not match:
-            return render_exception(
-                exception.NotFound(_('The resource could not be found.')),
-                user_locale=req.best_match_language())
+            msg = _('The resource could not be found.')
+            return render_exception(exception.NotFound(msg),
+                                    request=req,
+                                    user_locale=best_match_language(req))
         app = match['controller']
         return app
 
@@ -581,14 +595,37 @@ def render_response(body=None, status=None, headers=None):
                           headerlist=headers)
 
 
-def render_exception(error, user_locale=None):
+def render_exception(error, context=None, request=None, user_locale=None):
     """Forms a WSGI response based on the current error."""
+
+    error_message = error.args[0]
+    message = gettextutils.translate(error_message, desired_locale=user_locale)
+    if message is error_message:
+        # translate() didn't do anything because it wasn't a Message,
+        # convert to a string.
+        message = six.text_type(message)
+
     body = {'error': {
         'code': error.code,
         'title': error.title,
-        'message': unicode(gettextutils.get_localized_message(error.args[0],
-                                                              user_locale)),
+        'message': message,
     }}
+    headers = []
     if isinstance(error, exception.AuthPluginException):
         body['error']['identity'] = error.authentication
-    return render_response(status=(error.code, error.title), body=body)
+    elif isinstance(error, exception.Unauthorized):
+        url = CONF.public_endpoint
+        if not url:
+            if request:
+                context = {'host_url': request.host_url}
+            if context:
+                url = Application.base_url(context, 'public')
+            else:
+                url = 'http://localhost:%d' % CONF.public_port
+        else:
+            url = url % CONF
+
+        headers.append(('WWW-Authenticate', 'Keystone uri="%s"' % url))
+    return render_response(status=(error.code, error.title),
+                           body=body,
+                           headers=headers)
