@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -19,27 +17,60 @@ import os
 
 from keystone.common import config
 from keystone import exception
+from keystone.openstack.common import log
 
 
-config.configure()
 CONF = config.CONF
 
-setup_logging = config.setup_logging
 setup_authentication = config.setup_authentication
 configure = config.configure
 
 
+def set_default_for_default_log_levels():
+    """Set the default for the default_log_levels option for keystone.
+
+    Keystone uses some packages that other OpenStack services don't use that do
+    logging. This will set the default_log_levels default level for those
+    packages.
+
+    This function needs to be called before CONF().
+
+    """
+
+    extra_log_level_defaults = [
+        'dogpile=INFO',
+        'routes=INFO',
+    ]
+
+    def find_default_log_levels_opt():
+        for opt in log.log_opts:
+            if opt.dest == 'default_log_levels':
+                return opt
+
+    opt = find_default_log_levels_opt()
+    opt.default.extend(extra_log_level_defaults)
+
+
+def setup_logging():
+    """Sets up logging for the keystone package."""
+    log.setup('keystone')
+
+
 def find_paste_config():
-    """Selects Keystone paste.deploy configuration file.
+    """Find Keystone's paste.deploy configuration file.
+
+    Keystone's paste.deploy configuration file is specified in the
+    ``[paste_deploy]`` section of the main Keystone configuration file,
+    ``keystone.conf``.
+
+    For example::
 
-    Keystone paste.deploy configuration file is selectd in [paste_deploy]
-    section of the main Keystone configuration file.
-    For example:
         [paste_deploy]
         config_file = keystone-paste.ini
 
     :returns: The selected configuration filename
-    :raises: exception.PasteConfigNotFound
+    :raises: exception.ConfigFileNotFound
+
     """
     if CONF.paste_deploy.config_file:
         paste_config = CONF.paste_deploy.config_file
@@ -50,8 +81,11 @@ def find_paste_config():
         paste_config = CONF.config_file[0]
         paste_config_value = paste_config
     else:
+        # this provides backwards compatibility for keystone.conf files that
+        # still have the entire paste configuration included, rather than just
+        # a [paste_deploy] configuration section referring to an external file
         paste_config = CONF.find_file('keystone.conf')
         paste_config_value = 'keystone.conf'
     if not paste_config or not os.path.exists(paste_config):
-        raise exception.PasteConfigNotFound(config_file=paste_config_value)
+        raise exception.ConfigFileNotFound(config_file=paste_config_value)
     return paste_config
