@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -34,19 +32,27 @@ Glance to list images needed to perform the requested task.
 
 """
 
+import abc
 import uuid
 
+import six
+
 from keystoneclient.contrib.ec2 import utils as ec2_utils
 
 from keystone.common import controller
 from keystone.common import dependency
 from keystone.common import utils
+from keystone.common import wsgi
 from keystone import exception
+from keystone.openstack.common.gettextutils import _
+from keystone.openstack.common import jsonutils
 from keystone import token
 
 
-@dependency.requires('catalog_api', 'credential_api', 'token_provider_api')
-class Ec2Controller(controller.V2Controller):
+@dependency.requires('assignment_api', 'catalog_api', 'credential_api',
+                     'identity_api', 'token_api')
+@six.add_metaclass(abc.ABCMeta)
+class Ec2ControllerCommon(object):
     def check_signature(self, creds_ref, credentials):
         signer = ec2_utils.Ec2Signer(creds_ref['secret'])
         signature = signer.generate(credentials)
@@ -63,6 +69,7 @@ class Ec2Controller(controller.V2Controller):
         else:
             raise exception.Unauthorized(message='EC2 signature not supplied.')
 
+    @abc.abstractmethod
     def authenticate(self, context, credentials=None, ec2Credentials=None):
         """Validate a signed EC2 request and provide a token.
 
@@ -83,12 +90,19 @@ class Ec2Controller(controller.V2Controller):
         :returns: token: OpenStack token equivalent to access key along
                          with the corresponding service catalog and roles
         """
+        raise exception.NotImplemented()
+
+    def _authenticate(self, credentials=None, ec2credentials=None):
+        """Common code shared between the V2 and V3 authenticate methods.
+
+        :returns: user_ref, tenant_ref, metadata_ref, roles_ref, catalog_ref
+        """
 
         # FIXME(ja): validate that a service token was used!
 
         # NOTE(termie): backwards compat hack
-        if not credentials and ec2Credentials:
-            credentials = ec2Credentials
+        if not credentials and ec2credentials:
+            credentials = ec2credentials
 
         if 'access' not in credentials:
             raise exception.Unauthorized(message='EC2 signature not supplied.')
@@ -98,12 +112,11 @@ class Ec2Controller(controller.V2Controller):
 
         # TODO(termie): don't create new tokens every time
         # TODO(termie): this is copied from TokenController.authenticate
-        token_id = uuid.uuid4().hex
-        tenant_ref = self.identity_api.get_project(creds_ref['tenant_id'])
+        tenant_ref = self.assignment_api.get_project(creds_ref['tenant_id'])
         user_ref = self.identity_api.get_user(creds_ref['user_id'])
         metadata_ref = {}
         metadata_ref['roles'] = (
-            self.identity_api.get_roles_for_user_and_project(
+            self.assignment_api.get_roles_for_user_and_project(
                 user_ref['id'], tenant_ref['id']))
 
         trust_id = creds_ref.get('trust_id')
@@ -117,23 +130,13 @@ class Ec2Controller(controller.V2Controller):
         roles = metadata_ref.get('roles', [])
         if not roles:
             raise exception.Unauthorized(message='User not valid for tenant.')
-        roles_ref = [self.identity_api.get_role(role_id)
+        roles_ref = [self.assignment_api.get_role(role_id)
                      for role_id in roles]
 
         catalog_ref = self.catalog_api.get_catalog(
             user_ref['id'], tenant_ref['id'], metadata_ref)
 
-        # NOTE(morganfainberg): Make sure the data is in correct form since it
-        # might be consumed external to Keystone and this is a v2.0 controller.
-        # The token provider doesn't actually expect either v2 or v3 user data.
-        user_ref = self.identity_api.v3_to_v2_user(user_ref)
-        auth_token_data = dict(user=user_ref,
-                               tenant=tenant_ref,
-                               metadata=metadata_ref,
-                               id='placeholder')
-        (token_id, token_data) = self.token_provider_api.issue_v2_token(
-            auth_token_data, roles_ref, catalog_ref)
-        return token_data
+        return user_ref, tenant_ref, metadata_ref, roles_ref, catalog_ref
 
     def create_credential(self, context, user_id, tenant_id):
         """Create a secret/access pair for use with ec2 style auth.
@@ -146,77 +149,74 @@ class Ec2Controller(controller.V2Controller):
         :param tenant_id: id of tenant
         :returns: credential: dict of ec2 credential
         """
-        if not self._is_admin(context):
-            self._assert_identity(context, user_id)
 
-        self._assert_valid_user_id(user_id)
-        self._assert_valid_project_id(tenant_id)
-        trust_id = self._context_trust_id(context)
+        self.identity_api.get_user(user_id)
+        self.assignment_api.get_project(tenant_id)
+        trust_id = self._get_trust_id_for_request(context)
         blob = {'access': uuid.uuid4().hex,
                 'secret': uuid.uuid4().hex,
                 'trust_id': trust_id}
         credential_id = utils.hash_access_key(blob['access'])
         cred_ref = {'user_id': user_id,
                     'project_id': tenant_id,
-                    'blob': blob,
+                    'blob': jsonutils.dumps(blob),
                     'id': credential_id,
                     'type': 'ec2'}
         self.credential_api.create_credential(credential_id, cred_ref)
         return {'credential': self._convert_v3_to_ec2_credential(cred_ref)}
 
-    def get_credentials(self, context, user_id):
+    def get_credentials(self, user_id):
         """List all credentials for a user.
 
-        :param context: standard context
         :param user_id: id of user
         :returns: credentials: list of ec2 credential dicts
         """
-        if not self._is_admin(context):
-            self._assert_identity(context, user_id)
-        self._assert_valid_user_id(user_id)
+
+        self.identity_api.get_user(user_id)
         credential_refs = self.credential_api.list_credentials(
             user_id=user_id)
         return {'credentials':
                 [self._convert_v3_to_ec2_credential(credential)
-                for credential in credential_refs]}
+                    for credential in credential_refs]}
 
-    def get_credential(self, context, user_id, credential_id):
+    def get_credential(self, user_id, credential_id):
         """Retrieve a user's access/secret pair by the access key.
 
         Grab the full access/secret pair for a given access key.
 
-        :param context: standard context
         :param user_id: id of user
         :param credential_id: access key for credentials
         :returns: credential: dict of ec2 credential
         """
-        if not self._is_admin(context):
-            self._assert_identity(context, user_id)
-        self._assert_valid_user_id(user_id)
+
+        self.identity_api.get_user(user_id)
         return {'credential': self._get_credentials(credential_id)}
 
-    def delete_credential(self, context, user_id, credential_id):
+    def delete_credential(self, user_id, credential_id):
         """Delete a user's access/secret pair.
 
         Used to revoke a user's access/secret pair
 
-        :param context: standard context
         :param user_id: id of user
         :param credential_id: access key for credentials
         :returns: bool: success
         """
-        if not self._is_admin(context):
-            self._assert_identity(context, user_id)
-            self._assert_owner(user_id, credential_id)
 
-        self._assert_valid_user_id(user_id)
+        self.identity_api.get_user(user_id)
         self._get_credentials(credential_id)
         ec2_credential_id = utils.hash_access_key(credential_id)
         return self.credential_api.delete_credential(ec2_credential_id)
 
-    def _convert_v3_to_ec2_credential(self, credential):
-
-        blob = credential['blob']
+    @staticmethod
+    def _convert_v3_to_ec2_credential(credential):
+        # Prior to bug #1259584 fix, blob was stored unserialized
+        # but it should be stored as a json string for compatibility
+        # with the v3 credentials API.  Fall back to the old behavior
+        # for backwards compatibility with existing DB contents
+        try:
+            blob = jsonutils.loads(credential['blob'])
+        except TypeError:
+            blob = credential['blob']
         return {'user_id': credential.get('user_id'),
                 'tenant_id': credential.get('project_id'),
                 'access': blob.get('access'),
@@ -236,6 +236,58 @@ class Ec2Controller(controller.V2Controller):
             raise exception.Unauthorized(message='EC2 access key not found.')
         return self._convert_v3_to_ec2_credential(creds)
 
+
+@dependency.requires('policy_api', 'token_provider_api')
+class Ec2Controller(Ec2ControllerCommon, controller.V2Controller):
+
+    @controller.v2_deprecated
+    def authenticate(self, context, credentials=None, ec2Credentials=None):
+        (user_ref, tenant_ref, metadata_ref, roles_ref,
+         catalog_ref) = self._authenticate(credentials=credentials,
+                                           ec2credentials=ec2Credentials)
+
+        # NOTE(morganfainberg): Make sure the data is in correct form since it
+        # might be consumed external to Keystone and this is a v2.0 controller.
+        # The token provider does not explicitly care about user_ref version
+        # in this case, but the data is stored in the token itself and should
+        # match the version
+        user_ref = self.v3_to_v2_user(user_ref)
+        auth_token_data = dict(user=user_ref,
+                               tenant=tenant_ref,
+                               metadata=metadata_ref,
+                               id='placeholder')
+        (token_id, token_data) = self.token_provider_api.issue_v2_token(
+            auth_token_data, roles_ref, catalog_ref)
+        return token_data
+
+    @controller.v2_deprecated
+    def get_credential(self, context, user_id, credential_id):
+        if not self._is_admin(context):
+            self._assert_identity(context, user_id)
+        return super(Ec2Controller, self).get_credential(user_id,
+                                                         credential_id)
+
+    @controller.v2_deprecated
+    def get_credentials(self, context, user_id):
+        if not self._is_admin(context):
+            self._assert_identity(context, user_id)
+        return super(Ec2Controller, self).get_credentials(user_id)
+
+    @controller.v2_deprecated
+    def create_credential(self, context, user_id, tenant_id):
+        if not self._is_admin(context):
+            self._assert_identity(context, user_id)
+        return super(Ec2Controller, self).create_credential(context, user_id,
+                                                            tenant_id)
+
+    @controller.v2_deprecated
+    def delete_credential(self, context, user_id, credential_id):
+        if not self._is_admin(context):
+            self._assert_identity(context, user_id)
+            self._assert_owner(user_id, credential_id)
+        return super(Ec2Controller, self).delete_credential(user_id,
+                                                            credential_id)
+
     def _assert_identity(self, context, user_id):
         """Check that the provided token belongs to the user.
 
@@ -252,13 +304,6 @@ class Ec2Controller(controller.V2Controller):
         if token_ref['user'].get('id') != user_id:
             raise exception.Forbidden(_('Token belongs to another user'))
 
-    def _context_trust_id(self, context):
-        try:
-            token_ref = self.token_api.get_token(context['token_id'])
-        except exception.TokenNotFound as e:
-            raise exception.Unauthorized(e)
-        return token_ref.get('trust_id')
-
     def _is_admin(self, context):
         """Wrap admin assertion error return statement.
 
@@ -267,6 +312,8 @@ class Ec2Controller(controller.V2Controller):
 
         """
         try:
+            # NOTE(morganfainberg): policy_api is required for assert_admin
+            # to properly perform policy enforcement.
             self.assert_admin(context)
             return True
         except exception.Forbidden:
@@ -285,26 +332,71 @@ class Ec2Controller(controller.V2Controller):
         if user_id != cred_ref['user_id']:
             raise exception.Forbidden(_('Credential belongs to another user'))
 
-    def _assert_valid_user_id(self, user_id):
-        """Ensure a valid user id.
 
-        :param context: standard context
-        :param user_id: expected credential owner
-        :raises exception.UserNotFound: on failure
+@dependency.requires('policy_api', 'token_provider_api')
+class Ec2ControllerV3(Ec2ControllerCommon, controller.V3Controller):
 
-        """
-        user_ref = self.identity_api.get_user(user_id)
-        if not user_ref:
-            raise exception.UserNotFound(user_id=user_id)
+    member_name = 'project'
 
-    def _assert_valid_project_id(self, project_id):
-        """Ensure a valid project id.
+    def __init__(self):
+        super(Ec2ControllerV3, self).__init__()
+        self.get_member_from_driver = self.credential_api.get_credential
 
-        :param context: standard context
-        :param project_id: expected project
-        :raises exception.ProjectNotFound: on failure
+    def _check_credential_owner_and_user_id_match(self, context, prep_info,
+                                                  user_id, credential_id):
+        # NOTE(morganfainberg): this method needs to capture the arguments of
+        # the method that is decorated with @controller.protected() (with
+        # exception of the first argument ('context') since the protected
+        # method passes in *args, **kwargs. In this case, it is easier to see
+        # the expected input if the argspec is `user_id` and `credential_id`
+        # explicitly (matching the :class:`.ec2_delete_credential()` method
+        # below).
+        ref = {}
+        credential_id = utils.hash_access_key(credential_id)
+        ref['credential'] = self.credential_api.get_credential(credential_id)
+        # NOTE(morganfainberg): policy_api is required for this
+        # check_protection to properly be able to perform policy enforcement.
+        self.check_protection(context, prep_info, ref)
 
-        """
-        project_ref = self.identity_api.get_project(project_id)
-        if not project_ref:
-            raise exception.ProjectNotFound(project_id=project_id)
+    def authenticate(self, context, credentials=None, ec2Credentials=None):
+        (user_ref, project_ref, metadata_ref, roles_ref,
+         catalog_ref) = self._authenticate(credentials=credentials,
+                                           ec2credentials=ec2Credentials)
+
+        method_names = ['ec2credential']
+
+        token_id, token_data = self.token_provider_api.issue_v3_token(
+            user_ref['id'], method_names, project_id=project_ref['id'],
+            metadata_ref=metadata_ref)
+        return render_token_data_response(token_id, token_data)
+
+    @controller.protected()
+    def ec2_get_credential(self, context, user_id, credential_id):
+        return super(Ec2ControllerV3, self).get_credential(user_id,
+                                                           credential_id)
+
+    @controller.protected()
+    def ec2_list_credentials(self, context, user_id):
+        return super(Ec2ControllerV3, self).get_credentials(user_id)
+
+    @controller.protected()
+    def ec2_create_credential(self, context, user_id, tenant_id):
+        return super(Ec2ControllerV3, self).create_credential(context, user_id,
+                                                              tenant_id)
+
+    @controller.protected(callback=_check_credential_owner_and_user_id_match)
+    def ec2_delete_credential(self, context, user_id, credential_id):
+        return super(Ec2ControllerV3, self).delete_credential(user_id,
+                                                              credential_id)
+
+
+def render_token_data_response(token_id, token_data):
+    """Render token data HTTP response.
+
+    Stash token ID into the X-Subject-Token header.
+
+    """
+    headers = [('X-Subject-Token', token_id)]
+
+    return wsgi.render_response(body=token_data,
+                                status=(200, 'OK'), headers=headers)
