@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,25 +12,28 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import abc
+
+import six
 
 from keystone.common import dependency
 from keystone.common import extension
 from keystone.common import manager
 from keystone import config
 from keystone import exception
-from keystone.openstack.common import log as logging
+from keystone.openstack.common import log
 
 
 CONF = config.CONF
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 extension_data = {
-    'name': 'Openstack Keystone Endpoint Filter API',
+    'name': 'OpenStack Keystone Endpoint Filter API',
     'namespace': 'http://docs.openstack.org/identity/api/ext/'
                  'OS-EP-FILTER/v1.0',
     'alias': 'OS-EP-FILTER',
     'updated': '2013-07-23T12:00:0-00:00',
-    'description': 'Openstack Keystone Endpoint Filter API.',
+    'description': 'OpenStack Keystone Endpoint Filter API.',
     'links': [
         {
             'rel': 'describedby',
@@ -59,9 +60,11 @@ class Manager(manager.Manager):
         super(Manager, self).__init__(CONF.endpoint_filter.driver)
 
 
+@six.add_metaclass(abc.ABCMeta)
 class Driver(object):
     """Interface description for an Endpoint Filter driver."""
 
+    @abc.abstractmethod
     def add_endpoint_to_project(self, endpoint_id, project_id):
         """Creates an endpoint to project association.
 
@@ -75,6 +78,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def remove_endpoint_from_project(self, endpoint_id, project_id):
         """Removes an endpoint to project association.
 
@@ -88,6 +92,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def check_endpoint_in_project(self, endpoint_id, project_id):
         """Checks if an endpoint is associated with a project.
 
@@ -101,6 +106,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def list_endpoints_for_project(self, project_id):
         """List all endpoints associated with a project.
 
@@ -111,6 +117,7 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
+    @abc.abstractmethod
     def list_projects_for_endpoint(self, endpoint_id):
         """List all projects associated with an endpoint.
 
