@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -18,10 +16,15 @@ import datetime
 import random
 import uuid
 
+import six
+
 from keystone.common import sql
-from keystone.common.sql import migration
+from keystone.common.sql import migration_helpers
+from keystone.contrib import oauth1
 from keystone.contrib.oauth1 import core
 from keystone import exception
+from keystone.openstack.common.db.sqlalchemy import migration
+from keystone.openstack.common.gettextutils import _
 from keystone.openstack.common import jsonutils
 from keystone.openstack.common import timeutils
 
@@ -55,7 +58,7 @@ class RequestToken(sql.ModelBase, sql.DictBase):
         return cls(**user_dict)
 
     def to_dict(self):
-        return dict(self.iteritems())
+        return dict(six.iteritems(self))
 
 
 class AccessToken(sql.ModelBase, sql.DictBase):
@@ -78,12 +81,13 @@ class AccessToken(sql.ModelBase, sql.DictBase):
         return cls(**user_dict)
 
     def to_dict(self):
-        return dict(self.iteritems())
+        return dict(six.iteritems(self))
 
 
-class OAuth1(sql.Base):
+class OAuth1(object):
     def db_sync(self):
-        migration.db_sync()
+        migration.db_sync(sql.get_engine(),
+                          migration_helpers.find_migrate_repo(oauth1))
 
     def _get_consumer(self, session, consumer_id):
         consumer_ref = session.query(Consumer).get(consumer_id)
@@ -92,7 +96,7 @@ class OAuth1(sql.Base):
         return consumer_ref
 
     def get_consumer_with_secret(self, consumer_id):
-        session = self.get_session()
+        session = sql.get_session()
         consumer_ref = self._get_consumer(session, consumer_id)
         return consumer_ref.to_dict()
 
@@ -104,11 +108,10 @@ class OAuth1(sql.Base):
         consumer['secret'] = uuid.uuid4().hex
         if not consumer.get('description'):
             consumer['description'] = None
-        session = self.get_session()
+        session = sql.get_session()
         with session.begin():
             consumer_ref = Consumer.from_dict(consumer)
             session.add(consumer_ref)
-            session.flush()
         return consumer_ref.to_dict()
 
     def _delete_consumer(self, session, consumer_id):
@@ -141,20 +144,19 @@ class OAuth1(sql.Base):
             session.delete(token_ref)
 
     def delete_consumer(self, consumer_id):
-        session = self.get_session()
+        session = sql.get_session()
         with session.begin():
             self._delete_request_tokens(session, consumer_id)
             self._delete_access_tokens(session, consumer_id)
             self._delete_consumer(session, consumer_id)
-            session.flush()
 
     def list_consumers(self):
-        session = self.get_session()
+        session = sql.get_session()
         cons = session.query(Consumer)
         return [core.filter_consumer(x.to_dict()) for x in cons]
 
     def update_consumer(self, consumer_id, consumer):
-        session = self.get_session()
+        session = sql.get_session()
         with session.begin():
             consumer_ref = self._get_consumer(session, consumer_id)
             old_consumer_dict = consumer_ref.to_dict()
@@ -162,10 +164,14 @@ class OAuth1(sql.Base):
             new_consumer = Consumer.from_dict(old_consumer_dict)
             consumer_ref.description = new_consumer.description
             consumer_ref.extra = new_consumer.extra
-            session.flush()
         return core.filter_consumer(consumer_ref.to_dict())
 
-    def create_request_token(self, consumer_id, project_id, token_duration):
+    def create_request_token(self, consumer_id, project_id, token_duration,
+                             request_token_id=None, request_token_secret=None):
+        if request_token_id is None:
+            request_token_id = uuid.uuid4().hex
+        if request_token_secret is None:
+            request_token_secret = uuid.uuid4().hex
         expiry_date = None
         if token_duration:
             now = timeutils.utcnow()
@@ -173,20 +179,18 @@ class OAuth1(sql.Base):
             expiry_date = timeutils.isotime(future, subsecond=True)
 
         ref = {}
-        request_token_id = uuid.uuid4().hex
         ref['id'] = request_token_id
-        ref['request_secret'] = uuid.uuid4().hex
+        ref['request_secret'] = request_token_secret
         ref['verifier'] = None
         ref['authorizing_user_id'] = None
         ref['requested_project_id'] = project_id
         ref['role_ids'] = None
         ref['consumer_id'] = consumer_id
         ref['expires_at'] = expiry_date
-        session = self.get_session()
+        session = sql.get_session()
         with session.begin():
             token_ref = RequestToken.from_dict(ref)
             session.add(token_ref)
-            session.flush()
         return token_ref.to_dict()
 
     def _get_request_token(self, session, request_token_id):
@@ -196,13 +200,13 @@ class OAuth1(sql.Base):
         return token_ref
 
     def get_request_token(self, request_token_id):
-        session = self.get_session()
+        session = sql.get_session()
         token_ref = self._get_request_token(session, request_token_id)
         return token_ref.to_dict()
 
     def authorize_request_token(self, request_token_id, user_id,
                                 role_ids):
-        session = self.get_session()
+        session = sql.get_session()
         with session.begin():
             token_ref = self._get_request_token(session, request_token_id)
             token_dict = token_ref.to_dict()
@@ -216,11 +220,15 @@ class OAuth1(sql.Base):
                         or attr == 'role_ids'):
                     setattr(token_ref, attr, getattr(new_token, attr))
 
-            session.flush()
         return token_ref.to_dict()
 
-    def create_access_token(self, request_token_id, token_duration):
-        session = self.get_session()
+    def create_access_token(self, request_token_id, token_duration,
+                            access_token_id=None, access_token_secret=None):
+        if access_token_id is None:
+            access_token_id = uuid.uuid4().hex
+        if access_token_secret is None:
+            access_token_secret = uuid.uuid4().hex
+        session = sql.get_session()
         with session.begin():
             req_token_ref = self._get_request_token(session, request_token_id)
             token_dict = req_token_ref.to_dict()
@@ -233,9 +241,8 @@ class OAuth1(sql.Base):
 
             # add Access Token
             ref = {}
-            access_token_id = uuid.uuid4().hex
             ref['id'] = access_token_id
-            ref['access_secret'] = uuid.uuid4().hex
+            ref['access_secret'] = access_token_secret
             ref['authorizing_user_id'] = token_dict['authorizing_user_id']
             ref['project_id'] = token_dict['requested_project_id']
             ref['role_ids'] = token_dict['role_ids']
@@ -250,7 +257,6 @@ class OAuth1(sql.Base):
             q.delete(False)
             session.delete(req_token_ref)
 
-            session.flush()
         return token_ref.to_dict()
 
     def _get_access_token(self, session, access_token_id):
@@ -260,18 +266,18 @@ class OAuth1(sql.Base):
         return token_ref
 
     def get_access_token(self, access_token_id):
-        session = self.get_session()
+        session = sql.get_session()
         token_ref = self._get_access_token(session, access_token_id)
         return token_ref.to_dict()
 
     def list_access_tokens(self, user_id):
-        session = self.get_session()
+        session = sql.get_session()
         q = session.query(AccessToken)
         user_auths = q.filter_by(authorizing_user_id=user_id)
         return [core.filter_token(x.to_dict()) for x in user_auths]
 
     def delete_access_token(self, user_id, access_token_id):
-        session = self.get_session()
+        session = sql.get_session()
         with session.begin():
             token_ref = self._get_access_token(session, access_token_id)
             token_dict = token_ref.to_dict()
@@ -283,4 +289,3 @@ class OAuth1(sql.Base):
             q.delete(False)
 
             session.delete(token_ref)
-            session.flush()
