@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -21,7 +19,9 @@ from keystone.common import dependency
 from keystone.common import wsgi
 from keystone import config
 from keystone.contrib.oauth1 import core as oauth1
+from keystone.contrib.oauth1 import validator
 from keystone import exception
+from keystone.openstack.common.gettextutils import _
 from keystone.openstack.common import jsonutils
 from keystone.openstack.common import timeutils
 
@@ -34,6 +34,15 @@ class ConsumerCrudV3(controller.V3Controller):
     collection_name = 'consumers'
     member_name = 'consumer'
 
+    @classmethod
+    def base_url(cls, context, path=None):
+        """Construct a path and pass it to V3Controller.base_url method."""
+
+        # NOTE(stevemar): Overriding path to /OS-OAUTH1/consumers so that
+        # V3Controller.base_url handles setting the self link correctly.
+        path = '/OS-OAUTH1/' + cls.collection_name
+        return controller.V3Controller.base_url(context, path=path)
+
     @controller.protected()
     def create_consumer(self, context, consumer):
         ref = self._assign_unique_id(self._normalize_dict(consumer))
@@ -44,8 +53,8 @@ class ConsumerCrudV3(controller.V3Controller):
     def update_consumer(self, context, consumer_id, consumer):
         self._require_matching_id(consumer_id, consumer)
         ref = self._normalize_dict(consumer)
-        self._validate_consumer_ref(consumer)
-        ref = self.oauth_api.update_consumer(consumer_id, consumer)
+        self._validate_consumer_ref(ref)
+        ref = self.oauth_api.update_consumer(consumer_id, ref)
         return ConsumerCrudV3.wrap_member(context, ref)
 
     @controller.protected()
@@ -71,7 +80,7 @@ class ConsumerCrudV3(controller.V3Controller):
             raise exception.ValidationError(message=msg)
 
 
-@dependency.requires('oauth_api')
+@dependency.requires('oauth_api', 'token_api')
 class AccessTokenCrudV3(controller.V3Controller):
     collection_name = 'access_tokens'
     member_name = 'access_token'
@@ -81,13 +90,14 @@ class AccessTokenCrudV3(controller.V3Controller):
         access_token = self.oauth_api.get_access_token(access_token_id)
         if access_token['authorizing_user_id'] != user_id:
             raise exception.NotFound()
-        access_token = self._format_token_entity(access_token)
+        access_token = self._format_token_entity(context, access_token)
         return AccessTokenCrudV3.wrap_member(context, access_token)
 
     @controller.protected()
     def list_access_tokens(self, context, user_id):
         refs = self.oauth_api.list_access_tokens(user_id)
-        formatted_refs = ([self._format_token_entity(x) for x in refs])
+        formatted_refs = ([self._format_token_entity(context, x)
+                           for x in refs])
         return AccessTokenCrudV3.wrap_collection(context, formatted_refs)
 
     @controller.protected()
@@ -98,7 +108,7 @@ class AccessTokenCrudV3(controller.V3Controller):
         return self.oauth_api.delete_access_token(
             user_id, access_token_id)
 
-    def _format_token_entity(self, entity):
+    def _format_token_entity(self, context, entity):
 
         formatted_entity = entity.copy()
         access_token_id = formatted_entity['id']
@@ -115,12 +125,12 @@ class AccessTokenCrudV3(controller.V3Controller):
                            'access_token_id': access_token_id})
 
         formatted_entity.setdefault('links', {})
-        formatted_entity['links']['roles'] = (self.base_url(url))
+        formatted_entity['links']['roles'] = (self.base_url(context, url))
 
         return formatted_entity
 
 
-@dependency.requires('oauth_api', 'assignment_api')
+@dependency.requires('assignment_api', 'oauth_api')
 class AccessTokenRolesV3(controller.V3Controller):
     collection_name = 'roles'
     member_name = 'role'
@@ -159,8 +169,7 @@ class AccessTokenRolesV3(controller.V3Controller):
         return formatted_entity
 
 
-@dependency.requires('oauth_api', 'token_api', 'identity_api',
-                     'token_provider_api', 'assignment_api')
+@dependency.requires('assignment_api', 'oauth_api', 'token_api')
 class OAuthControllerV3(controller.V3Controller):
     collection_name = 'not_used'
     member_name = 'not_used'
@@ -177,26 +186,21 @@ class OAuthControllerV3(controller.V3Controller):
             raise exception.ValidationError(
                 attribute='requested_project_id', target='request')
 
-        consumer_ref = self.oauth_api.get_consumer_with_secret(consumer_id)
-        consumer = oauth1.Consumer(key=consumer_ref['id'],
-                                   secret=consumer_ref['secret'])
+        url = self.base_url(context, context['path'])
 
-        url = oauth1.rebuild_url(context['path'])
-        oauth_request = oauth1.Request.from_request(
+        req_headers = {'Requested-Project-Id': requested_project_id}
+        req_headers.update(headers)
+        request_verifier = oauth1.RequestTokenEndpoint(
+            request_validator=validator.OAuthValidator(),
+            token_generator=oauth1.token_generator)
+        h, b, s = request_verifier.create_request_token_response(
+            url,
             http_method='POST',
-            http_url=url,
-            headers=context['headers'],
-            query_string=context['query_string'],
-            parameters={'requested_project_id': requested_project_id})
-        oauth_server = oauth1.Server()
-        oauth_server.add_signature_method(oauth1.SignatureMethod_HMAC_SHA1())
-        params = oauth_server.verify_request(oauth_request,
-                                             consumer,
-                                             token=None)
+            body=context['query_string'],
+            headers=req_headers)
 
-        project_params = params['requested_project_id']
-        if project_params != requested_project_id:
-            msg = _('Non-oauth parameter - project, do not match')
+        if (not b) or int(s) > 399:
+            msg = _('Invalid signature')
             raise exception.Unauthorized(message=msg)
 
         request_token_duration = CONF.oauth1.request_token_duration
@@ -236,7 +240,6 @@ class OAuthControllerV3(controller.V3Controller):
             raise exception.ValidationError(
                 attribute='oauth_verifier', target='request')
 
-        consumer = self.oauth_api.get_consumer_with_secret(consumer_id)
         req_token = self.oauth_api.get_request_token(
             request_token_id)
 
@@ -248,24 +251,17 @@ class OAuthControllerV3(controller.V3Controller):
             if now > expires:
                 raise exception.Unauthorized(_('Request token is expired'))
 
-        consumer_obj = oauth1.Consumer(key=consumer['id'],
-                                       secret=consumer['secret'])
-        req_token_obj = oauth1.Token(key=req_token['id'],
-                                     secret=req_token['request_secret'])
-        req_token_obj.set_verifier(oauth_verifier)
+        url = self.base_url(context, context['path'])
 
-        url = oauth1.rebuild_url(context['path'])
-        oauth_request = oauth1.Request.from_request(
+        access_verifier = oauth1.AccessTokenEndpoint(
+            request_validator=validator.OAuthValidator(),
+            token_generator=oauth1.token_generator)
+        h, b, s = access_verifier.create_access_token_response(
+            url,
             http_method='POST',
-            http_url=url,
-            headers=context['headers'],
-            query_string=context['query_string'])
-        oauth_server = oauth1.Server()
-        oauth_server.add_signature_method(oauth1.SignatureMethod_HMAC_SHA1())
-        params = oauth_server.verify_request(oauth_request,
-                                             consumer_obj,
-                                             token=req_token_obj)
-
+            body=context['query_string'],
+            headers=headers)
+        params = oauth1.extract_non_oauth_params(b)
         if len(params) != 0:
             msg = _('There should not be any non-oauth parameters')
             raise exception.Unauthorized(message=msg)
