@@ -10,28 +10,25 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-"""Main entry point into the Revoke service."""
-
 import abc
 import datetime
 
-from oslo_config import cfg
-from oslo_log import log
-from oslo_log import versionutils
-from oslo_utils import timeutils
 import six
 
 from keystone.common import cache
 from keystone.common import dependency
 from keystone.common import extension
 from keystone.common import manager
+from keystone import config
 from keystone.contrib.revoke import model
 from keystone import exception
-from keystone.i18n import _
 from keystone import notifications
+from keystone.openstack.common.gettextutils import _
+from keystone.openstack.common import log
+from keystone.openstack.common import timeutils
 
 
-CONF = cfg.CONF
+CONF = config.CONF
 LOG = log.getLogger(__name__)
 
 
@@ -54,7 +51,9 @@ EXTENSION_DATA = {
 extension.register_admin_extension(EXTENSION_DATA['alias'], EXTENSION_DATA)
 extension.register_public_extension(EXTENSION_DATA['alias'], EXTENSION_DATA)
 
-MEMOIZE = cache.get_memoization_decorator(section='revoke')
+SHOULD_CACHE = cache.should_cache_fn('revoke')
+# TODO(ayoung): migrate from the token section
+REVOCATION_CACHE_EXPIRATION_TIME = lambda: CONF.token.revocation_cache_time
 
 
 def revoked_before_cutoff_time():
@@ -66,17 +65,12 @@ def revoked_before_cutoff_time():
 
 @dependency.provider('revoke_api')
 class Manager(manager.Manager):
-    """Default pivot point for the Revoke backend.
+    """Revoke API Manager.
 
     Performs common logic for recording revocations.
 
-    See :mod:`keystone.common.manager.Manager` for more details on
-    how this dynamically calls the backend.
-
     """
 
-    driver_namespace = 'keystone.revoke'
-
     def __init__(self):
         super(Manager, self).__init__(CONF.revoke.driver)
         self._register_listeners()
@@ -116,77 +110,28 @@ class Manager(manager.Manager):
         self.revoke(
             model.RevokeEvent(access_token_id=payload['resource_info']))
 
-    def _role_assignment_callback(self, service, resource_type, operation,
-                                  payload):
-        info = payload['resource_info']
-        self.revoke_by_grant(role_id=info['role_id'], user_id=info['user_id'],
-                             domain_id=info.get('domain_id'),
-                             project_id=info.get('project_id'))
-
     def _register_listeners(self):
-        callbacks = {
-            notifications.ACTIONS.deleted: [
-                ['OS-TRUST:trust', self._trust_callback],
-                ['OS-OAUTH1:consumer', self._consumer_callback],
-                ['OS-OAUTH1:access_token', self._access_token_callback],
-                ['role', self._role_callback],
-                ['user', self._user_callback],
-                ['project', self._project_callback],
-                ['role_assignment', self._role_assignment_callback]
-            ],
-            notifications.ACTIONS.disabled: [
-                ['user', self._user_callback],
-                ['project', self._project_callback],
-                ['domain', self._domain_callback],
-            ],
-            notifications.ACTIONS.internal: [
-                [notifications.INVALIDATE_USER_TOKEN_PERSISTENCE,
-                 self._user_callback],
-            ]
-        }
-
-        for event, cb_info in callbacks.items():
-            for resource_type, callback_fns in cb_info:
-                notifications.register_event_callback(event, resource_type,
-                                                      callback_fns)
+        callbacks = [
+            ['deleted', 'OS-TRUST:trust', self._trust_callback],
+            ['deleted', 'OS-OAUTH1:consumer', self._consumer_callback],
+            ['deleted', 'OS-OAUTH1:access_token',
+             self._access_token_callback],
+            ['deleted', 'role', self._role_callback],
+            ['deleted', 'user', self._user_callback],
+            ['disabled', 'user', self._user_callback],
+            ['deleted', 'project', self._project_callback],
+            ['disabled', 'project', self._project_callback],
+            ['disabled', 'domain', self._domain_callback]]
+        for cb in callbacks:
+            notifications.register_event_callback(*cb)
 
     def revoke_by_user(self, user_id):
         return self.revoke(model.RevokeEvent(user_id=user_id))
 
-    def _assert_not_domain_and_project_scoped(self, domain_id=None,
-                                              project_id=None):
-        if domain_id is not None and project_id is not None:
-            msg = _('The revoke call must not have both domain_id and '
-                    'project_id. This is a bug in the Keystone server. The '
-                    'current request is aborted.')
-            raise exception.UnexpectedError(exception=msg)
-
-    @versionutils.deprecated(as_of=versionutils.deprecated.JUNO,
-                             remove_in=0)
-    def revoke_by_expiration(self, user_id, expires_at,
-                             domain_id=None, project_id=None):
-
-        self._assert_not_domain_and_project_scoped(domain_id=domain_id,
-                                                   project_id=project_id)
-
+    def revoke_by_expiration(self, user_id, expires_at):
         self.revoke(
             model.RevokeEvent(user_id=user_id,
-                              expires_at=expires_at,
-                              domain_id=domain_id,
-                              project_id=project_id))
-
-    def revoke_by_audit_id(self, audit_id):
-        self.revoke(model.RevokeEvent(audit_id=audit_id))
-
-    def revoke_by_audit_chain_id(self, audit_chain_id, project_id=None,
-                                 domain_id=None):
-
-        self._assert_not_domain_and_project_scoped(domain_id=domain_id,
-                                                   project_id=project_id)
-
-        self.revoke(model.RevokeEvent(audit_chain_id=audit_chain_id,
-                                      domain_id=domain_id,
-                                      project_id=project_id))
+                              expires_at=expires_at))
 
     def revoke_by_grant(self, role_id, user_id=None,
                         domain_id=None, project_id=None):
@@ -206,9 +151,10 @@ class Manager(manager.Manager):
     def revoke_by_domain_role_assignment(self, domain_id, role_id):
         self.revoke(model.RevokeEvent(domain_id=domain_id, role_id=role_id))
 
-    @MEMOIZE
+    @cache.on_arguments(should_cache_fn=SHOULD_CACHE,
+                        expiration_time=REVOCATION_CACHE_EXPIRATION_TIME)
     def _get_revoke_tree(self):
-        events = self.driver.list_events()
+        events = self.driver.get_events()
         revoke_tree = model.RevokeTree(revoke_events=events)
 
         return revoke_tree
@@ -232,11 +178,11 @@ class Manager(manager.Manager):
 
 
 @six.add_metaclass(abc.ABCMeta)
-class RevokeDriverV8(object):
+class Driver(object):
     """Interface for recording and reporting revocation events."""
 
     @abc.abstractmethod
-    def list_events(self, last_fetch=None):
+    def get_events(self, last_fetch=None):
         """return the revocation events, as a list of objects
 
         :param last_fetch:   Time of last fetch.  Return all events newer.
@@ -246,7 +192,7 @@ class RevokeDriverV8(object):
                   for tokens issued after the expiration cutoff.
 
         """
-        raise exception.NotImplemented()  # pragma: no cover
+        raise exception.NotImplemented()
 
     @abc.abstractmethod
     def revoke(self, event):
@@ -256,7 +202,4 @@ class RevokeDriverV8(object):
             keystone.contrib.revoke.model.RevocationEvent
 
         """
-        raise exception.NotImplemented()  # pragma: no cover
-
-
-Driver = manager.create_legacy_driver(RevokeDriverV8)
+        raise exception.NotImplemented()
