@@ -10,19 +10,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from oslo_utils import timeutils
-from six.moves import map
-
-from keystone.common import utils
-
+from keystone.openstack.common import timeutils
 
 # The set of attributes common between the RevokeEvent
 # and the dictionaries created from the token Data.
 _NAMES = ['trust_id',
           'consumer_id',
           'access_token_id',
-          'audit_id',
-          'audit_chain_id',
           'expires_at',
           'domain_id',
           'project_id',
@@ -33,10 +27,6 @@ _NAMES = ['trust_id',
 # Additional arguments for creating a RevokeEvent
 _EVENT_ARGS = ['issued_before', 'revoked_at']
 
-# Names of attributes in the RevocationEvent, including "virtual" attributes.
-# Virtual attributes are those added based on other values.
-_EVENT_NAMES = _NAMES + ['domain_scope_id']
-
 # Values that will be in the token data but not in the event.
 # These will compared with event values that have different names.
 # For example: both trustor_id and trustee_id are compared against user_id
@@ -46,15 +36,6 @@ _TOKEN_KEYS = ['identity_domain_id',
                'trustor_id',
                'trustee_id']
 
-# Alternative names to be checked in token for every field in
-# revoke tree.
-ALTERNATIVES = {
-    'user_id': ['user_id', 'trustor_id', 'trustee_id'],
-    'domain_id': ['identity_domain_id', 'assignment_domain_id'],
-    # For a domain-scoped token, the domain is in assignment_domain_id.
-    'domain_scope_id': ['assignment_domain_id', ],
-}
-
 
 REVOKE_KEYS = _NAMES + _EVENT_ARGS
 
@@ -75,20 +56,6 @@ class RevokeEvent(object):
         for k in REVOKE_KEYS:
             v = kwargs.get(k, None)
             setattr(self, k, v)
-
-        if self.domain_id and self.expires_at:
-            # This is revoking a domain-scoped token.
-            self.domain_scope_id = self.domain_id
-            self.domain_id = None
-        else:
-            # This is revoking all tokens for a domain.
-            self.domain_scope_id = None
-
-        if self.expires_at is not None:
-            # Trim off the expiration time because MySQL timestamps are only
-            # accurate to the second.
-            self.expires_at = self.expires_at.replace(microsecond=0)
-
         if self.revoked_at is None:
             self.revoked_at = timeutils.utcnow()
         if self.issued_before is None:
@@ -98,13 +65,9 @@ class RevokeEvent(object):
         keys = ['user_id',
                 'role_id',
                 'domain_id',
-                'domain_scope_id',
-                'project_id',
-                'audit_id',
-                'audit_chain_id',
-                ]
-        event = {key: self.__dict__[key] for key in keys
-                 if self.__dict__[key] is not None}
+                'project_id']
+        event = dict((key, self.__dict__[key]) for key in keys
+                     if self.__dict__[key] is not None)
         if self.trust_id is not None:
             event['OS-TRUST:trust_id'] = self.trust_id
         if self.consumer_id is not None:
@@ -112,10 +75,11 @@ class RevokeEvent(object):
         if self.consumer_id is not None:
             event['OS-OAUTH1:access_token_id'] = self.access_token_id
         if self.expires_at is not None:
-            event['expires_at'] = utils.isotime(self.expires_at)
+            event['expires_at'] = timeutils.isotime(self.expires_at,
+                                                    subsecond=True)
         if self.issued_before is not None:
-            event['issued_before'] = utils.isotime(self.issued_before,
-                                                   subsecond=True)
+            event['issued_before'] = timeutils.isotime(self.issued_before,
+                                                       subsecond=True)
         return event
 
     def key_for_name(self, name):
@@ -123,7 +87,7 @@ class RevokeEvent(object):
 
 
 def attr_keys(event):
-    return list(map(event.key_for_name, _EVENT_NAMES))
+    return map(event.key_for_name, _NAMES)
 
 
 class RevokeTree(object):
@@ -173,7 +137,7 @@ class RevokeTree(object):
         """
         stack = []
         revoke_map = self.revoke_map
-        for name in _EVENT_NAMES:
+        for name in _NAMES:
             key = event.key_for_name(name)
             nxt = revoke_map.get(key)
             if nxt is None:
@@ -188,52 +152,7 @@ class RevokeTree(object):
                 del parent[key]
 
     def add_events(self, revoke_events):
-        return list(map(self.add_event, revoke_events or []))
-
-    @staticmethod
-    def _next_level_keys(name, token_data):
-        """Generate keys based on current field name and token data
-
-        Generate all keys to look for in the next iteration of revocation
-        event tree traversal.
-        """
-        yield '*'
-        if name == 'role_id':
-            # Roles are very special since a token has a list of them.
-            # If the revocation event matches any one of them,
-            # revoke the token.
-            for role_id in token_data.get('roles', []):
-                yield role_id
-        else:
-            # For other fields we try to get any branch that concur
-            # with any alternative field in the token.
-            for alt_name in ALTERNATIVES.get(name, [name]):
-                yield token_data[alt_name]
-
-    def _search(self, revoke_map, names, token_data):
-        """Search for revocation event by token_data
-
-        Traverse the revocation events tree looking for event matching token
-        data issued after the token.
-        """
-        if not names:
-            # The last (leaf) level is checked in a special way because we
-            # verify issued_at field differently.
-            try:
-                return revoke_map['issued_before'] >= token_data['issued_at']
-            except KeyError:
-                return False
-
-        name, remaining_names = names[0], names[1:]
-
-        for key in self._next_level_keys(name, token_data):
-            subtree = revoke_map.get('%s=%s' % (name, key))
-            if subtree and self._search(subtree, remaining_names, token_data):
-                return True
-
-        # If we made it out of the loop then no element in revocation tree
-        # corresponds to our token and it is good.
-        return False
+        return map(self.add_event, revoke_events or [])
 
     def is_revoked(self, token_data):
         """Check if a token matches the revocation event
@@ -252,25 +171,65 @@ class RevokeTree(object):
            'consumer_id', 'access_token_id'
 
         """
-        return self._search(self.revoke_map, _EVENT_NAMES, token_data)
+        # Alternative names to be checked in token for every field in
+        # revoke tree.
+        alternatives = {
+            'user_id': ['user_id', 'trustor_id', 'trustee_id'],
+            'domain_id': ['identity_domain_id', 'assignment_domain_id'],
+        }
+        # Contains current forest (collection of trees) to be checked.
+        partial_matches = [self.revoke_map]
+        # We iterate over every layer of our revoke tree (except the last one).
+        for name in _NAMES:
+            # bundle is the set of partial matches for the next level down
+            # the tree
+            bundle = []
+            wildcard = '%s=*' % (name,)
+            # For every tree in current forest.
+            for tree in partial_matches:
+                # If there is wildcard node on current level we take it.
+                bundle.append(tree.get(wildcard))
+                if name == 'role_id':
+                    # Roles are very special since a token has a list of them.
+                    # If the revocation event matches any one of them,
+                    # revoke the token.
+                    for role_id in token_data.get('roles', []):
+                        bundle.append(tree.get('role_id=%s' % role_id))
+                else:
+                    # For other fields we try to get any branch that concur
+                    # with any alternative field in the token.
+                    for alt_name in alternatives.get(name, [name]):
+                        bundle.append(
+                            tree.get('%s=%s' % (name, token_data[alt_name])))
+            # tree.get returns `None` if there is no match, so `bundle.append`
+            # adds a 'None' entry. This call remoes the `None` entries.
+            partial_matches = [x for x in bundle if x is not None]
+            if not partial_matches:
+                # If we end up with no branches to follow means that the token
+                # is definitely not in the revoke tree and all further
+                # iterations will be for nothing.
+                return False
+
+        # The last (leaf) level is checked in a special way because we verify
+        # issued_at field differently.
+        for leaf in partial_matches:
+            try:
+                if leaf['issued_before'] > token_data['issued_at']:
+                    return True
+            except KeyError:
+                pass
+        # If we made it out of the loop then no element in revocation tree
+        # corresponds to our token and it is good.
+        return False
 
 
 def build_token_values_v2(access, default_domain_id):
     token_data = access['token']
-
-    token_expires_at = timeutils.parse_isotime(token_data['expires'])
-
-    # Trim off the microseconds because the revocation event only has
-    # expirations accurate to the second.
-    token_expires_at = token_expires_at.replace(microsecond=0)
-
     token_values = {
-        'expires_at': timeutils.normalize_time(token_expires_at),
+        'expires_at': timeutils.normalize_time(
+            timeutils.parse_isotime(token_data['expires'])),
         'issued_at': timeutils.normalize_time(
-            timeutils.parse_isotime(token_data['issued_at'])),
-        'audit_id': token_data.get('audit_ids', [None])[0],
-        'audit_chain_id': token_data.get('audit_ids', [None])[-1],
-    }
+            timeutils.parse_isotime(token_data['issued_at']))}
 
     token_values['user_id'] = access.get('user', {}).get('id')
 
@@ -306,27 +265,16 @@ def build_token_values_v2(access, default_domain_id):
 
 
 def build_token_values(token_data):
-
-    token_expires_at = timeutils.parse_isotime(token_data['expires_at'])
-
-    # Trim off the microseconds because the revocation event only has
-    # expirations accurate to the second.
-    token_expires_at = token_expires_at.replace(microsecond=0)
-
     token_values = {
-        'expires_at': timeutils.normalize_time(token_expires_at),
+        'expires_at': timeutils.normalize_time(
+            timeutils.parse_isotime(token_data['expires_at'])),
         'issued_at': timeutils.normalize_time(
-            timeutils.parse_isotime(token_data['issued_at'])),
-        'audit_id': token_data.get('audit_ids', [None])[0],
-        'audit_chain_id': token_data.get('audit_ids', [None])[-1],
-    }
+            timeutils.parse_isotime(token_data['issued_at']))}
 
     user = token_data.get('user')
     if user is not None:
         token_values['user_id'] = user['id']
-        # Federated users do not have a domain, be defensive and get the user
-        # domain set to None in the federated user case.
-        token_values['identity_domain_id'] = user.get('domain', {}).get('id')
+        token_values['identity_domain_id'] = user['domain']['id']
     else:
         token_values['user_id'] = None
         token_values['identity_domain_id'] = None
@@ -337,12 +285,7 @@ def build_token_values(token_data):
         token_values['assignment_domain_id'] = project['domain']['id']
     else:
         token_values['project_id'] = None
-
-        domain = token_data.get('domain')
-        if domain is not None:
-            token_values['assignment_domain_id'] = domain['id']
-        else:
-            token_values['assignment_domain_id'] = None
+        token_values['assignment_domain_id'] = None
 
     role_list = []
     roles = token_data.get('roles')
