@@ -10,4 +10,5 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystone.contrib.simple_cert.routers import SimpleCertExtension  # noqa
+from keystone.contrib.simple_cert.core import *  # flake8: noqa
+from keystone.contrib.simple_cert.routers import SimpleCertExtension
