@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,18 +15,19 @@
 import six
 
 from keystone.common import config
-from keystone.openstack.common import log as logging
+from keystone.openstack.common.gettextutils import _ # flake8: noqa
+from keystone.openstack.common import log
 from keystone.openstack.common import strutils
 
 
 CONF = config.CONF
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 # Tests use this to make exception message format errors fatal
 _FATAL_EXCEPTION_FORMAT_ERRORS = False
 
 
-class Error(StandardError):
+class Error(Exception):
     """Base error class.
 
     Child classes should define an HTTP status code, title, and a
@@ -85,6 +84,15 @@ class ValidationError(Error):
     title = 'Bad Request'
 
 
+class ValidationTimeStampError(Error):
+    message_format = _("Timestamp not in expected format."
+                       " The server could not comply with the request"
+                       " since it is either malformed or otherwise"
+                       " incorrect. The client is assumed to be in error.")
+    code = 400
+    title = 'Bad Request'
+
+
 class StringLengthExceeded(ValidationError):
     message_format = _("String length exceeded.The length of"
                        " string '%(string)s' exceeded the limit"
@@ -101,6 +109,14 @@ class ValidationSizeError(Error):
     title = 'Bad Request'
 
 
+class PKITokenExpected(Error):
+    message_format = _('The certificates you requested are not available. '
+                       'It is likely that this server does not use PKI tokens '
+                       'otherwise this is the result of misconfiguration.')
+    code = 403
+    title = 'Cannot retrieve certificates'
+
+
 class SecurityError(Error):
     """Avoids exposing details of security failures, unless in debug mode."""
 
@@ -126,6 +142,11 @@ class AuthPluginException(Unauthorized):
         self.authentication = {}
 
 
+class MissingGroups(Unauthorized):
+    message_format = _("Unable to find valid groups while using "
+                       "mapping %(mapping_id)s")
+
+
 class AuthMethodNotSupported(AuthPluginException):
     message_format = _("Attempted to authenticate with an unsupported method.")
 
@@ -154,6 +175,11 @@ class ForbiddenAction(Forbidden):
                        " requested action, %(action)s.")
 
 
+class ImmutableAttributeError(Forbidden):
+    message_format = _("Could not change immutable attribute %(attribute)s"
+                       " in target %(target)s")
+
+
 class NotFound(Error):
     message_format = _("Could not find, %(target)s.")
     code = 404
@@ -180,6 +206,10 @@ class RoleNotFound(NotFound):
     message_format = _("Could not find role, %(role_id)s.")
 
 
+class RegionNotFound(NotFound):
+    message_format = _("Could not find region, %(region_id)s.")
+
+
 class ServiceNotFound(NotFound):
     message_format = _("Could not find service, %(service_id)s.")
 
@@ -204,10 +234,18 @@ class GroupNotFound(NotFound):
     message_format = _("Could not find group, %(group_id)s.")
 
 
+class MappingNotFound(NotFound):
+    message_format = _("Could not find mapping, %(mapping_id)s.")
+
+
 class TrustNotFound(NotFound):
     message_format = _("Could not find trust, %(trust_id)s.")
 
 
+class TrustUseLimitReached(Forbidden):
+    message_format = _("No remaining uses for trust %(trust_id)s.")
+
+
 class CredentialNotFound(NotFound):
     message_format = _("Could not find credential, %(credential_id)s.")
 
@@ -216,6 +254,15 @@ class VersionNotFound(NotFound):
     message_format = _("Could not find version, %(version)s.")
 
 
+class IdentityProviderNotFound(NotFound):
+    message_format = _("Could not find IdentityProvider, %(idp_id)s.")
+
+
+class FederatedProtocolNotFound(NotFound):
+    message_format = _("Could not find federated protocol %(protocol_id)s for"
+                       " IdentityProvider, %(idp_id)s")
+
+
 class Conflict(Error):
     message_format = _("Conflict occurred attempting to store %(type)s."
                        " %(details)s")
@@ -229,16 +276,47 @@ class RequestTooLarge(Error):
     title = 'Request is too large.'
 
 
-class UnexpectedError(Error):
-    message_format = _("An unexpected error prevented the server"
-                       " from fulfilling your request. %(exception)s")
+class UnexpectedError(SecurityError):
+    """Avoids exposing details of failures, unless in debug mode."""
+    _message_format = _("An unexpected error prevented the server "
+                        "from fulfilling your request.")
+
+    debug_message_format = _("An unexpected error prevented the server "
+                             "from fulfilling your request. %(exception)s")
+
+    @property
+    def message_format(self):
+        """Return the generic message format string unless debug is enabled."""
+        if CONF.debug:
+            return self.debug_message_format
+        return self._message_format
+
+    def _build_message(self, message, **kwargs):
+        if CONF.debug and 'exception' not in kwargs:
+            # Ensure that exception has a value to be extra defensive for
+            # substitutions and make sure the exception doesn't raise an
+            # exception.
+            kwargs['exception'] = ''
+        return super(UnexpectedError, self)._build_message(message, **kwargs)
+
     code = 500
     title = 'Internal Server Error'
 
 
+class CertificateFilesUnavailable(UnexpectedError):
+    debug_message_format = _("Expected signing certificates are not available "
+                             "on the server. Please check Keystone "
+                             "configuration.")
+
+
 class MalformedEndpoint(UnexpectedError):
-    message_format = _("Malformed endpoint URL (%(endpoint)s),"
-                       " see ERROR log for details.")
+    debug_message_format = _("Malformed endpoint URL (%(endpoint)s),"
+                             " see ERROR log for details.")
+
+
+class MappedGroupNotFound(UnexpectedError):
+    debug_message_format = _("Group %(group_id)s returned by mapping "
+                             "%(mapping_id)s was not found in the backend.")
 
 
 class NotImplemented(Error):
@@ -248,6 +326,21 @@ class NotImplemented(Error):
     title = 'Not Implemented'
 
 
-class PasteConfigNotFound(UnexpectedError):
-    message_format = _("The Keystone paste configuration file"
-                       " %(config_file)s could not be found.")
+class Gone(Error):
+    message_format = _("The service you have requested is no"
+                       " longer available on this server.")
+    code = 410
+    title = 'Gone'
+
+
+class ConfigFileNotFound(UnexpectedError):
+    debug_message_format = _("The Keystone configuration file %(config_file)s "
+                             "could not be found.")
+
+
+class MigrationNotProvided(Exception):
+    def __init__(self, mod_name, path):
+        super(MigrationNotProvided, self).__init__(_(
+            "%(mod_name)s doesn't provide database migrations. The migration"
+            " repository path at %(path)s doesn't exist or isn't a directory."
+        ) % {'mod_name': mod_name, 'path': path})
