@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,8 +13,9 @@
 # under the License.
 
 from keystone.common import sql
-from keystone.common.sql import migration
+from keystone.common.sql import migration_helpers
 from keystone import exception
+from keystone.openstack.common.db.sqlalchemy import migration
 from keystone.policy.backends import rules
 
 
@@ -29,24 +28,25 @@ class PolicyModel(sql.ModelBase, sql.DictBase):
     extra = sql.Column(sql.JsonBlob())
 
 
-class Policy(sql.Base, rules.Policy):
+class Policy(rules.Policy):
     # Internal interface to manage the database
     def db_sync(self, version=None):
-        migration.db_sync(version=version)
+        migration.db_sync(
+            sql.get_engine(), migration_helpers.find_migrate_repo(),
+            version=version)
 
-    @sql.handle_conflicts(type='policy')
+    @sql.handle_conflicts(conflict_type='policy')
     def create_policy(self, policy_id, policy):
-        session = self.get_session()
+        session = sql.get_session()
 
         with session.begin():
             ref = PolicyModel.from_dict(policy)
             session.add(ref)
-            session.flush()
 
         return ref.to_dict()
 
     def list_policies(self):
-        session = self.get_session()
+        session = sql.get_session()
 
         refs = session.query(PolicyModel).all()
         return [ref.to_dict() for ref in refs]
@@ -59,13 +59,13 @@ class Policy(sql.Base, rules.Policy):
         return ref
 
     def get_policy(self, policy_id):
-        session = self.get_session()
+        session = sql.get_session()
 
         return self._get_policy(session, policy_id).to_dict()
 
-    @sql.handle_conflicts(type='policy')
+    @sql.handle_conflicts(conflict_type='policy')
     def update_policy(self, policy_id, policy):
-        session = self.get_session()
+        session = sql.get_session()
 
         with session.begin():
             ref = self._get_policy(session, policy_id)
@@ -75,14 +75,12 @@ class Policy(sql.Base, rules.Policy):
             ref.blob = new_policy.blob
             ref.type = new_policy.type
             ref.extra = new_policy.extra
-            session.flush()
 
         return ref.to_dict()
 
     def delete_policy(self, policy_id):
-        session = self.get_session()
+        session = sql.get_session()
 
         with session.begin():
             ref = self._get_policy(session, policy_id)
             session.delete(ref)
-            session.flush()
