@@ -16,10 +16,14 @@ import copy
 import datetime
 import uuid
 
+import mock
+
+from keystone import assignment
 from keystone import auth
+from keystone.common import authorization
+from keystone.common import environment
 from keystone import config
 from keystone import exception
-from keystone import identity
 from keystone.openstack.common import timeutils
 from keystone import tests
 from keystone.tests import default_fixtures
@@ -31,6 +35,8 @@ CONF = config.CONF
 TIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
 DEFAULT_DOMAIN_ID = CONF.identity.default_domain_id
 
+HOST_URL = 'http://keystone:5001'
+
 
 def _build_user_auth(token=None, user_id=None, username=None,
                      password=None, tenant_id=None, tenant_name=None,
@@ -64,15 +70,23 @@ class AuthTest(tests.TestCase):
     def setUp(self):
         super(AuthTest, self).setUp()
 
-        CONF.identity.driver = 'keystone.identity.backends.kvs.Identity'
         self.load_backends()
         self.load_fixtures(default_fixtures)
 
         # need to register the token provider first because auth controller
         # depends on it
         token.provider.Manager()
+        self.context_with_remote_user = {'environment':
+                                         {'REMOTE_USER': 'FOO',
+                                          'AUTH_TYPE': 'Negotiate'}}
+        self.empty_context = {'environment': {}}
 
         self.controller = token.controllers.Auth()
+        #This call sets up, among other things, the call to popen
+        #that will be used to run the CMS command.  These tests were
+        #passing only due to the global nature of the call.  If the
+        #tests in this file are run alone, API calls return unauthorized.
+        environment.use_eventlet(monkeypatch_thread=False)
 
     def assertEqualTokens(self, a, b):
         """Assert that two tokens are equal.
@@ -195,8 +209,7 @@ class AuthWithToken(AuthTest):
         body_dict = _build_user_auth(username='FOO',
                                      password='foo2')
         unscoped_token = self.controller.authenticate({}, body_dict)
-        tenant = unscoped_token["access"]["token"].get("tenant", None)
-        self.assertEqual(tenant, None)
+        self.assertNotIn('tenant', unscoped_token['access']['token'])
 
     def test_auth_invalid_token(self):
         """Verify exception is raised if invalid token."""
@@ -230,7 +243,7 @@ class AuthWithToken(AuthTest):
     def test_auth_unscoped_token_project(self):
         """Verify getting a token in a tenant with an unscoped token."""
         # Add a role in so we can check we get this back
-        self.identity_api.add_role_to_user_and_project(
+        self.assignment_api.add_role_to_user_and_project(
             self.user_foo['id'],
             self.tenant_bar['id'],
             self.role_member['id'])
@@ -247,25 +260,25 @@ class AuthWithToken(AuthTest):
 
         tenant = scoped_token["access"]["token"]["tenant"]
         roles = scoped_token["access"]["metadata"]["roles"]
-        self.assertEquals(tenant["id"], self.tenant_bar['id'])
-        self.assertEquals(roles[0], self.role_member['id'])
+        self.assertEqual(self.tenant_bar['id'], tenant["id"])
+        self.assertEqual(self.role_member['id'], roles[0])
 
     def test_auth_token_project_group_role(self):
         """Verify getting a token in a tenant with group roles."""
         # Add a v2 style role in so we can check we get this back
-        self.identity_api.add_role_to_user_and_project(
+        self.assignment_api.add_role_to_user_and_project(
             self.user_foo['id'],
             self.tenant_bar['id'],
             self.role_member['id'])
         # Now create a group role for this user as well
         domain1 = {'id': uuid.uuid4().hex, 'name': uuid.uuid4().hex}
-        self.identity_api.create_domain(domain1['id'], domain1)
+        self.assignment_api.create_domain(domain1['id'], domain1)
         new_group = {'id': uuid.uuid4().hex, 'domain_id': domain1['id'],
                      'name': uuid.uuid4().hex}
         self.identity_api.create_group(new_group['id'], new_group)
         self.identity_api.add_user_to_group(self.user_foo['id'],
                                             new_group['id'])
-        self.identity_api.create_grant(
+        self.assignment_api.create_grant(
             group_id=new_group['id'],
             project_id=self.tenant_bar['id'],
             role_id=self.role_admin['id'])
@@ -280,7 +293,7 @@ class AuthWithToken(AuthTest):
 
         tenant = scoped_token["access"]["token"]["tenant"]
         roles = scoped_token["access"]["metadata"]["roles"]
-        self.assertEquals(tenant["id"], self.tenant_bar['id'])
+        self.assertEqual(self.tenant_bar['id'], tenant["id"])
         self.assertIn(self.role_member['id'], roles)
         self.assertIn(self.role_admin['id'], roles)
 
@@ -288,38 +301,38 @@ class AuthWithToken(AuthTest):
         """Verify getting a token in cross domain group/project roles."""
         # create domain, project and group and grant roles to user
         domain1 = {'id': uuid.uuid4().hex, 'name': uuid.uuid4().hex}
-        self.identity_api.create_domain(domain1['id'], domain1)
+        self.assignment_api.create_domain(domain1['id'], domain1)
         project1 = {'id': uuid.uuid4().hex, 'name': uuid.uuid4().hex,
                     'domain_id': domain1['id']}
         self.assignment_api.create_project(project1['id'], project1)
         role_foo_domain1 = {'id': uuid.uuid4().hex,
                             'name': uuid.uuid4().hex}
-        self.identity_api.create_role(role_foo_domain1['id'],
-                                      role_foo_domain1)
+        self.assignment_api.create_role(role_foo_domain1['id'],
+                                        role_foo_domain1)
         role_group_domain1 = {'id': uuid.uuid4().hex,
                               'name': uuid.uuid4().hex}
-        self.identity_api.create_role(role_group_domain1['id'],
-                                      role_group_domain1)
-        self.identity_api.add_user_to_project(project1['id'],
-                                              self.user_foo['id'])
+        self.assignment_api.create_role(role_group_domain1['id'],
+                                        role_group_domain1)
+        self.assignment_api.add_user_to_project(project1['id'],
+                                                self.user_foo['id'])
         new_group = {'id': uuid.uuid4().hex, 'domain_id': domain1['id'],
                      'name': uuid.uuid4().hex}
         self.identity_api.create_group(new_group['id'], new_group)
         self.identity_api.add_user_to_group(self.user_foo['id'],
                                             new_group['id'])
-        self.identity_api.create_grant(
+        self.assignment_api.create_grant(
             user_id=self.user_foo['id'],
             project_id=project1['id'],
             role_id=self.role_member['id'])
-        self.identity_api.create_grant(
+        self.assignment_api.create_grant(
             group_id=new_group['id'],
             project_id=project1['id'],
             role_id=self.role_admin['id'])
-        self.identity_api.create_grant(
+        self.assignment_api.create_grant(
             user_id=self.user_foo['id'],
             domain_id=domain1['id'],
             role_id=role_foo_domain1['id'])
-        self.identity_api.create_grant(
+        self.assignment_api.create_grant(
             group_id=new_group['id'],
             domain_id=domain1['id'],
             role_id=role_group_domain1['id'])
@@ -333,7 +346,7 @@ class AuthWithToken(AuthTest):
         scoped_token = self.controller.authenticate({}, body_dict)
         tenant = scoped_token["access"]["token"]["tenant"]
         roles = scoped_token["access"]["metadata"]["roles"]
-        self.assertEquals(tenant["id"], project1['id'])
+        self.assertEqual(project1['id'], tenant["id"])
         self.assertIn(self.role_member['id'], roles)
         self.assertIn(self.role_admin['id'], roles)
         self.assertNotIn(role_foo_domain1['id'], roles)
@@ -377,14 +390,14 @@ class AuthWithToken(AuthTest):
             token_id=scoped_token_id)
 
     def test_token_auth_with_binding(self):
-        CONF.token.bind = ['kerberos']
+        self.config_fixture.config(group='token', bind=['kerberos'])
         body_dict = _build_user_auth()
-        context = {'REMOTE_USER': 'FOO', 'AUTH_TYPE': 'Negotiate'}
-        unscoped_token = self.controller.authenticate(context, body_dict)
+        unscoped_token = self.controller.authenticate(
+            self.context_with_remote_user, body_dict)
 
         # the token should have bind information in it
         bind = unscoped_token['access']['token']['bind']
-        self.assertEqual(bind['kerberos'], 'FOO')
+        self.assertEqual('FOO', bind['kerberos'])
 
         body_dict = _build_user_auth(
             token=unscoped_token['access']['token'],
@@ -394,23 +407,24 @@ class AuthWithToken(AuthTest):
         self.assertRaises(
             exception.Unauthorized,
             self.controller.authenticate,
-            {}, body_dict)
+            self.empty_context, body_dict)
 
         # using token with remote user context succeeds
-        scoped_token = self.controller.authenticate(context, body_dict)
+        scoped_token = self.controller.authenticate(
+            self.context_with_remote_user, body_dict)
 
         # the bind information should be carried over from the original token
         bind = scoped_token['access']['token']['bind']
-        self.assertEqual(bind['kerberos'], 'FOO')
+        self.assertEqual('FOO', bind['kerberos'])
 
     def test_deleting_role_revokes_token(self):
-        role_controller = identity.controllers.Role()
+        role_controller = assignment.controllers.Role()
         project1 = {'id': 'Project1', 'name': uuid.uuid4().hex,
                     'domain_id': DEFAULT_DOMAIN_ID}
         self.assignment_api.create_project(project1['id'], project1)
         role_one = {'id': 'role_one', 'name': uuid.uuid4().hex}
         self.assignment_api.create_role(role_one['id'], role_one)
-        self.identity_api.add_role_to_user_and_project(
+        self.assignment_api.add_role_to_user_and_project(
             self.user_foo['id'], project1['id'], role_one['id'])
         no_context = {}
 
@@ -496,12 +510,55 @@ class AuthWithPasswordCredentials(AuthTest):
                           {}, body_dict)
 
     def test_bind_without_remote_user(self):
-        CONF.token.bind = ['kerberos']
+        self.config_fixture.config(group='token', bind=['kerberos'])
         body_dict = _build_user_auth(username='FOO', password='foo2',
                                      tenant_name='BAR')
         token = self.controller.authenticate({}, body_dict)
         self.assertNotIn('bind', token['access']['token'])
 
+    def test_change_default_domain_id(self):
+        # If the default_domain_id config option is not the default then the
+        # user in auth data is from the new default domain.
+
+        # 1) Create a new domain.
+        new_domain_id = uuid.uuid4().hex
+        new_domain = {
+            'description': uuid.uuid4().hex,
+            'enabled': True,
+            'id': new_domain_id,
+            'name': uuid.uuid4().hex,
+        }
+
+        self.assignment_api.create_domain(new_domain_id, new_domain)
+
+        # 2) Create user "foo" in new domain with different password than
+        #    default-domain foo.
+        new_user_id = uuid.uuid4().hex
+        new_user_password = uuid.uuid4().hex
+        new_user = {
+            'id': new_user_id,
+            'name': self.user_foo['name'],
+            'domain_id': new_domain_id,
+            'password': new_user_password,
+            'email': 'foo@bar2.com',
+        }
+
+        self.identity_api.create_user(new_user_id, new_user)
+
+        # 3) Update the default_domain_id config option to the new domain
+
+        self.config_fixture.config(group='identity',
+                                   default_domain_id=new_domain_id)
+
+        # 4) Authenticate as "foo" using the password in the new domain.
+
+        body_dict = _build_user_auth(
+            username=self.user_foo['name'],
+            password=new_user_password)
+
+        # The test is successful if this doesn't raise, so no need to assert.
+        self.controller.authenticate({}, body_dict)
+
 
 class AuthWithRemoteUser(AuthTest):
     def setUp(self):
@@ -517,7 +574,7 @@ class AuthWithRemoteUser(AuthTest):
 
         body_dict = _build_user_auth()
         remote_token = self.controller.authenticate(
-            {'REMOTE_USER': 'FOO'}, body_dict)
+            self.context_with_remote_user, body_dict)
 
         self.assertEqualTokens(local_token, remote_token)
 
@@ -541,7 +598,7 @@ class AuthWithRemoteUser(AuthTest):
         body_dict = _build_user_auth(
             tenant_name='BAR')
         remote_token = self.controller.authenticate(
-            {'REMOTE_USER': 'FOO'}, body_dict)
+            self.context_with_remote_user, body_dict)
 
         self.assertEqualTokens(local_token, remote_token)
 
@@ -556,7 +613,7 @@ class AuthWithRemoteUser(AuthTest):
 
         body_dict = _build_user_auth(tenant_name='BAZ')
         remote_token = self.controller.authenticate(
-            {'REMOTE_USER': 'TWO'}, body_dict)
+            {'environment': {'REMOTE_USER': 'TWO'}}, body_dict)
 
         self.assertEqualTokens(local_token, remote_token)
 
@@ -566,28 +623,27 @@ class AuthWithRemoteUser(AuthTest):
         self.assertRaises(
             exception.Unauthorized,
             self.controller.authenticate,
-            {'REMOTE_USER': uuid.uuid4().hex},
+            {'environment': {'REMOTE_USER': uuid.uuid4().hex}},
             body_dict)
 
     def test_bind_with_kerberos(self):
-        CONF.token.bind = ['kerberos']
-        kerb = {'REMOTE_USER': 'FOO', 'AUTH_TYPE': 'Negotiate'}
+        self.config_fixture.config(group='token', bind=['kerberos'])
         body_dict = _build_user_auth(tenant_name="BAR")
-        token = self.controller.authenticate(kerb, body_dict)
-        self.assertEqual(token['access']['token']['bind']['kerberos'], 'FOO')
+        token = self.controller.authenticate(self.context_with_remote_user,
+                                             body_dict)
+        self.assertEqual('FOO', token['access']['token']['bind']['kerberos'])
 
     def test_bind_without_config_opt(self):
-        CONF.token.bind = ['x509']
-        kerb = {'REMOTE_USER': 'FOO', 'AUTH_TYPE': 'Negotiate'}
+        self.config_fixture.config(group='token', bind=['x509'])
         body_dict = _build_user_auth(tenant_name='BAR')
-        token = self.controller.authenticate(kerb, body_dict)
+        token = self.controller.authenticate(self.context_with_remote_user,
+                                             body_dict)
         self.assertNotIn('bind', token['access']['token'])
 
 
 class AuthWithTrust(AuthTest):
     def setUp(self):
         super(AuthWithTrust, self).setUp()
-        self.opt_in_group('trust', enabled=True)
 
         trust.Manager()
         self.trust_controller = trust.controllers.TrustV3()
@@ -597,7 +653,7 @@ class AuthWithTrust(AuthTest):
         self.assigned_roles = [self.role_member['id'],
                                self.role_browser['id']]
         for assigned_role in self.assigned_roles:
-            self.identity_api.add_role_to_user_and_project(
+            self.assignment_api.add_role_to_user_and_project(
                 self.trustor['id'], self.tenant_bar['id'], assigned_role)
 
         self.sample_data = {'trustor_user_id': self.trustor['id'],
@@ -611,12 +667,25 @@ class AuthWithTrust(AuthTest):
                                        fmt=TIME_FORMAT)
         self.create_trust(expires_at=expires_at)
 
+    def config_overrides(self):
+        super(AuthWithTrust, self).config_overrides()
+        self.config_fixture.config(group='trust', enabled=True)
+
+    def _create_auth_context(self, token_id):
+        token_ref = self.token_api.get_token(token_id)
+        auth_context = authorization.token_to_auth_context(
+            token_ref['token_data'])
+        return {'environment': {authorization.AUTH_CONTEXT_ENV: auth_context},
+                'token_id': token_id,
+                'host_url': HOST_URL}
+
     def create_trust(self, expires_at=None, impersonation=True):
-        username = self.trustor['name'],
+        username = self.trustor['name']
         password = 'foo2'
         body_dict = _build_user_auth(username=username, password=password)
         self.unscoped_token = self.controller.authenticate({}, body_dict)
-        context = {'token_id': self.unscoped_token['access']['token']['id']}
+        context = self._create_auth_context(
+            self.unscoped_token['access']['token']['id'])
         trust_data = copy.deepcopy(self.sample_data)
         trust_data['expires_at'] = expires_at
         trust_data['impersonation'] = impersonation
@@ -634,65 +703,74 @@ class AuthWithTrust(AuthTest):
         return request_body
 
     def test_create_trust_bad_data_fails(self):
-        context = {'token_id': self.unscoped_token['access']['token']['id']}
-        bad_sample_data = {'trustor_user_id': self.trustor['id']}
+        context = self._create_auth_context(
+            self.unscoped_token['access']['token']['id'])
+        bad_sample_data = {'trustor_user_id': self.trustor['id'],
+                           'project_id': self.tenant_bar['id'],
+                           'roles': [{'id': self.role_browser['id']}]}
 
         self.assertRaises(exception.ValidationError,
                           self.trust_controller.create_trust,
                           context, trust=bad_sample_data)
 
     def test_create_trust_no_roles(self):
-        self.new_trust = None
+        context = {'token_id': self.unscoped_token['access']['token']['id']}
         self.sample_data['roles'] = []
-        self.create_trust()
-        self.assertEquals(self.new_trust['roles'], [])
+        self.assertRaises(exception.Forbidden,
+                          self.trust_controller.create_trust,
+                          context, trust=self.sample_data)
 
     def test_create_trust(self):
-        self.assertEquals(self.new_trust['trustor_user_id'],
-                          self.trustor['id'])
-        self.assertEquals(self.new_trust['trustee_user_id'],
-                          self.trustee['id'])
+        self.assertEqual(self.trustor['id'], self.new_trust['trustor_user_id'])
+        self.assertEqual(self.trustee['id'], self.new_trust['trustee_user_id'])
         role_ids = [self.role_browser['id'], self.role_member['id']]
         self.assertTrue(timeutils.parse_strtime(self.new_trust['expires_at'],
                                                 fmt=TIME_FORMAT))
-        self.assertIn('http://localhost:5000/v3/OS-TRUST/',
+        self.assertIn('%s/v3/OS-TRUST/' % HOST_URL,
                       self.new_trust['links']['self'])
-        self.assertIn('http://localhost:5000/v3/OS-TRUST/',
+        self.assertIn('%s/v3/OS-TRUST/' % HOST_URL,
                       self.new_trust['roles_links']['self'])
 
         for role in self.new_trust['roles']:
             self.assertIn(role['id'], role_ids)
 
+    def test_create_trust_expires_bad(self):
+        self.assertRaises(exception.ValidationTimeStampError,
+                          self.create_trust,
+                          expires_at="bad")
+        self.assertRaises(exception.ValidationTimeStampError,
+                          self.create_trust,
+                          expires_at="")
+        self.assertRaises(exception.ValidationTimeStampError,
+                          self.create_trust,
+                          expires_at="Z")
+
     def test_get_trust(self):
-        context = {'token_id': self.unscoped_token['access']['token']['id']}
+        context = {'token_id': self.unscoped_token['access']['token']['id'],
+                   'host_url': HOST_URL}
         trust = self.trust_controller.get_trust(context,
                                                 self.new_trust['id'])['trust']
-        self.assertEquals(trust['trustor_user_id'],
-                          self.trustor['id'])
-        self.assertEquals(trust['trustee_user_id'],
-                          self.trustee['id'])
+        self.assertEqual(self.trustor['id'], trust['trustor_user_id'])
+        self.assertEqual(self.trustee['id'], trust['trustee_user_id'])
         role_ids = [self.role_browser['id'], self.role_member['id']]
         for role in self.new_trust['roles']:
             self.assertIn(role['id'], role_ids)
 
     def test_create_trust_no_impersonation(self):
         self.create_trust(expires_at=None, impersonation=False)
-        self.assertEquals(self.new_trust['trustor_user_id'],
-                          self.trustor['id'])
-        self.assertEquals(self.new_trust['trustee_user_id'],
-                          self.trustee['id'])
+        self.assertEqual(self.trustor['id'], self.new_trust['trustor_user_id'])
+        self.assertEqual(self.trustee['id'], self.new_trust['trustee_user_id'])
         self.assertIs(self.new_trust['impersonation'], False)
         auth_response = self.fetch_v2_token_from_trust()
         token_user = auth_response['access']['user']
-        self.assertEquals(token_user['id'],
-                          self.new_trust['trustee_user_id'])
+        self.assertEqual(token_user['id'], self.new_trust['trustee_user_id'])
 
         # TODO(ayoung): Endpoints
 
     def test_create_trust_impersonation(self):
         self.create_trust(expires_at=None)
-        self.assertEqual(self.new_trust['trustor_user_id'], self.trustor['id'])
-        self.assertEqual(self.new_trust['trustee_user_id'], self.trustee['id'])
+        self.assertEqual(self.trustor['id'], self.new_trust['trustor_user_id'])
+        self.assertEqual(self.trustee['id'], self.new_trust['trustee_user_id'])
         self.assertIs(self.new_trust['impersonation'], True)
         auth_response = self.fetch_v2_token_from_trust()
         token_user = auth_response['access']['user']
@@ -722,7 +800,9 @@ class AuthWithTrust(AuthTest):
                 'project': {
                     'id': self.tenant_baz['id']}}}
         auth_response = (self.auth_v3_controller.authenticate_for_token
-                         ({'query_string': {}}, v3_password_data))
+                         ({'environment': {},
+                           'query_string': {}},
+                          v3_password_data))
         token = auth_response.headers['X-Subject-Token']
 
         v3_req_with_trust = {
@@ -732,24 +812,26 @@ class AuthWithTrust(AuthTest):
             "scope": {
                 "OS-TRUST:trust": {"id": self.new_trust['id']}}}
         token_auth_response = (self.auth_v3_controller.authenticate_for_token
-                               ({'query_string': {}}, v3_req_with_trust))
+                               ({'environment': {},
+                                 'query_string': {}},
+                                v3_req_with_trust))
         return token_auth_response
 
     def test_create_v3_token_from_trust(self):
         auth_response = self.fetch_v3_token_from_trust()
 
         trust_token_user = auth_response.json['token']['user']
-        self.assertEquals(trust_token_user['id'], self.trustor['id'])
+        self.assertEqual(self.trustor['id'], trust_token_user['id'])
 
         trust_token_trust = auth_response.json['token']['OS-TRUST:trust']
-        self.assertEquals(trust_token_trust['id'], self.new_trust['id'])
-        self.assertEquals(trust_token_trust['trustor_user']['id'],
-                          self.trustor['id'])
-        self.assertEquals(trust_token_trust['trustee_user']['id'],
-                          self.trustee['id'])
+        self.assertEqual(trust_token_trust['id'], self.new_trust['id'])
+        self.assertEqual(self.trustor['id'],
+                         trust_token_trust['trustor_user']['id'])
+        self.assertEqual(self.trustee['id'],
+                         trust_token_trust['trustee_user']['id'])
 
         trust_token_roles = auth_response.json['token']['roles']
-        self.assertEquals(len(trust_token_roles), 2)
+        self.assertEqual(2, len(trust_token_roles))
 
     def test_v3_trust_token_get_token_fails(self):
         auth_response = self.fetch_v3_token_from_trust()
@@ -761,28 +843,29 @@ class AuthWithTrust(AuthTest):
         self.assertRaises(
             exception.Forbidden,
             self.auth_v3_controller.authenticate_for_token,
-            {'query_string': {}}, v3_token_data)
+            {'environment': {},
+             'query_string': {}}, v3_token_data)
 
     def test_token_from_trust(self):
         auth_response = self.fetch_v2_token_from_trust()
 
         self.assertIsNotNone(auth_response)
-        self.assertEquals(len(auth_response['access']['metadata']['roles']),
-                          2,
-                          "user_foo has three roles, but the token should"
-                          " only get the two roles specified in the trust.")
+        self.assertEqual(2,
+                         len(auth_response['access']['metadata']['roles']),
+                         "user_foo has three roles, but the token should"
+                         " only get the two roles specified in the trust.")
 
     def assert_token_count_for_trust(self, expected_value):
-        tokens = self.trust_controller.token_api.list_tokens(
+        tokens = self.trust_controller.token_api._list_tokens(
             self.trustee['id'], trust_id=self.new_trust['id'])
         token_count = len(tokens)
-        self.assertEquals(token_count, expected_value)
+        self.assertEqual(expected_value, token_count)
 
     def test_delete_tokens_for_user_invalidates_tokens_from_trust(self):
         self.assert_token_count_for_trust(0)
         self.fetch_v2_token_from_trust()
         self.assert_token_count_for_trust(1)
-        self.trust_controller._delete_tokens_for_user(self.trustee['id'])
+        self.token_api.delete_tokens_for_user(self.trustee['id'])
         self.assert_token_count_for_trust(0)
 
     def test_token_from_trust_cant_get_another_token(self):
@@ -795,20 +878,21 @@ class AuthWithTrust(AuthTest):
             self.controller.authenticate, {}, request_body)
 
     def test_delete_trust_revokes_token(self):
-        context = {'token_id': self.unscoped_token['access']['token']['id']}
+        context = self._create_auth_context(
+            self.unscoped_token['access']['token']['id'])
         self.fetch_v2_token_from_trust()
         trust_id = self.new_trust['id']
-        tokens = self.token_api.list_tokens(self.trustor['id'],
-                                            trust_id=trust_id)
-        self.assertEquals(len(tokens), 1)
+        tokens = self.token_api._list_tokens(self.trustor['id'],
+                                             trust_id=trust_id)
+        self.assertEqual(1, len(tokens))
         self.trust_controller.delete_trust(context, trust_id=trust_id)
-        tokens = self.token_api.list_tokens(self.trustor['id'],
-                                            trust_id=trust_id)
-        self.assertEquals(len(tokens), 0)
+        tokens = self.token_api._list_tokens(self.trustor['id'],
+                                             trust_id=trust_id)
+        self.assertEqual(0, len(tokens))
 
     def test_token_from_trust_with_no_role_fails(self):
         for assigned_role in self.assigned_roles:
-            self.identity_api.remove_role_from_user_and_project(
+            self.assignment_api.remove_role_from_user_and_project(
                 self.trustor['id'], self.tenant_bar['id'], assigned_role)
         request_body = self.build_v2_token_request('TWO', 'two2')
         self.assertRaises(
@@ -824,12 +908,12 @@ class AuthWithTrust(AuthTest):
             self.controller.authenticate, {}, request_body)
 
     def test_token_from_trust_with_wrong_role_fails(self):
-        self.identity_api.add_role_to_user_and_project(
+        self.assignment_api.add_role_to_user_and_project(
             self.trustor['id'],
             self.tenant_bar['id'],
             self.role_other['id'])
         for assigned_role in self.assigned_roles:
-            self.identity_api.remove_role_from_user_and_project(
+            self.assignment_api.remove_role_from_user_and_project(
                 self.trustor['id'], self.tenant_bar['id'], assigned_role)
 
         request_body = self.build_v2_token_request('TWO', 'two2')
@@ -840,9 +924,12 @@ class AuthWithTrust(AuthTest):
 
 
 class TokenExpirationTest(AuthTest):
-    def _maintain_token_expiration(self):
+
+    @mock.patch.object(timeutils, 'utcnow')
+    def _maintain_token_expiration(self, mock_utcnow):
         """Token expiration should be maintained after re-auth & validation."""
-        timeutils.set_time_override()
+        now = datetime.datetime.utcnow()
+        mock_utcnow.return_value = now
 
         r = self.controller.authenticate(
             {},
@@ -855,14 +942,14 @@ class TokenExpirationTest(AuthTest):
         unscoped_token_id = r['access']['token']['id']
         original_expiration = r['access']['token']['expires']
 
-        timeutils.advance_time_seconds(1)
+        mock_utcnow.return_value = now + datetime.timedelta(seconds=1)
 
         r = self.controller.validate_token(
             dict(is_admin=True, query_string={}),
             token_id=unscoped_token_id)
         self.assertEqual(original_expiration, r['access']['token']['expires'])
 
-        timeutils.advance_time_seconds(1)
+        mock_utcnow.return_value = now + datetime.timedelta(seconds=2)
 
         r = self.controller.authenticate(
             {},
@@ -875,7 +962,7 @@ class TokenExpirationTest(AuthTest):
         scoped_token_id = r['access']['token']['id']
         self.assertEqual(original_expiration, r['access']['token']['expires'])
 
-        timeutils.advance_time_seconds(1)
+        mock_utcnow.return_value = now + datetime.timedelta(seconds=3)
 
         r = self.controller.validate_token(
             dict(is_admin=True, query_string={}),
@@ -883,13 +970,125 @@ class TokenExpirationTest(AuthTest):
         self.assertEqual(original_expiration, r['access']['token']['expires'])
 
     def test_maintain_uuid_token_expiration(self):
-        self.opt_in_group('signing', token_format='UUID')
+        self.config_fixture.config(group='signing', token_format='UUID')
         self._maintain_token_expiration()
 
 
+class AuthCatalog(tests.SQLDriverOverrides, AuthTest):
+    """Tests for the catalog provided in the auth response."""
+
+    def config_files(self):
+        config_files = super(AuthCatalog, self).config_files()
+        # We need to use a backend that supports disabled endpoints, like the
+        # SQL backend.
+        config_files.append(tests.dirs.tests_conf('backend_sql.conf'))
+        return config_files
+
+    def _create_endpoints(self):
+        def create_endpoint(service_id, region, **kwargs):
+            id_ = uuid.uuid4().hex
+            ref = {
+                'id': id_,
+                'interface': 'public',
+                'region': region,
+                'service_id': service_id,
+                'url': 'http://localhost/%s' % uuid.uuid4().hex,
+            }
+            ref.update(kwargs)
+            self.catalog_api.create_endpoint(id_, ref)
+            return ref
+
+        # Create a service for use with the endpoints.
+        def create_service(**kwargs):
+            id_ = uuid.uuid4().hex
+            ref = {
+                'id': id_,
+                'name': uuid.uuid4().hex,
+                'type': uuid.uuid4().hex,
+            }
+            ref.update(kwargs)
+            self.catalog_api.create_service(id_, ref)
+            return ref
+
+        enabled_service_ref = create_service(enabled=True)
+        disabled_service_ref = create_service(enabled=False)
+
+        region = uuid.uuid4().hex
+
+        # Create endpoints
+        enabled_endpoint_ref = create_endpoint(
+            enabled_service_ref['id'], region)
+        create_endpoint(
+            enabled_service_ref['id'], region, enabled=False,
+            interface='internal')
+        create_endpoint(
+            disabled_service_ref['id'], region)
+
+        return enabled_endpoint_ref
+
+    def test_auth_catalog_disabled_endpoint(self):
+        """On authenticate, get a catalog that excludes disabled endpoints."""
+        endpoint_ref = self._create_endpoints()
+
+        # Authenticate
+        body_dict = _build_user_auth(
+            username='FOO',
+            password='foo2',
+            tenant_name="BAR")
+
+        token = self.controller.authenticate({}, body_dict)
+
+        # Check the catalog
+        self.assertEqual(1, len(token['access']['serviceCatalog']))
+        endpoint = token['access']['serviceCatalog'][0]['endpoints'][0]
+        self.assertEqual(
+            1, len(token['access']['serviceCatalog'][0]['endpoints']))
+
+        exp_endpoint = {
+            'id': endpoint_ref['id'],
+            'publicURL': endpoint_ref['url'],
+            'region': endpoint_ref['region'],
+        }
+
+        self.assertEqual(exp_endpoint, endpoint)
+
+    def test_validate_catalog_disabled_endpoint(self):
+        """On validate, get back a catalog that excludes disabled endpoints."""
+        endpoint_ref = self._create_endpoints()
+
+        # Authenticate
+        body_dict = _build_user_auth(
+            username='FOO',
+            password='foo2',
+            tenant_name="BAR")
+
+        token = self.controller.authenticate({}, body_dict)
+
+        # Validate
+        token_id = token['access']['token']['id']
+        validate_ref = self.controller.validate_token(
+            dict(is_admin=True, query_string={}),
+            token_id=token_id)
+
+        # Check the catalog
+        self.assertEqual(1, len(token['access']['serviceCatalog']))
+        endpoint = validate_ref['access']['serviceCatalog'][0]['endpoints'][0]
+        self.assertEqual(
+            1, len(token['access']['serviceCatalog'][0]['endpoints']))
+
+        exp_endpoint = {
+            'id': endpoint_ref['id'],
+            'publicURL': endpoint_ref['url'],
+            'region': endpoint_ref['region'],
+        }
+
+        self.assertEqual(exp_endpoint, endpoint)
+
+
 class NonDefaultAuthTest(tests.TestCase):
 
     def test_add_non_default_auth_method(self):
-        self.opt_in_group('auth', methods=['password', 'token', 'custom'])
+        self.config_fixture.config(group='auth',
+                                   methods=['password', 'token', 'custom'])
         config.setup_authentication()
         self.assertTrue(hasattr(CONF.auth, 'custom'))
