@@ -1,5 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
+# -*- coding: utf-8 -*-
 # Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,11 +16,13 @@
 import uuid
 
 import sqlalchemy
+from sqlalchemy import exc
 
 from keystone.common import sql
 from keystone import config
 from keystone import exception
 from keystone.identity.backends import sql as identity_sql
+from keystone.openstack.common.db import exception as db_exception
 from keystone.openstack.common.fixture import moxstubout
 from keystone import tests
 from keystone.tests import default_fixtures
@@ -33,31 +34,21 @@ CONF = config.CONF
 DEFAULT_DOMAIN_ID = CONF.identity.default_domain_id
 
 
-class SqlTests(tests.TestCase, sql.Base):
+class SqlTests(tests.SQLDriverOverrides, tests.TestCase):
 
     def setUp(self):
         super(SqlTests, self).setUp()
-        self.config([tests.etcdir('keystone.conf.sample'),
-                     tests.testsdir('test_overrides.conf'),
-                     tests.testsdir('backend_sql.conf')])
-
         self.load_backends()
 
-        # create tables and keep an engine reference for cleanup.
-        # this must be done after the models are loaded by the managers.
-        self.engine = self.get_engine()
-        sql.ModelBase.metadata.create_all(bind=self.engine)
-
         # populate the engine with tables & fixtures
         self.load_fixtures(default_fixtures)
         #defaulted by the data load
         self.user_foo['enabled'] = True
 
-    def tearDown(self):
-        sql.ModelBase.metadata.drop_all(bind=self.engine)
-        self.engine.dispose()
-        sql.set_global_engine(None)
-        super(SqlTests, self).tearDown()
+    def config_files(self):
+        config_files = super(SqlTests, self).config_files()
+        config_files.append(tests.dirs.tests_conf('backend_sql.conf'))
+        return config_files
 
 
 class SqlModels(SqlTests):
@@ -79,7 +70,7 @@ class SqlModels(SqlTests):
         for col, type_, length in cols:
             self.assertIsInstance(table.c[col].type, type_)
             if length:
-                self.assertEquals(table.c[col].type.length, length)
+                self.assertEqual(table.c[col].type.length, length)
 
     def test_user_model(self):
         cols = (('id', sql.String, 64),
@@ -118,29 +109,13 @@ class SqlModels(SqlTests):
                 ('name', sql.String, 255))
         self.assertExpectedSchema('role', cols)
 
-    def test_user_project_metadata_model(self):
-        cols = (('user_id', sql.String, 64),
-                ('project_id', sql.String, 64),
-                ('data', sql.JsonBlob, None))
-        self.assertExpectedSchema('user_project_metadata', cols)
-
-    def test_user_domain_metadata_model(self):
-        cols = (('user_id', sql.String, 64),
-                ('domain_id', sql.String, 64),
-                ('data', sql.JsonBlob, None))
-        self.assertExpectedSchema('user_domain_metadata', cols)
-
-    def test_group_project_metadata_model(self):
-        cols = (('group_id', sql.String, 64),
-                ('project_id', sql.String, 64),
-                ('data', sql.JsonBlob, None))
-        self.assertExpectedSchema('group_project_metadata', cols)
-
-    def test_group_domain_metadata_model(self):
-        cols = (('group_id', sql.String, 64),
-                ('domain_id', sql.String, 64),
-                ('data', sql.JsonBlob, None))
-        self.assertExpectedSchema('group_domain_metadata', cols)
+    def test_role_assignment_model(self):
+        cols = (('type', sql.Enum, None),
+                ('actor_id', sql.String, 64),
+                ('target_id', sql.String, 64),
+                ('role_id', sql.String, 64),
+                ('inherited', sql.Boolean, False))
+        self.assertExpectedSchema('assignment', cols)
 
     def test_user_group_membership(self):
         cols = (('group_id', sql.String, 64),
@@ -150,7 +125,7 @@ class SqlModels(SqlTests):
 
 class SqlIdentity(SqlTests, test_backend.IdentityTests):
     def test_password_hashed(self):
-        session = self.identity_api.get_session()
+        session = sql.get_session()
         user_ref = self.identity_api._get_user(session, self.user_foo['id'])
         self.assertNotEqual(user_ref['password'], self.user_foo['password'])
 
@@ -160,8 +135,8 @@ class SqlIdentity(SqlTests, test_backend.IdentityTests):
                 'domain_id': DEFAULT_DOMAIN_ID,
                 'password': uuid.uuid4().hex}
         self.identity_api.create_user(user['id'], user)
-        self.identity_api.add_user_to_project(self.tenant_bar['id'],
-                                              user['id'])
+        self.assignment_api.add_user_to_project(self.tenant_bar['id'],
+                                                user['id'])
         self.identity_api.delete_user(user['id'])
         self.assertRaises(exception.UserNotFound,
                           self.assignment_api.list_projects_for_user,
@@ -193,10 +168,10 @@ class SqlIdentity(SqlTests, test_backend.IdentityTests):
                           tenant['id'],
                           tenant)
         self.assertRaises(exception.ProjectNotFound,
-                          self.identity_api.get_project,
+                          self.assignment_api.get_project,
                           tenant['id'])
         self.assertRaises(exception.ProjectNotFound,
-                          self.identity_api.get_project_by_name,
+                          self.assignment_api.get_project_by_name,
                           tenant['name'],
                           DEFAULT_DOMAIN_ID)
 
@@ -204,11 +179,11 @@ class SqlIdentity(SqlTests, test_backend.IdentityTests):
         role = {'id': uuid.uuid4().hex,
                 'name': None}
         self.assertRaises(exception.Conflict,
-                          self.identity_api.create_role,
+                          self.assignment_api.create_role,
                           role['id'],
                           role)
         self.assertRaises(exception.RoleNotFound,
-                          self.identity_api.get_role,
+                          self.assignment_api.get_role,
                           role['id'])
 
     def test_delete_project_with_user_association(self):
@@ -217,11 +192,11 @@ class SqlIdentity(SqlTests, test_backend.IdentityTests):
                 'domain_id': DEFAULT_DOMAIN_ID,
                 'password': 'passwd'}
         self.identity_api.create_user('fake', user)
-        self.identity_api.add_user_to_project(self.tenant_bar['id'],
-                                              user['id'])
+        self.assignment_api.add_user_to_project(self.tenant_bar['id'],
+                                                user['id'])
         self.assignment_api.delete_project(self.tenant_bar['id'])
         tenants = self.assignment_api.list_projects_for_user(user['id'])
-        self.assertEquals(tenants, [])
+        self.assertEqual(tenants, [])
 
     def test_metadata_removed_on_delete_user(self):
         # A test to check that the internal representation
@@ -233,8 +208,8 @@ class SqlIdentity(SqlTests, test_backend.IdentityTests):
         self.identity_api.create_user(user['id'], user)
         role = {'id': uuid.uuid4().hex,
                 'name': uuid.uuid4().hex}
-        self.identity_api.create_role(role['id'], role)
-        self.identity_api.add_role_to_user_and_project(
+        self.assignment_api.create_role(role['id'], role)
+        self.assignment_api.add_role_to_user_and_project(
             user['id'],
             self.tenant_bar['id'],
             role['id'])
@@ -257,8 +232,8 @@ class SqlIdentity(SqlTests, test_backend.IdentityTests):
         self.identity_api.create_user(user['id'], user)
         role = {'id': uuid.uuid4().hex,
                 'name': uuid.uuid4().hex}
-        self.identity_api.create_role(role['id'], role)
-        self.identity_api.add_role_to_user_and_project(
+        self.assignment_api.create_role(role['id'], role)
+        self.assignment_api.add_role_to_user_and_project(
             user['id'],
             self.tenant_bar['id'],
             role['id'])
@@ -339,7 +314,7 @@ class SqlIdentity(SqlTests, test_backend.IdentityTests):
             'password': uuid.uuid4().hex}
 
         self.identity_api.create_user(user_id, user)
-        session = self.get_session()
+        session = sql.get_session()
         query = session.query(identity_sql.User)
         query = query.filter_by(id=user_id)
         raw_user_ref = query.one()
@@ -361,17 +336,50 @@ class SqlToken(SqlTests, test_backend.TokenTests):
         fixture = self.useFixture(moxstubout.MoxStubout())
         self.mox = fixture.mox
         tok = token_sql.Token()
-        session = tok.get_session()
+        session = sql.get_session()
         q = session.query(token_sql.TokenModel.id,
                           token_sql.TokenModel.expires)
         self.mox.StubOutWithMock(session, 'query')
         session.query(token_sql.TokenModel.id,
                       token_sql.TokenModel.expires).AndReturn(q)
-        self.mox.StubOutWithMock(tok, 'get_session')
-        tok.get_session().AndReturn(session)
+        self.mox.StubOutWithMock(sql, 'get_session')
+        sql.get_session().AndReturn(session)
         self.mox.ReplayAll()
         tok.list_revoked_tokens()
 
+    def test_flush_expired_tokens_batch(self):
+        # This test simply executes the code under test to verify
+        # that the code is legal.  It is not possible to test
+        # whether records are deleted in batches using sqlite,
+        # because the limit function does not seem to affect
+        # delete subqueries; these are, however, legal.
+        # After several failed attempts of using mox, it would
+        # seem that the use of mock objects for testing
+        # the target code does not seem possible, because of
+        # the unique way the SQLAlchemy Query class's filter
+        # method works.
+        fixture = self.useFixture(moxstubout.MoxStubout())
+        self.mox = fixture.mox
+        tok = token_sql.Token()
+        self.mox.StubOutWithMock(tok, 'token_flush_batch_size')
+        # Just need a batch larger than 0; note that the code
+        # path with batch_size = 0 is covered by test_backend,
+        # where all backends' flush_expired_tokens methods
+        # are tested.
+        tok.token_flush_batch_size('sqlite').AndReturn(1)
+        self.mox.ReplayAll()
+        tok.flush_expired_tokens()
+
+    def test_token_flush_batch_size_default(self):
+        tok = token_sql.Token()
+        sqlite_batch = tok.token_flush_batch_size('sqlite')
+        self.assertEqual(sqlite_batch, 0)
+
+    def test_token_flush_batch_size_db2(self):
+        tok = token_sql.Token()
+        db2_batch = tok.token_flush_batch_size('ibm_db_sa')
+        self.assertEqual(db2_batch, 100)
+
 
 class SqlCatalog(SqlTests, test_backend.CatalogTests):
     def test_malformed_catalog_throws_error(self):
@@ -459,3 +467,54 @@ class SqlTokenCacheInvalidation(SqlTests, test_backend.TokenCacheInvalidation):
     def setUp(self):
         super(SqlTokenCacheInvalidation, self).setUp()
         self._create_test_data()
+
+
+class SqlFilterTests(SqlTests, test_backend.FilterTests):
+    pass
+
+
+class SqlLimitTests(SqlTests, test_backend.LimitTests):
+    def setUp(self):
+        super(SqlLimitTests, self).setUp()
+        test_backend.LimitTests.setUp(self)
+
+
+class FakeTable(sql.ModelBase):
+    __tablename__ = 'test_table'
+    col = sql.Column(sql.String(32), primary_key=True)
+
+    @sql.handle_conflicts('keystone')
+    def insert(self):
+        raise db_exception.DBDuplicateEntry
+
+    @sql.handle_conflicts('keystone')
+    def update(self):
+        raise db_exception.DBError(
+            inner_exception=exc.IntegrityError('a', 'a', 'a'))
+
+    @sql.handle_conflicts('keystone')
+    def lookup(self):
+        raise KeyError
+
+
+class SqlDecorators(tests.TestCase):
+
+    def test_initialization_fail(self):
+        self.assertRaises(exception.StringLengthExceeded,
+                          FakeTable, col='a' * 64)
+
+    def test_initialization(self):
+        tt = FakeTable(col='a')
+        self.assertEqual('a', tt.col)
+
+    def test_non_ascii_init(self):
+        # NOTE(I159): Non ASCII characters must cause UnicodeDecodeError
+        # if encoding is not provided explicitly.
+        self.assertRaises(UnicodeDecodeError, FakeTable, col='Я')
+
+    def test_conflict_happend(self):
+        self.assertRaises(exception.Conflict, FakeTable().insert)
+        self.assertRaises(exception.Conflict, FakeTable().update)
+
+    def test_not_conflict_error(self):
+        self.assertRaises(KeyError, FakeTable().lookup)
