@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,6 +14,8 @@
 
 import uuid
 
+import six
+
 from keystone.common import wsgi
 from keystone import config
 from keystone import exception
@@ -60,14 +60,14 @@ class ExceptionTestCase(tests.TestCase):
         attribute = uuid.uuid4().hex
         e = exception.ValidationError(target=target, attribute=attribute)
         self.assertValidJsonRendering(e)
-        self.assertIn(target, unicode(e))
-        self.assertIn(attribute, unicode(e))
+        self.assertIn(target, six.text_type(e))
+        self.assertIn(attribute, six.text_type(e))
 
     def test_not_found(self):
         target = uuid.uuid4().hex
         e = exception.NotFound(target=target)
         self.assertValidJsonRendering(e)
-        self.assertIn(target, unicode(e))
+        self.assertIn(target, six.text_type(e))
 
     def test_403_title(self):
         e = exception.Forbidden()
@@ -81,7 +81,7 @@ class ExceptionTestCase(tests.TestCase):
         e = exception.Error(message)
 
         try:
-            self.assertEqual(unicode(e), message)
+            self.assertEqual(six.text_type(e), message)
         except UnicodeEncodeError:
             self.fail("unicode error message not supported")
 
@@ -100,71 +100,119 @@ class ExceptionTestCase(tests.TestCase):
         self.assertIn('%(attribute)', e.message)
 
 
+class UnexpectedExceptionTestCase(ExceptionTestCase):
+    """Tests if internal info is exposed to the API user on UnexpectedError."""
+
+    class SubClassExc(exception.UnexpectedError):
+        debug_message_format = 'Debug Message: %(debug_info)s'
+
+    def setUp(self):
+        super(UnexpectedExceptionTestCase, self).setUp()
+        self.exc_str = uuid.uuid4().hex
+
+    def test_unexpected_error_no_debug(self):
+        self.config_fixture.config(debug=False)
+        e = exception.UnexpectedError(exception=self.exc_str)
+        self.assertNotIn(self.exc_str, six.text_type(e))
+
+    def test_unexpected_error_debug(self):
+        self.config_fixture.config(debug=True)
+        e = exception.UnexpectedError(exception=self.exc_str)
+        self.assertIn(self.exc_str, six.text_type(e))
+
+    def test_unexpected_error_subclass_no_debug(self):
+        self.config_fixture.config(debug=False)
+        e = UnexpectedExceptionTestCase.SubClassExc(
+            debug_info=self.exc_str)
+        self.assertEqual(exception.UnexpectedError._message_format,
+                         six.text_type(e))
+
+    def test_unexpected_error_subclass_debug(self):
+        self.config_fixture.config(debug=True)
+        subclass = self.SubClassExc
+
+        e = subclass(debug_info=self.exc_str)
+        expected = subclass.debug_message_format % {'debug_info': self.exc_str}
+        self.assertEqual(expected, six.text_type(e))
+
+    def test_unexpected_error_custom_message_no_debug(self):
+        self.config_fixture.config(debug=False)
+        e = exception.UnexpectedError(self.exc_str)
+        self.assertEqual(exception.UnexpectedError._message_format,
+                         six.text_type(e))
+
+    def test_unexpected_error_custom_message_debug(self):
+        self.config_fixture.config(debug=True)
+        e = exception.UnexpectedError(self.exc_str)
+        self.assertEqual(self.exc_str,
+                         six.text_type(e))
+
+
 class SecurityErrorTestCase(ExceptionTestCase):
     """Tests whether security-related info is exposed to the API user."""
     def test_unauthorized_exposure(self):
-        self.opt(debug=False)
+        self.config_fixture.config(debug=False)
 
         risky_info = uuid.uuid4().hex
         e = exception.Unauthorized(message=risky_info)
         self.assertValidJsonRendering(e)
-        self.assertNotIn(risky_info, unicode(e))
+        self.assertNotIn(risky_info, six.text_type(e))
 
     def test_unauthorized_exposure_in_debug(self):
-        self.opt(debug=True)
+        self.config_fixture.config(debug=True)
 
         risky_info = uuid.uuid4().hex
         e = exception.Unauthorized(message=risky_info)
         self.assertValidJsonRendering(e)
-        self.assertIn(risky_info, unicode(e))
+        self.assertIn(risky_info, six.text_type(e))
 
     def test_forbidden_exposure(self):
-        self.opt(debug=False)
+        self.config_fixture.config(debug=False)
 
         risky_info = uuid.uuid4().hex
         e = exception.Forbidden(message=risky_info)
         self.assertValidJsonRendering(e)
-        self.assertNotIn(risky_info, unicode(e))
+        self.assertNotIn(risky_info, six.text_type(e))
 
     def test_forbidden_exposure_in_debug(self):
-        self.opt(debug=True)
+        self.config_fixture.config(debug=True)
 
         risky_info = uuid.uuid4().hex
         e = exception.Forbidden(message=risky_info)
         self.assertValidJsonRendering(e)
-        self.assertIn(risky_info, unicode(e))
+        self.assertIn(risky_info, six.text_type(e))
 
     def test_forbidden_action_exposure(self):
-        self.opt(debug=False)
+        self.config_fixture.config(debug=False)
 
         risky_info = uuid.uuid4().hex
         action = uuid.uuid4().hex
         e = exception.ForbiddenAction(message=risky_info, action=action)
         self.assertValidJsonRendering(e)
-        self.assertNotIn(risky_info, unicode(e))
-        self.assertIn(action, unicode(e))
+        self.assertNotIn(risky_info, six.text_type(e))
+        self.assertIn(action, six.text_type(e))
 
         e = exception.ForbiddenAction(action=risky_info)
         self.assertValidJsonRendering(e)
-        self.assertIn(risky_info, unicode(e))
+        self.assertIn(risky_info, six.text_type(e))
 
     def test_forbidden_action_exposure_in_debug(self):
-        self.opt(debug=True)
+        self.config_fixture.config(debug=True)
 
         risky_info = uuid.uuid4().hex
 
         e = exception.ForbiddenAction(message=risky_info)
         self.assertValidJsonRendering(e)
-        self.assertIn(risky_info, unicode(e))
+        self.assertIn(risky_info, six.text_type(e))
 
         e = exception.ForbiddenAction(action=risky_info)
         self.assertValidJsonRendering(e)
-        self.assertIn(risky_info, unicode(e))
+        self.assertIn(risky_info, six.text_type(e))
 
     def test_unicode_argument_message(self):
-        self.opt(debug=False)
+        self.config_fixture.config(debug=False)
 
         risky_info = u'\u7ee7\u7eed\u884c\u7f29\u8fdb\u6216'
         e = exception.Forbidden(message=risky_info)
         self.assertValidJsonRendering(e)
-        self.assertNotIn(risky_info, unicode(e))
+        self.assertNotIn(risky_info, six.text_type(e))
