@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -21,9 +19,9 @@ from keystone.common import dependency
 
 
 class TestDependencyInjection(testtools.TestCase):
-    def tearDown(self):
-        dependency.reset()
-        super(TestDependencyInjection, self).tearDown()
+    def setUp(self):
+        super(TestDependencyInjection, self).setUp()
+        self.addCleanup(dependency.reset)
 
     def test_dependency_injection(self):
         class Interface(object):
