@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,14 +12,19 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import datetime
+import os
 import uuid
 import webob
 
+import mock
+
 from keystone import config
 from keystone.openstack.common import jsonutils
 from keystone.openstack.common import timeutils
 from keystone import tests
 from keystone.tests import default_fixtures
+from keystone.tests.ksfixtures import appserver
 
 
 CONF = config.CONF
@@ -30,37 +33,42 @@ OPENSTACK_REPO = 'https://review.openstack.org/p/openstack'
 KEYSTONECLIENT_REPO = '%s/python-keystoneclient.git' % OPENSTACK_REPO
 
 
-class CompatTestCase(tests.TestCase):
+class CompatTestCase(tests.NoModule, tests.TestCase):
+
     def setUp(self):
         super(CompatTestCase, self).setUp()
 
-        # The backends should be loaded and initialized before the servers are
-        # started because the servers use the backends.
-
+        # FIXME(morganfainberg): Since we are running tests through the
+        # controllers and some internal api drivers are SQL-only, the correct
+        # approach is to ensure we have the correct backing store. The
+        # credential api makes some very SQL specific assumptions that should
+        # be addressed allowing for non-SQL based testing to occur.
         self.load_backends()
+
         self.load_fixtures(default_fixtures)
 
         # TODO(termie): add an admin user to the fixtures and use that user
         # override the fixtures, for now
-        self.metadata_foobar = self.identity_api.add_role_to_user_and_project(
+        self.assignment_api.add_role_to_user_and_project(
             self.user_foo['id'],
             self.tenant_bar['id'],
             self.role_admin['id'])
 
-        self.public_server = self.serveapp('keystone', name='main')
-        self.admin_server = self.serveapp('keystone', name='admin')
+        conf = self._paste_config('keystone')
+        fixture = self.useFixture(appserver.AppServer(conf, appserver.MAIN))
+        self.public_server = fixture.server
+        fixture = self.useFixture(appserver.AppServer(conf, appserver.ADMIN))
+        self.admin_server = fixture.server
 
-        revdir = tests.checkout_vendor(*self.get_checkout())
+        self.addCleanup(self.cleanup_instance('public_server', 'admin_server'))
+
+        if isinstance(self.checkout_info, str):
+            revdir = self.checkout_info
+        else:
+            revdir = tests.checkout_vendor(*self.checkout_info)
         self.add_path(revdir)
         self.clear_module('keystoneclient')
 
-    def tearDown(self):
-        self.public_server.kill()
-        self.admin_server.kill()
-        self.public_server = None
-        self.admin_server = None
-        super(CompatTestCase, self).tearDown()
-
     def _public_url(self):
         public_port = self.public_server.socket_info['socket'][1]
         return "http://localhost:%s/v2.0" % public_port
@@ -103,14 +111,14 @@ class KeystoneClientTests(object):
     def test_authenticate_tenant_name_and_tenants(self):
         client = self.get_client()
         tenants = client.tenants.list()
-        self.assertEquals(tenants[0].id, self.tenant_bar['id'])
+        self.assertEqual(tenants[0].id, self.tenant_bar['id'])
 
     def test_authenticate_tenant_id_and_tenants(self):
         client = self._client(username=self.user_foo['name'],
                               password=self.user_foo['password'],
                               tenant_id='bar')
         tenants = client.tenants.list()
-        self.assertEquals(tenants[0].id, self.tenant_bar['id'])
+        self.assertEqual(tenants[0].id, self.tenant_bar['id'])
 
     def test_authenticate_invalid_tenant_id(self):
         from keystoneclient import exceptions as client_exceptions
@@ -125,14 +133,14 @@ class KeystoneClientTests(object):
         token = client.auth_token
         token_client = self._client(token=token)
         tenants = token_client.tenants.list()
-        self.assertEquals(tenants[0].id, self.tenant_bar['id'])
+        self.assertEqual(tenants[0].id, self.tenant_bar['id'])
 
     def test_authenticate_token_tenant_id(self):
         client = self.get_client()
         token = client.auth_token
         token_client = self._client(token=token, tenant_id='bar')
         tenants = token_client.tenants.list()
-        self.assertEquals(tenants[0].id, self.tenant_bar['id'])
+        self.assertEqual(tenants[0].id, self.tenant_bar['id'])
 
     def test_authenticate_token_invalid_tenant_id(self):
         from keystoneclient import exceptions as client_exceptions
@@ -155,8 +163,8 @@ class KeystoneClientTests(object):
         token = client.auth_token
         token_client = self._client(token=token, tenant_name='BAR')
         tenants = token_client.tenants.list()
-        self.assertEquals(tenants[0].id, self.tenant_bar['id'])
-        self.assertEquals(tenants[0].id, self.tenant_bar['id'])
+        self.assertEqual(tenants[0].id, self.tenant_bar['id'])
+        self.assertEqual(tenants[0].id, self.tenant_bar['id'])
 
     def test_authenticate_and_delete_token(self):
         from keystoneclient import exceptions as client_exceptions
@@ -165,7 +173,7 @@ class KeystoneClientTests(object):
         token = client.auth_token
         token_client = self._client(token=token)
         tenants = token_client.tenants.list()
-        self.assertEquals(tenants[0].id, self.tenant_bar['id'])
+        self.assertEqual(tenants[0].id, self.tenant_bar['id'])
 
         client.tokens.delete(token_client.auth_token)
 
@@ -254,19 +262,19 @@ class KeystoneClientTests(object):
         tenant = client.tenants.create(tenant_name=tenant_name,
                                        description=tenant_description,
                                        enabled=tenant_enabled)
-        self.assertEquals(tenant.name, tenant_name)
-        self.assertEquals(tenant.description, tenant_description)
-        self.assertEquals(tenant.enabled, tenant_enabled)
+        self.assertEqual(tenant.name, tenant_name)
+        self.assertEqual(tenant.description, tenant_description)
+        self.assertEqual(tenant.enabled, tenant_enabled)
 
         tenant = client.tenants.get(tenant_id=tenant.id)
-        self.assertEquals(tenant.name, tenant_name)
-        self.assertEquals(tenant.description, tenant_description)
-        self.assertEquals(tenant.enabled, tenant_enabled)
+        self.assertEqual(tenant.name, tenant_name)
+        self.assertEqual(tenant.description, tenant_description)
+        self.assertEqual(tenant.enabled, tenant_enabled)
 
         tenant = [t for t in client.tenants.list() if t.id == tenant.id].pop()
-        self.assertEquals(tenant.name, tenant_name)
-        self.assertEquals(tenant.description, tenant_description)
-        self.assertEquals(tenant.enabled, tenant_enabled)
+        self.assertEqual(tenant.name, tenant_name)
+        self.assertEqual(tenant.description, tenant_description)
+        self.assertEqual(tenant.enabled, tenant_enabled)
 
         # update, get, and list a tenant
         tenant_name = 'updated_tenant'
@@ -276,19 +284,19 @@ class KeystoneClientTests(object):
                                        tenant_name=tenant_name,
                                        enabled=tenant_enabled,
                                        description=tenant_description)
-        self.assertEquals(tenant.name, tenant_name)
-        self.assertEquals(tenant.description, tenant_description)
-        self.assertEquals(tenant.enabled, tenant_enabled)
+        self.assertEqual(tenant.name, tenant_name)
+        self.assertEqual(tenant.description, tenant_description)
+        self.assertEqual(tenant.enabled, tenant_enabled)
 
         tenant = client.tenants.get(tenant_id=tenant.id)
-        self.assertEquals(tenant.name, tenant_name)
-        self.assertEquals(tenant.description, tenant_description)
-        self.assertEquals(tenant.enabled, tenant_enabled)
+        self.assertEqual(tenant.name, tenant_name)
+        self.assertEqual(tenant.description, tenant_description)
+        self.assertEqual(tenant.enabled, tenant_enabled)
 
         tenant = [t for t in client.tenants.list() if t.id == tenant.id].pop()
-        self.assertEquals(tenant.name, tenant_name)
-        self.assertEquals(tenant.description, tenant_description)
-        self.assertEquals(tenant.enabled, tenant_enabled)
+        self.assertEqual(tenant.name, tenant_name)
+        self.assertEqual(tenant.description, tenant_description)
+        self.assertEqual(tenant.enabled, tenant_enabled)
 
         # delete, get, and list a tenant
         client.tenants.delete(tenant=tenant.id)
@@ -384,12 +392,12 @@ class KeystoneClientTests(object):
     def test_tenant_list(self):
         client = self.get_client()
         tenants = client.tenants.list()
-        self.assertEquals(len(tenants), 1)
+        self.assertEqual(len(tenants), 1)
 
         # Admin endpoint should return *all* tenants
         client = self.get_client(admin=True)
         tenants = client.tenants.list()
-        self.assertEquals(len(tenants), len(default_fixtures.TENANTS))
+        self.assertEqual(len(tenants), len(default_fixtures.TENANTS))
 
     def test_invalid_password(self):
         from keystoneclient import exceptions as client_exceptions
@@ -515,12 +523,14 @@ class KeystoneClientTests(object):
                           client.tokens.authenticate,
                           token=token_id)
 
-    def test_token_expiry_maintained(self):
-        timeutils.set_time_override()
+    @mock.patch.object(timeutils, 'utcnow')
+    def test_token_expiry_maintained(self, mock_utcnow):
+        now = datetime.datetime.utcnow()
+        mock_utcnow.return_value = now
         foo_client = self.get_client(self.user_foo)
 
         orig_token = foo_client.service_catalog.catalog['token']
-        timeutils.advance_time_seconds(1)
+        mock_utcnow.return_value = now + datetime.timedelta(seconds=1)
         reauthenticated_token = foo_client.tokens.authenticate(
             token=foo_client.auth_token)
 
@@ -536,15 +546,15 @@ class KeystoneClientTests(object):
         user = client.users.create(name=test_username,
                                    password='password',
                                    email='user1@test.com')
-        self.assertEquals(user.name, test_username)
+        self.assertEqual(user.name, test_username)
 
         user = client.users.get(user=user.id)
-        self.assertEquals(user.name, test_username)
+        self.assertEqual(user.name, test_username)
 
         user = client.users.update(user=user,
                                    name=test_username,
                                    email='user2@test.com')
-        self.assertEquals(user.email, 'user2@test.com')
+        self.assertEqual(user.email, 'user2@test.com')
 
         # NOTE(termie): update_enabled doesn't return anything, probably a bug
         client.users.update_enabled(user=user, enabled=False)
@@ -576,7 +586,7 @@ class KeystoneClientTests(object):
                                     password='password',
                                     email='user1@test.com',
                                     tenant_id='bar')
-        self.assertEquals(user2.name, test_username)
+        self.assertEqual(user2.name, test_username)
 
     def test_update_default_tenant_to_existing_value(self):
         client = self.get_client(admin=True)
@@ -640,14 +650,12 @@ class KeystoneClientTests(object):
                           client.users.update,
                           user=uuid.uuid4().hex)
 
-    def test_user_update_tenant_404(self):
-        self.skipTest('N/A')
-        from keystoneclient import exceptions as client_exceptions
+    def test_user_update_tenant(self):
         client = self.get_client(admin=True)
-        self.assertRaises(client_exceptions.NotFound,
-                          client.users.update,
-                          user=self.user_foo['id'],
-                          tenant_id=uuid.uuid4().hex)
+        tenant_id = uuid.uuid4().hex
+        user = client.users.update(user=self.user_foo['id'],
+                                   tenant_id=tenant_id)
+        self.assertEqual(tenant_id, user.tenant_id)
 
     def test_user_update_password_404(self):
         from keystoneclient import exceptions as client_exceptions
@@ -679,7 +687,7 @@ class KeystoneClientTests(object):
     def test_role_get(self):
         client = self.get_client(admin=True)
         role = client.roles.get(role=self.role_admin['id'])
-        self.assertEquals(role.id, self.role_admin['id'])
+        self.assertEqual(role.id, self.role_admin['id'])
 
     def test_role_crud(self):
         from keystoneclient import exceptions as client_exceptions
@@ -687,10 +695,10 @@ class KeystoneClientTests(object):
         test_role = 'new_role'
         client = self.get_client(admin=True)
         role = client.roles.create(name=test_role)
-        self.assertEquals(role.name, test_role)
+        self.assertEqual(role.name, test_role)
 
         role = client.roles.get(role=role.id)
-        self.assertEquals(role.name, test_role)
+        self.assertEqual(role.name, test_role)
 
         client.roles.delete(role=role.id)
 
@@ -756,19 +764,19 @@ class KeystoneClientTests(object):
         service = client.services.create(name=service_name,
                                          service_type=service_type,
                                          description=service_desc)
-        self.assertEquals(service_name, service.name)
-        self.assertEquals(service_type, service.type)
-        self.assertEquals(service_desc, service.description)
+        self.assertEqual(service_name, service.name)
+        self.assertEqual(service_type, service.type)
+        self.assertEqual(service_desc, service.description)
 
         service = client.services.get(id=service.id)
-        self.assertEquals(service_name, service.name)
-        self.assertEquals(service_type, service.type)
-        self.assertEquals(service_desc, service.description)
+        self.assertEqual(service_name, service.name)
+        self.assertEqual(service_type, service.type)
+        self.assertEqual(service_desc, service.description)
 
         service = [x for x in client.services.list() if x.id == service.id][0]
-        self.assertEquals(service_name, service.name)
-        self.assertEquals(service_type, service.type)
-        self.assertEquals(service_desc, service.description)
+        self.assertEqual(service_name, service.name)
+        self.assertEqual(service_type, service.type)
+        self.assertEqual(service_desc, service.description)
 
         # update is not supported in API v2...
 
@@ -778,7 +786,7 @@ class KeystoneClientTests(object):
                           client.services.get,
                           id=service.id)
         services = [x for x in client.services.list() if x.id == service.id]
-        self.assertEquals(len(services), 0)
+        self.assertEqual(len(services), 0)
 
     def test_service_delete_404(self):
         from keystoneclient import exceptions as client_exceptions
@@ -854,18 +862,13 @@ class KeystoneClientTests(object):
         # TODO(ja): MEMBERSHIP CRUD
         # TODO(ja): determine what else todo
 
-
-class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
-    def get_checkout(self):
-        return KEYSTONECLIENT_REPO, 'master'
-
     def test_tenant_add_and_remove_user(self):
         client = self.get_client(admin=True)
         client.roles.add_user_role(tenant=self.tenant_bar['id'],
                                    user=self.user_two['id'],
                                    role=self.role_other['id'])
         user_refs = client.tenants.list_users(tenant=self.tenant_bar['id'])
-        self.assert_(self.user_two['id'] in [x.id for x in user_refs])
+        self.assertTrue(self.user_two['id'] in [x.id for x in user_refs])
         client.roles.remove_user_role(tenant=self.tenant_bar['id'],
                                       user=self.user_two['id'],
                                       role=self.role_other['id'])
@@ -883,17 +886,19 @@ class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
                           tenant=uuid.uuid4().hex,
                           user=self.user_foo['id'],
                           role=self.role_member['id'])
-        self.assertRaises(client_exceptions.NotFound,
-                          client.roles.add_user_role,
-                          tenant=self.tenant_baz['id'],
-                          user=uuid.uuid4().hex,
-                          role=self.role_member['id'])
         self.assertRaises(client_exceptions.NotFound,
                           client.roles.add_user_role,
                           tenant=self.tenant_baz['id'],
                           user=self.user_foo['id'],
                           role=uuid.uuid4().hex)
 
+    def test_user_role_add_no_user(self):
+        # If add_user_role and user doesn't exist, doesn't fail.
+        client = self.get_client(admin=True)
+        client.roles.add_user_role(tenant=self.tenant_baz['id'],
+                                   user=uuid.uuid4().hex,
+                                   role=self.role_member['id'])
+
     def test_user_role_remove_404(self):
         from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
@@ -927,8 +932,8 @@ class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
             tenant = {'name': 'tenant-%s' % tenant_id, 'id': tenant_id,
                       'domain_id': DEFAULT_DOMAIN_ID}
             self.assignment_api.create_project(tenant_id, tenant)
-            self.identity_api.add_user_to_project(tenant_id,
-                                                  self.user_foo['id'])
+            self.assignment_api.add_user_to_project(tenant_id,
+                                                    self.user_foo['id'])
 
         tenants = client.tenants.list()
         self.assertEqual(len(tenants), 3)
@@ -954,8 +959,8 @@ class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
             tenant = {'name': 'tenant-%s' % tenant_id, 'id': tenant_id,
                       'domain_id': DEFAULT_DOMAIN_ID}
             self.assignment_api.create_project(tenant_id, tenant)
-            self.identity_api.add_user_to_project(tenant_id,
-                                                  self.user_foo['id'])
+            self.assignment_api.add_user_to_project(tenant_id,
+                                                    self.user_foo['id'])
 
         tenants = client.tenants.list()
         self.assertEqual(len(tenants), 3)
@@ -1028,7 +1033,7 @@ class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
                     (new_password, self.user_two['password']))
         self.public_server.application(req.environ,
                                        responseobject.start_fake_response)
-        self.assertEquals(403, responseobject.response_status)
+        self.assertEqual(403, responseobject.response_status)
 
         self.user_two['password'] = new_password
         self.assertRaises(client_exceptions.Unauthorized,
@@ -1067,112 +1072,20 @@ class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
         client.tenants.list()
 
 
-class KcEssex3TestCase(CompatTestCase, KeystoneClientTests):
-    def get_checkout(self):
-        return KEYSTONECLIENT_REPO, 'essex-3'
-
-    def test_tenant_add_and_remove_user(self):
-        client = self.get_client(admin=True)
-        client.roles.add_user_to_tenant(tenant_id=self.tenant_bar['id'],
-                                        user_id=self.user_two['id'],
-                                        role_id=self.role_member['id'])
-        role_refs = client.roles.get_user_role_refs(
-            user_id=self.user_two['id'])
-        self.assert_(self.tenant_baz['id'] in [x.tenantId for x in role_refs])
-
-        # get the "role_refs" so we get the proper id, this is how the clients
-        # do it
-        roleref_refs = client.roles.get_user_role_refs(
-            user_id=self.user_two['id'])
-        for roleref_ref in roleref_refs:
-            if (roleref_ref.roleId == self.role_member['id']
-                    and roleref_ref.tenantId == self.tenant_baz['id']):
-                # use python's scope fall through to leave roleref_ref set
-                break
-
-        client.roles.remove_user_from_tenant(tenant_id=self.tenant_bar['id'],
-                                             user_id=self.user_two['id'],
-                                             role_id=roleref_ref.id)
-
-        role_refs = client.roles.get_user_role_refs(
-            user_id=self.user_two['id'])
-        self.assert_(self.tenant_baz['id'] not in
-                     [x.tenantId for x in role_refs])
-
-    def test_roles_get_by_user(self):
-        client = self.get_client(admin=True)
-        roles = client.roles.get_user_role_refs(user_id='foo')
-        self.assertTrue(len(roles) > 0)
-
-    def test_role_list_404(self):
-        self.skipTest('N/A')
-
-    def test_authenticate_and_delete_token(self):
-        self.skipTest('N/A')
-
-    def test_user_create_update_delete(self):
-        from keystoneclient import exceptions as client_exceptions
-
-        test_username = 'new_user'
-        client = self.get_client(admin=True)
-        user = client.users.create(name=test_username,
-                                   password='password',
-                                   email='user1@test.com')
-        self.assertEquals(user.name, test_username)
-
-        user = client.users.get(user=user.id)
-        self.assertEquals(user.name, test_username)
-
-        user = client.users.update_email(user=user, email='user2@test.com')
-        self.assertEquals(user.email, 'user2@test.com')
-
-        # NOTE(termie): update_enabled doesn't return anything, probably a bug
-        client.users.update_enabled(user=user, enabled=False)
-        user = client.users.get(user.id)
-        self.assertFalse(user.enabled)
-
-        self.assertRaises(client_exceptions.Unauthorized,
-                          self._client,
-                          username=test_username,
-                          password='password')
-        client.users.update_enabled(user, True)
-
-        user = client.users.update_password(user=user, password='password2')
-
-        self._client(username=test_username,
-                     password='password2')
-
-        user = client.users.update_tenant(user=user, tenant='bar')
-        # TODO(ja): once keystonelight supports default tenant
-        #           when you login without specifying tenant, the
-        #           token should be scoped to tenant 'bar'
-
-        client.users.delete(user.id)
-        self.assertRaises(client_exceptions.NotFound, client.users.get,
-                          user.id)
-
-    def test_user_update_404(self):
-        self.skipTest('N/A')
-
-    def test_endpoint_create_404(self):
-        self.skipTest('N/A')
-
-    def test_endpoint_delete_404(self):
-        self.skipTest('N/A')
-
-    def test_policy_crud(self):
-        self.skipTest('N/A due to lack of endpoint CRUD')
-
-    def test_disable_tenant_invalidates_token(self):
-        self.skipTest('N/A')
-
-    def test_delete_tenant_invalidates_token(self):
-        self.skipTest('N/A')
+class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
+    checkout_info = (KEYSTONECLIENT_REPO, 'master')
 
 
-class Kc11TestCase(CompatTestCase, KeystoneClientTests):
-    def get_checkout(self):
-        return KEYSTONECLIENT_REPO, '0.1.1'
+class KcOptTestCase(KcMasterTestCase):
+    # Set KSCTEST_PATH to the keystoneclient directory, then run this test.
+    #
+    # For example, to test your local keystoneclient,
+    #
+    # KSCTEST_PATH=/opt/stack/python-keystoneclient \
+    #  tox -e py27 test_keystoneclient.KcOptTestCase
 
-    def test_policy_crud(self):
-        self.skipTest('N/A')
+    def setUp(self):
+        self.checkout_info = os.environ.get('KSCTEST_PATH')
+        if not self.checkout_info:
+            self.skip('Set KSCTEST_PATH env to test with local client')
+        super(KcOptTestCase, self).setUp()
