@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2011 Piston Cloud Computing, Inc.
 # All Rights Reserved.
 
@@ -15,12 +13,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import StringIO
+import json
 import tempfile
-import urllib2
+
+import six
+from six.moves.urllib import request as urlrequest
+from testtools import matchers
 
 from keystone import config
 from keystone import exception
+from keystone.openstack.common.fixture import moxstubout
 from keystone.openstack.common import policy as common_policy
 from keystone.policy.backends import rules
 from keystone import tests
@@ -31,17 +33,19 @@ CONF = config.CONF
 
 class PolicyFileTestCase(tests.TestCase):
     def setUp(self):
-        super(PolicyFileTestCase, self).setUp()
-        self.orig_policy_file = CONF.policy_file
-        rules.reset()
+        # self.tmpfilename should exist before setUp super is called
+        # this is to ensure it is available for the config_fixture in
+        # the config_overrides call.
         _unused, self.tmpfilename = tempfile.mkstemp()
-        self.opt(policy_file=self.tmpfilename)
+        super(PolicyFileTestCase, self).setUp()
+
+        rules.reset()
+        self.addCleanup(rules.reset)
         self.target = {}
 
-    def tearDown(self):
-        super(PolicyFileTestCase, self).tearDown()
-        rules.reset()
-        self.opt(policy_file=self.orig_policy_file)
+    def config_overrides(self):
+        super(PolicyFileTestCase, self).config_overrides()
+        self.config_fixture.config(policy_file=self.tmpfilename)
 
     def test_modified_policy_reloads(self):
         action = "example:test"
@@ -61,6 +65,7 @@ class PolicyTestCase(tests.TestCase):
     def setUp(self):
         super(PolicyTestCase, self).setUp()
         rules.reset()
+        self.addCleanup(rules.reset)
         # NOTE(vish): preload rules to circumvent reloading from file
         rules.init()
         self.rules = {
@@ -81,16 +86,15 @@ class PolicyTestCase(tests.TestCase):
         self.credentials = {}
         self.target = {}
 
+        fixture = self.useFixture(moxstubout.MoxStubout())
+        self.stubs = fixture.stubs
+
     def _set_rules(self):
         these_rules = common_policy.Rules(
             dict((k, common_policy.parse_rule(v))
                  for k, v in self.rules.items()))
         rules._ENFORCER.set_rules(these_rules)
 
-    def tearDown(self):
-        rules.reset()
-        super(PolicyTestCase, self).tearDown()
-
     def test_enforce_nonexistent_action_throws(self):
         action = "example:noexist"
         self.assertRaises(exception.ForbiddenAction, rules.enforce,
@@ -108,9 +112,9 @@ class PolicyTestCase(tests.TestCase):
     def test_enforce_http_true(self):
 
         def fakeurlopen(url, post_data):
-            return StringIO.StringIO("True")
+            return six.StringIO("True")
 
-        self.stubs.Set(urllib2, 'urlopen', fakeurlopen)
+        self.stubs.Set(urlrequest, 'urlopen', fakeurlopen)
         action = "example:get_http"
         target = {}
         result = rules.enforce(self.credentials, action, target)
@@ -119,8 +123,8 @@ class PolicyTestCase(tests.TestCase):
     def test_enforce_http_false(self):
 
         def fakeurlopen(url, post_data):
-            return StringIO.StringIO("False")
-        self.stubs.Set(urllib2, 'urlopen', fakeurlopen)
+            return six.StringIO("False")
+        self.stubs.Set(urlrequest, 'urlopen', fakeurlopen)
         action = "example:get_http"
         target = {}
         self.assertRaises(exception.ForbiddenAction, rules.enforce,
@@ -158,6 +162,7 @@ class DefaultPolicyTestCase(tests.TestCase):
     def setUp(self):
         super(DefaultPolicyTestCase, self).setUp()
         rules.reset()
+        self.addCleanup(rules.reset)
         rules.init()
 
         self.rules = {
@@ -174,6 +179,8 @@ class DefaultPolicyTestCase(tests.TestCase):
         # Oslo policy as we shoudn't have to reload the rules if they have
         # already been set using set_rules().
         self._old_load_rules = rules._ENFORCER.load_rules
+        self.addCleanup(setattr, rules._ENFORCER, 'load_rules',
+                        self._old_load_rules)
         setattr(rules._ENFORCER, 'load_rules', lambda *args, **kwargs: None)
 
     def _set_rules(self, default_rule):
@@ -182,11 +189,6 @@ class DefaultPolicyTestCase(tests.TestCase):
                  for k, v in self.rules.items()), default_rule)
         rules._ENFORCER.set_rules(these_rules)
 
-    def tearDown(self):
-        super(DefaultPolicyTestCase, self).tearDown()
-        rules._ENFORCER.load_rules = self._old_load_rules
-        rules.reset()
-
     def test_policy_called(self):
         self.assertRaises(exception.ForbiddenAction, rules.enforce,
                           self.credentials, "example:exist", {})
@@ -204,3 +206,18 @@ class DefaultPolicyTestCase(tests.TestCase):
         self._set_rules(new_default_rule)
         self.assertRaises(exception.ForbiddenAction, rules.enforce,
                           self.credentials, "example:noexist", {})
+
+
+class PolicyJsonTestCase(tests.TestCase):
+
+    def _load_entries(self, filename):
+        return set(json.load(file(filename)))
+
+    def test_json_examples_have_matching_entries(self):
+        policy_keys = self._load_entries(tests.dirs.etc('policy.json'))
+        cloud_policy_keys = self._load_entries(
+            tests.dirs.etc('policy.v3cloudsample.json'))
+
+        diffs = set(policy_keys).difference(set(cloud_policy_keys))
+
+        self.assertThat(diffs, matchers.Equals(set()))
