@@ -13,11 +13,9 @@
 # under the License.
 
 from keystone.common import wsgi
-from keystone import config
 from keystone import exception
 from keystone import tests
 
-CONF = config.CONF
 
 KERBEROS_BIND = 'USER@REALM'
 
@@ -49,12 +47,13 @@ class BindTest(tests.TestCase):
                 self.assert_kerberos_bind(tokens, bind_level,
                                           use_kerberos=val, success=success)
         else:
-            context = {}
-            CONF.token.enforce_token_bind = bind_level
+            context = {'environment': {}}
+            self.config_fixture.config(group='token',
+                                       enforce_token_bind=bind_level)
 
             if use_kerberos:
-                context['REMOTE_USER'] = KERBEROS_BIND
-                context['AUTH_TYPE'] = 'Negotiate'
+                context['environment']['REMOTE_USER'] = KERBEROS_BIND
+                context['environment']['AUTH_TYPE'] = 'Negotiate'
 
             if not success:
                 self.assertRaises(exception.Unauthorized,
