@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,14 +14,18 @@
 
 import datetime
 
+from keystone import config
 from keystone import exception
 from keystone.openstack.common import timeutils
 from keystone import tests
 from keystone.tests import default_fixtures
 from keystone import token
+from keystone.token.providers import pki
 
 
-FUTURE_DELTA = datetime.timedelta(seconds=86400)
+CONF = config.CONF
+
+FUTURE_DELTA = datetime.timedelta(seconds=CONF.token.expiration)
 CURRENT_DATE = timeutils.utcnow()
 
 SAMPLE_V2_TOKEN = {
@@ -92,10 +94,10 @@ SAMPLE_V2_TOKEN = {
             {
                 "endpoints": [
                     {
-                        "adminURL": "http://localhost:8888/v1",
+                        "adminURL": "http://localhost:8080/v1",
                         "id": "7bd0c643e05a4a2ab40902b2fa0dd4e6",
-                        "internalURL": "http://localhost:8888/v1/AUTH_01257",
-                        "publicURL": "http://localhost:8888/v1/AUTH_01257",
+                        "internalURL": "http://localhost:8080/v1/AUTH_01257",
+                        "publicURL": "http://localhost:8080/v1/AUTH_01257",
                         "region": "RegionOne"
                     }
                 ],
@@ -202,19 +204,19 @@ SAMPLE_V3_TOKEN = {
                         "id": "7bd0c643e05a4a2ab40902b2fa0dd4e6",
                         "interface": "admin",
                         "region": "RegionOne",
-                        "url": "http://localhost:8888/v1"
+                        "url": "http://localhost:8080/v1"
                     },
                     {
                         "id": "43bef154594d4ccb8e49014d20624e1d",
                         "interface": "internal",
                         "region": "RegionOne",
-                        "url": "http://localhost:8888/v1/AUTH_01257"
+                        "url": "http://localhost:8080/v1/AUTH_01257"
                     },
                     {
                         "id": "e63b5f5d7aa3493690189d0ff843b9b3",
                         "interface": "public",
                         "region": "RegionOne",
-                        "url": "http://localhost:8888/v1/AUTH_01257"
+                        "url": "http://localhost:8080/v1/AUTH_01257"
                     }
                 ],
                 "id": "a669e152f1104810a4b6701aade721bb",
@@ -399,10 +401,10 @@ SAMPLE_V2_TOKEN_WITH_EMBEDED_VERSION = {
             {
                 "endpoints": [
                     {
-                        "adminURL": "http://localhost:8888/v1",
+                        "adminURL": "http://localhost:8080/v1",
                         "id": "7bd0c643e05a4a2ab40902b2fa0dd4e6",
-                        "internalURL": "http://localhost:8888/v1/AUTH_01257",
-                        "publicURL": "http://localhost:8888/v1/AUTH_01257",
+                        "internalURL": "http://localhost:8080/v1/AUTH_01257",
+                        "publicURL": "http://localhost:8080/v1/AUTH_01257",
                         "region": "RegionOne"
                     }
                 ],
@@ -509,19 +511,19 @@ SAMPLE_V3_TOKEN_WITH_EMBEDED_VERSION = {
                         "id": "7bd0c643e05a4a2ab40902b2fa0dd4e6",
                         "interface": "admin",
                         "region": "RegionOne",
-                        "url": "http://localhost:8888/v1"
+                        "url": "http://localhost:8080/v1"
                     },
                     {
                         "id": "43bef154594d4ccb8e49014d20624e1d",
                         "interface": "internal",
                         "region": "RegionOne",
-                        "url": "http://localhost:8888/v1/AUTH_01257"
+                        "url": "http://localhost:8080/v1/AUTH_01257"
                     },
                     {
                         "id": "e63b5f5d7aa3493690189d0ff843b9b3",
                         "interface": "public",
                         "region": "RegionOne",
-                        "url": "http://localhost:8888/v1/AUTH_01257"
+                        "url": "http://localhost:8080/v1/AUTH_01257"
                     }
                 ],
                 "id": "a669e152f1104810a4b6701aade721bb",
@@ -641,19 +643,22 @@ SAMPLE_V3_TOKEN_WITH_EMBEDED_VERSION = {
     'token_version': 'v3.0'
 }
 
-SAMPLE_V2_TOKEN_VALID = {
-    "access": {
-        "token": {
-            "expires": timeutils.isotime(CURRENT_DATE + FUTURE_DELTA),
-            "issued_at": "2013-05-21T00:02:43.941473Z",
-            "tenant": {
-                "enabled": True,
-                "id": "01257",
-                "name": "service"
+
+def create_v2_token():
+    return {
+        "access": {
+            "token": {
+                "expires": timeutils.isotime(CURRENT_DATE + FUTURE_DELTA),
+                "issued_at": "2013-05-21T00:02:43.941473Z",
+                "tenant": {
+                    "enabled": True,
+                    "id": "01257",
+                    "name": "service"
+                }
             }
         }
     }
-}
+
 
 SAMPLE_V2_TOKEN_EXPIRED = {
     "access": {
@@ -669,12 +674,16 @@ SAMPLE_V2_TOKEN_EXPIRED = {
     }
 }
 
-SAMPLE_V3_TOKEN_VALID = {
-    "token": {
-        "expires_at": timeutils.isotime(CURRENT_DATE + FUTURE_DELTA),
-        "issued_at": "2013-05-21T00:02:43.941473Z",
+
+def create_v3_token():
+    return {
+        "token": {
+            'methods': [],
+            "expires_at": timeutils.isotime(CURRENT_DATE + FUTURE_DELTA),
+            "issued_at": "2013-05-21T00:02:43.941473Z",
+        }
     }
-}
+
 
 SAMPLE_V3_TOKEN_EXPIRED = {
     "token": {
@@ -717,9 +726,9 @@ class TestTokenProvider(tests.TestCase):
                           'bogus')
 
     def test_token_format_provider_mismatch(self):
-        self.opt_in_group('signing', token_format='UUID')
-        self.opt_in_group('token',
-                          provider=token.provider.PKI_PROVIDER)
+        self.config_fixture.config(group='signing', token_format='UUID')
+        self.config_fixture.config(group='token',
+                                   provider=token.provider.PKI_PROVIDER)
         try:
             token.provider.Manager()
             raise Exception(
@@ -727,9 +736,9 @@ class TestTokenProvider(tests.TestCase):
         except exception.UnexpectedError:
             pass
 
-        self.opt_in_group('signing', token_format='PKI')
-        self.opt_in_group('token',
-                          provider=token.provider.UUID_PROVIDER)
+        self.config_fixture.config(group='signing', token_format='PKI')
+        self.config_fixture.config(group='token',
+                                   provider=token.provider.UUID_PROVIDER)
         try:
             token.provider.Manager()
             raise Exception(
@@ -738,14 +747,14 @@ class TestTokenProvider(tests.TestCase):
             pass
 
         # should be OK as token_format and provider aligns
-        self.opt_in_group('signing', token_format='PKI')
-        self.opt_in_group('token',
-                          provider=token.provider.PKI_PROVIDER)
+        self.config_fixture.config(group='signing', token_format='PKI')
+        self.config_fixture.config(group='token',
+                                   provider=token.provider.PKI_PROVIDER)
         token.provider.Manager()
 
-        self.opt_in_group('signing', token_format='UUID')
-        self.opt_in_group('token',
-                          provider=token.provider.UUID_PROVIDER)
+        self.config_fixture.config(group='signing', token_format='UUID')
+        self.config_fixture.config(group='token',
+                                   provider=token.provider.UUID_PROVIDER)
         token.provider.Manager()
 
     def test_default_token_format(self):
@@ -753,50 +762,52 @@ class TestTokenProvider(tests.TestCase):
                          token.provider.PKI_PROVIDER)
 
     def test_uuid_token_format_and_no_provider(self):
-        self.opt_in_group('signing', token_format='UUID')
+        self.config_fixture.config(group='signing', token_format='UUID')
         self.assertEqual(token.provider.Manager.get_token_provider(),
                          token.provider.UUID_PROVIDER)
 
     def test_default_providers_without_token_format(self):
-        self.opt_in_group('token',
-                          provider=token.provider.UUID_PROVIDER)
+        self.config_fixture.config(group='token',
+                                   provider=token.provider.UUID_PROVIDER)
         token.provider.Manager()
 
-        self.opt_in_group('token',
-                          provider=token.provider.PKI_PROVIDER)
+        self.config_fixture.config(group='token',
+                                   provider=token.provider.PKI_PROVIDER)
         token.provider.Manager()
 
     def test_unsupported_token_format(self):
-        self.opt_in_group('signing', token_format='CUSTOM')
+        self.config_fixture.config(group='signing', token_format='CUSTOM')
         self.assertRaises(exception.UnexpectedError,
                           token.provider.Manager.get_token_provider)
 
     def test_uuid_provider(self):
-        self.opt_in_group('token', provider=token.provider.UUID_PROVIDER)
+        self.config_fixture.config(group='token',
+                                   provider=token.provider.UUID_PROVIDER)
         self.assertEqual(token.provider.Manager.get_token_provider(),
                          token.provider.UUID_PROVIDER)
 
     def test_provider_override_token_format(self):
-        self.opt_in_group('token',
-                          provider='keystone.token.providers.pki.Test')
+        self.config_fixture.config(
+            group='token',
+            provider='keystone.token.providers.pki.Test')
         self.assertEqual(token.provider.Manager.get_token_provider(),
                          'keystone.token.providers.pki.Test')
 
-        self.opt_in_group('signing', token_format='UUID')
-        self.opt_in_group('token',
-                          provider=token.provider.UUID_PROVIDER)
+        self.config_fixture.config(group='signing', token_format='UUID')
+        self.config_fixture.config(group='token',
+                                   provider=token.provider.UUID_PROVIDER)
         self.assertEqual(token.provider.Manager.get_token_provider(),
                          token.provider.UUID_PROVIDER)
 
-        self.opt_in_group('signing', token_format='PKI')
-        self.opt_in_group('token',
-                          provider=token.provider.PKI_PROVIDER)
+        self.config_fixture.config(group='signing', token_format='PKI')
+        self.config_fixture.config(group='token',
+                                   provider=token.provider.PKI_PROVIDER)
         self.assertEqual(token.provider.Manager.get_token_provider(),
                          token.provider.PKI_PROVIDER)
 
-        self.opt_in_group('signing', token_format='CUSTOM')
-        self.opt_in_group('token',
-                          provider='my.package.MyProvider')
+        self.config_fixture.config(group='signing', token_format='CUSTOM')
+        self.config_fixture.config(group='token',
+                                   provider='my.package.MyProvider')
         self.assertEqual(token.provider.Manager.get_token_provider(),
                          'my.package.MyProvider')
 
@@ -812,16 +823,71 @@ class TestTokenProvider(tests.TestCase):
                           SAMPLE_MALFORMED_TOKEN)
         self.assertEqual(
             None,
-            self.token_provider_api._is_valid_token(SAMPLE_V2_TOKEN_VALID))
+            self.token_provider_api._is_valid_token(create_v2_token()))
         self.assertEqual(
             None,
-            self.token_provider_api._is_valid_token(SAMPLE_V3_TOKEN_VALID))
+            self.token_provider_api._is_valid_token(create_v3_token()))
 
     def test_uuid_provider_no_oauth_fails_oauth(self):
         self.load_fixtures(default_fixtures)
-        self.opt_in_group('token', provider=token.provider.UUID_PROVIDER)
+        self.config_fixture.config(group='token',
+                                   provider=token.provider.UUID_PROVIDER)
         driver = token.provider.Manager().driver
         driver.oauth_api = None
         self.assertRaises(exception.Forbidden,
                           driver.issue_v3_token,
                           self.user_foo['id'], ['oauth1'])
+
+
+class TestPKIProvider(object):
+
+    def setUp(self):
+        super(TestPKIProvider, self).setUp()
+
+        from keystoneclient.common import cms
+        self.cms = cms
+
+        from keystone.common import environment
+        self.environment = environment
+
+        old_cms_subprocess = cms.subprocess
+        self.addCleanup(setattr, cms, 'subprocess', old_cms_subprocess)
+
+        old_env_subprocess = environment.subprocess
+        self.addCleanup(setattr, environment, 'subprocess', old_env_subprocess)
+
+        self.cms.subprocess = self.target_subprocess
+        self.environment.subprocess = self.target_subprocess
+
+        reload(pki)  # force module reload so the imports get re-evaluated
+
+    def test_get_token_id_error_handling(self):
+        # cause command-line failure
+        self.config_fixture.config(group='signing',
+                                   keyfile='--please-break-me')
+
+        provider = pki.Provider()
+        token_data = {}
+        self.assertRaises(exception.UnexpectedError,
+                          provider._get_token_id,
+                          token_data)
+
+
+class TestPKIProviderWithEventlet(TestPKIProvider, tests.TestCase):
+
+    def setUp(self):
+        # force keystoneclient.common.cms to use eventlet's subprocess
+        from eventlet.green import subprocess
+        self.target_subprocess = subprocess
+
+        super(TestPKIProviderWithEventlet, self).setUp()
+
+
+class TestPKIProviderWithStdlib(TestPKIProvider, tests.TestCase):
+
+    def setUp(self):
+        # force keystoneclient.common.cms to use the stdlib subprocess
+        import subprocess
+        self.target_subprocess = subprocess
+
+        super(TestPKIProviderWithStdlib, self).setUp()
