@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -18,39 +16,38 @@ import datetime
 import uuid
 
 from lxml import etree
-import webtest
+import six
+from testtools import matchers
 
 from keystone import auth
+from keystone.common import authorization
 from keystone.common import cache
 from keystone.common import serializer
 from keystone import config
+from keystone import middleware
 from keystone.openstack.common import timeutils
 from keystone.policy.backends import rules
 from keystone import tests
-from keystone.tests import test_content_types
+from keystone.tests import rest
 
 
 CONF = config.CONF
-DEFAULT_DOMAIN_ID = CONF.identity.default_domain_id
+DEFAULT_DOMAIN_ID = 'default'
 
 TIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
 
 
-class RestfulTestCase(test_content_types.RestfulTestCase):
-    _config_file_list = [tests.etcdir('keystone.conf.sample'),
-                         tests.testsdir('test_overrides.conf'),
-                         tests.testsdir('backend_sql.conf'),
-                         tests.testsdir('backend_sql_disk.conf')]
-
-    #override this to sepcify the complete list of configuration files
+class RestfulTestCase(tests.SQLDriverOverrides, rest.RestfulTestCase):
     def config_files(self):
-        return self._config_file_list
+        config_files = super(RestfulTestCase, self).config_files()
+        config_files.append(tests.dirs.tests_conf('backend_sql.conf'))
+        return config_files
 
     def setup_database(self):
-        tests.setup_test_database()
+        tests.setup_database()
 
     def teardown_database(self):
-        tests.teardown_test_database()
+        tests.teardown_database()
 
     def generate_paste_config(self):
         new_paste_file = None
@@ -69,109 +66,102 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
         except AttributeError:
             pass
 
-    def setUp(self, load_sample_data=True, app_conf='keystone'):
+    def setUp(self, app_conf='keystone'):
         """Setup for v3 Restful Test Cases.
 
-        If a child class wants to create their own sample data
-        and provide their own auth data to obtain tokens, then
-        load_sample_data should be set to false.
-
         """
-        super(RestfulTestCase, self).setUp()
-        self.config(self.config_files())
-
-        self.setup_database()
-
         new_paste_file = self.generate_paste_config()
+        self.addCleanup(self.remove_generated_paste_config)
         if new_paste_file:
             app_conf = 'config:%s' % (new_paste_file)
 
+        super(RestfulTestCase, self).setUp(app_conf=app_conf)
+
+        self.empty_context = {'environment': {}}
+
+        #drop the policy rules
+        self.addCleanup(rules.reset)
+
+        self.addCleanup(self.teardown_database)
+
+    def load_backends(self):
+        self.setup_database()
+
         # ensure the cache region instance is setup
         cache.configure_cache_region(cache.REGION)
 
-        self.load_backends()
+        super(RestfulTestCase, self).load_backends()
 
-        self.public_app = webtest.TestApp(
-            self.loadapp(app_conf, name='main'))
-        self.admin_app = webtest.TestApp(
-            self.loadapp(app_conf, name='admin'))
+    def load_fixtures(self, fixtures):
+        self.load_sample_data()
 
-        if load_sample_data:
-            self.domain_id = uuid.uuid4().hex
-            self.domain = self.new_domain_ref()
-            self.domain['id'] = self.domain_id
-            self.identity_api.create_domain(self.domain_id, self.domain)
+    def load_sample_data(self):
+        self.domain_id = uuid.uuid4().hex
+        self.domain = self.new_domain_ref()
+        self.domain['id'] = self.domain_id
+        self.assignment_api.create_domain(self.domain_id, self.domain)
 
-            self.project_id = uuid.uuid4().hex
-            self.project = self.new_project_ref(
-                domain_id=self.domain_id)
-            self.project['id'] = self.project_id
-            self.assignment_api.create_project(self.project_id, self.project)
+        self.project_id = uuid.uuid4().hex
+        self.project = self.new_project_ref(
+            domain_id=self.domain_id)
+        self.project['id'] = self.project_id
+        self.assignment_api.create_project(self.project_id, self.project)
 
-            self.user_id = uuid.uuid4().hex
-            self.user = self.new_user_ref(domain_id=self.domain_id)
-            self.user['id'] = self.user_id
-            self.identity_api.create_user(self.user_id, self.user)
+        self.user_id = uuid.uuid4().hex
+        self.user = self.new_user_ref(domain_id=self.domain_id)
+        self.user['id'] = self.user_id
+        self.identity_api.create_user(self.user_id, self.user)
 
-            self.default_domain_project_id = uuid.uuid4().hex
-            self.default_domain_project = self.new_project_ref(
-                domain_id=DEFAULT_DOMAIN_ID)
-            self.default_domain_project['id'] = self.default_domain_project_id
-            self.assignment_api.create_project(self.default_domain_project_id,
-                                               self.default_domain_project)
+        self.default_domain_project_id = uuid.uuid4().hex
+        self.default_domain_project = self.new_project_ref(
+            domain_id=DEFAULT_DOMAIN_ID)
+        self.default_domain_project['id'] = self.default_domain_project_id
+        self.assignment_api.create_project(self.default_domain_project_id,
+                                           self.default_domain_project)
 
-            self.default_domain_user_id = uuid.uuid4().hex
-            self.default_domain_user = self.new_user_ref(
-                domain_id=DEFAULT_DOMAIN_ID)
-            self.default_domain_user['id'] = self.default_domain_user_id
-            self.identity_api.create_user(self.default_domain_user_id,
-                                          self.default_domain_user)
+        self.default_domain_user_id = uuid.uuid4().hex
+        self.default_domain_user = self.new_user_ref(
+            domain_id=DEFAULT_DOMAIN_ID)
+        self.default_domain_user['id'] = self.default_domain_user_id
+        self.identity_api.create_user(self.default_domain_user_id,
+                                      self.default_domain_user)
 
-            # create & grant policy.json's default role for admin_required
-            self.role_id = uuid.uuid4().hex
-            self.role = self.new_role_ref()
-            self.role['id'] = self.role_id
-            self.role['name'] = 'admin'
-            self.identity_api.create_role(self.role_id, self.role)
-            self.identity_api.add_role_to_user_and_project(
-                self.user_id, self.project_id, self.role_id)
-            self.identity_api.add_role_to_user_and_project(
-                self.default_domain_user_id, self.default_domain_project_id,
-                self.role_id)
-            self.identity_api.add_role_to_user_and_project(
-                self.default_domain_user_id, self.project_id,
-                self.role_id)
+        # create & grant policy.json's default role for admin_required
+        self.role_id = uuid.uuid4().hex
+        self.role = self.new_role_ref()
+        self.role['id'] = self.role_id
+        self.role['name'] = 'admin'
+        self.assignment_api.create_role(self.role_id, self.role)
+        self.assignment_api.add_role_to_user_and_project(
+            self.user_id, self.project_id, self.role_id)
+        self.assignment_api.add_role_to_user_and_project(
+            self.default_domain_user_id, self.default_domain_project_id,
+            self.role_id)
+        self.assignment_api.add_role_to_user_and_project(
+            self.default_domain_user_id, self.project_id,
+            self.role_id)
 
-            self.service_id = uuid.uuid4().hex
-            self.service = self.new_service_ref()
-            self.service['id'] = self.service_id
-            self.catalog_api.create_service(
-                self.service_id,
-                self.service.copy())
+        self.region_id = uuid.uuid4().hex
+        self.region = self.new_region_ref()
+        self.region['id'] = self.region_id
+        self.catalog_api.create_region(
+            self.region.copy())
 
-            self.endpoint_id = uuid.uuid4().hex
-            self.endpoint = self.new_endpoint_ref(service_id=self.service_id)
-            self.endpoint['id'] = self.endpoint_id
-            self.catalog_api.create_endpoint(
-                self.endpoint_id,
-                self.endpoint.copy())
+        self.service_id = uuid.uuid4().hex
+        self.service = self.new_service_ref()
+        self.service['id'] = self.service_id
+        self.catalog_api.create_service(
+            self.service_id,
+            self.service.copy())
 
-        self.public_server = self.serveapp(app_conf, name='main')
-        self.admin_server = self.serveapp(app_conf, name='admin')
-
-    def tearDown(self):
-        self.public_server.kill()
-        self.admin_server.kill()
-        self.public_server = None
-        self.admin_server = None
-        self.teardown_database()
-        self.remove_generated_paste_config()
-        # need to reset the plug-ins
-        auth.controllers.AUTH_METHODS = {}
-        #drop the policy rules
-        CONF.reset()
-        rules.reset()
-        super(RestfulTestCase, self).tearDown()
+        self.endpoint_id = uuid.uuid4().hex
+        self.endpoint = self.new_endpoint_ref(service_id=self.service_id)
+        self.endpoint['id'] = self.endpoint_id
+        self.catalog_api.create_endpoint(
+            self.endpoint_id,
+            self.endpoint.copy())
+        # The server adds 'enabled' and defaults to True.
+        self.endpoint['enabled'] = True
 
     def new_ref(self):
         """Populates a ref with attributes common to all API entities."""
@@ -181,17 +171,27 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
             'description': uuid.uuid4().hex,
             'enabled': True}
 
+    def new_region_ref(self):
+        ref = self.new_ref()
+        # Region doesn't have name or enabled.
+        del ref['name']
+        del ref['enabled']
+        ref['parent_region_id'] = None
+        return ref
+
     def new_service_ref(self):
         ref = self.new_ref()
         ref['type'] = uuid.uuid4().hex
         return ref
 
-    def new_endpoint_ref(self, service_id):
+    def new_endpoint_ref(self, service_id, **kwargs):
         ref = self.new_ref()
+        del ref['enabled']  # enabled is optional
         ref['interface'] = uuid.uuid4().hex[:8]
         ref['service_id'] = service_id
         ref['url'] = uuid.uuid4().hex
         ref['region'] = uuid.uuid4().hex
+        ref.update(kwargs)
         return ref
 
     def new_domain_ref(self):
@@ -238,15 +238,16 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
 
     def new_trust_ref(self, trustor_user_id, trustee_user_id, project_id=None,
                       impersonation=None, expires=None, role_ids=None,
-                      role_names=None):
+                      role_names=None, remaining_uses=None):
         ref = self.new_ref()
 
         ref['trustor_user_id'] = trustor_user_id
         ref['trustee_user_id'] = trustee_user_id
         ref['impersonation'] = impersonation or False
         ref['project_id'] = project_id
+        ref['remaining_uses'] = remaining_uses
 
-        if isinstance(expires, basestring):
+        if isinstance(expires, six.string_types):
             ref['expires_at'] = expires
         elif isinstance(expires, dict):
             ref['expires_at'] = timeutils.strtime(
@@ -268,6 +269,21 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
 
         return ref
 
+    def create_new_default_project_for_user(self, user_id, domain_id,
+                                            enable_project=True):
+        ref = self.new_project_ref(domain_id=domain_id)
+        ref['enabled'] = enable_project
+        r = self.post('/projects', body={'project': ref})
+        project = self.assertValidProjectResponse(r, ref)
+        # set the user's preferred project
+        body = {'user': {'default_project_id': project['id']}}
+        r = self.patch('/users/%(user_id)s' % {
+            'user_id': user_id},
+            body=body)
+        self.assertValidUserResponse(r)
+
+        return project
+
     def admin_request(self, *args, **kwargs):
         """Translates XML responses to dicts.
 
@@ -319,9 +335,9 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
     def v3_request(self, path, **kwargs):
         # Check if the caller has passed in auth details for
         # use in requesting the token
-        auth = kwargs.pop('auth', None)
-        if auth:
-            token = self.get_requested_token(auth)
+        auth_arg = kwargs.pop('auth', None)
+        if auth_arg:
+            token = self.get_requested_token(auth_arg)
         else:
             token = kwargs.pop('token', None)
             if not token:
@@ -380,19 +396,17 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
     def assertValidListLinks(self, links):
         self.assertIsNotNone(links)
         self.assertIsNotNone(links.get('self'))
-        self.assertIn(CONF.public_endpoint % CONF, links['self'])
+        self.assertThat(links['self'], matchers.StartsWith('http://localhost'))
 
         self.assertIn('next', links)
         if links['next'] is not None:
-            self.assertIn(
-                CONF.public_endpoint % CONF,
-                links['next'])
+            self.assertThat(links['next'],
+                            matchers.StartsWith('http://localhost'))
 
         self.assertIn('previous', links)
         if links['previous'] is not None:
-            self.assertIn(
-                CONF.public_endpoint % CONF,
-                links['previous'])
+            self.assertThat(links['previous'],
+                            matchers.StartsWith('http://localhost'))
 
     def assertValidListResponse(self, resp, key, entity_validator, ref=None,
                                 expected_length=None, keys_to_check=None):
@@ -441,7 +455,7 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
         If a reference is provided, the entity will also be compared against
         the reference.
         """
-        if keys_to_check:
+        if keys_to_check is not None:
             keys = keys_to_check
         else:
             keys = ['name', 'description', 'enabled']
@@ -452,13 +466,14 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
 
         self.assertIsNotNone(entity.get('links'))
         self.assertIsNotNone(entity['links'].get('self'))
-        self.assertIn(CONF.public_endpoint % CONF, entity['links']['self'])
+        self.assertThat(entity['links']['self'],
+                        matchers.StartsWith('http://localhost'))
         self.assertIn(entity['id'], entity['links']['self'])
 
         if ref:
             for k in keys:
                 msg = '%s not equal: %s != %s' % (k, ref[k], entity[k])
-                self.assertEquals(ref[k], entity[k])
+                self.assertEqual(ref[k], entity[k])
 
         return entity
 
@@ -470,7 +485,7 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
         except Exception:
             msg = '%s is not a valid ISO 8601 extended format date time.' % dt
             raise AssertionError(msg)
-        self.assertTrue(isinstance(dt, datetime.datetime))
+        self.assertIsInstance(dt, datetime.datetime)
 
     def assertValidTokenResponse(self, r, user=None):
         self.assertTrue(r.headers.get('X-Subject-Token'))
@@ -515,6 +530,15 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
 
         if require_catalog:
             self.assertIn('catalog', token)
+
+            if isinstance(token['catalog'], list):
+                # only test JSON
+                for service in token['catalog']:
+                    for endpoint in service['endpoints']:
+                        self.assertNotIn('enabled', endpoint)
+                        self.assertNotIn('legacy_endpoint_id', endpoint)
+                        self.assertNotIn('service_id', endpoint)
+
             # sub test for the OS-EP-FILTER extension enabled
             if endpoint_filter:
                 # verify the catalog hs no more than the endpoints
@@ -551,7 +575,7 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
         trust = token.get('OS-TRUST:trust')
         self.assertIsNotNone(trust)
         self.assertIsNotNone(trust.get('id'))
-        self.assertTrue(isinstance(trust.get('impersonation'), bool))
+        self.assertIsInstance(trust.get('impersonation'), bool)
         self.assertIsNotNone(trust.get('trustor_user'))
         self.assertIsNotNone(trust.get('trustee_user'))
         self.assertIsNotNone(trust['trustor_user'].get('id'))
@@ -591,6 +615,40 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
 
         return self.assertDictEqual(normalize(a), normalize(b))
 
+    # region validation
+
+    def assertValidRegionListResponse(self, resp, *args, **kwargs):
+        #NOTE(jaypipes): I have to pass in a blank keys_to_check parameter
+        #                below otherwise the base assertValidEntity method
+        #                tries to find a "name" and an "enabled" key in the
+        #                returned ref dicts. The issue is, I don't understand
+        #                how the service and endpoint entity assertions below
+        #                actually work (they don't raise assertions), since
+        #                AFAICT, the service and endpoint tables don't have
+        #                a "name" column either... :(
+        return self.assertValidListResponse(
+            resp,
+            'regions',
+            self.assertValidRegion,
+            keys_to_check=[],
+            *args,
+            **kwargs)
+
+    def assertValidRegionResponse(self, resp, *args, **kwargs):
+        return self.assertValidResponse(
+            resp,
+            'region',
+            self.assertValidRegion,
+            keys_to_check=[],
+            *args,
+            **kwargs)
+
+    def assertValidRegion(self, entity, ref=None):
+        self.assertIsNotNone(entity.get('description'))
+        if ref:
+            self.assertEqual(ref['description'], entity['description'])
+        return entity
+
     # service validation
 
     def assertValidServiceListResponse(self, resp, *args, **kwargs):
@@ -611,6 +669,7 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
 
     def assertValidService(self, entity, ref=None):
         self.assertIsNotNone(entity.get('type'))
+        self.assertIsInstance(entity.get('enabled'), bool)
         if ref:
             self.assertEqual(ref['type'], entity['type'])
         return entity
@@ -636,6 +695,7 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
     def assertValidEndpoint(self, entity, ref=None):
         self.assertIsNotNone(entity.get('interface'))
         self.assertIsNotNone(entity.get('service_id'))
+        self.assertIsInstance(entity['enabled'], bool)
 
         # this is intended to be an unexposed implementation detail
         self.assertNotIn('legacy_endpoint_id', entity)
@@ -941,7 +1001,7 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
 
         if summary:
             # Trust list contains no roles, but getting a specific
-            # trust by ID provides the detailed reponse containing roles
+            # trust by ID provides the detailed response containing roles
             self.assertNotIn('roles', entity)
             self.assertIn('project_id', entity)
         else:
@@ -1043,7 +1103,63 @@ class RestfulTestCase(test_content_types.RestfulTestCase):
             auth_data['scope'] = self.build_auth_scope(**kwargs)
         return {'auth': auth_data}
 
+    def build_external_auth_request(self, remote_user,
+                                    remote_domain=None, auth_data=None):
+        context = {'environment': {'REMOTE_USER': remote_user}}
+        if remote_domain:
+            context['environment']['REMOTE_DOMAIN'] = remote_domain
+        if not auth_data:
+            auth_data = self.build_authentication_request()['auth']
+        no_context = None
+        auth_info = auth.controllers.AuthInfo.create(no_context, auth_data)
+        auth_context = {'extras': {}, 'method_names': []}
+        return context, auth_info, auth_context
+
 
 class VersionTestCase(RestfulTestCase):
     def test_get_version(self):
         pass
+
+
+#NOTE(gyee): test AuthContextMiddleware here instead of test_middleware.py
+# because we need the token
+class AuthContextMiddlewareTestCase(RestfulTestCase):
+    def _mock_request_object(self, token_id):
+
+        class fake_req:
+            headers = {middleware.AUTH_TOKEN_HEADER: token_id}
+            environ = {}
+
+        return fake_req()
+
+    def test_auth_context_build_by_middleware(self):
+        # test to make sure AuthContextMiddleware successful build the auth
+        # context from the incoming auth token
+        admin_token = self.get_scoped_token()
+        req = self._mock_request_object(admin_token)
+        application = None
+        middleware.AuthContextMiddleware(application).process_request(req)
+        self.assertEqual(
+            req.environ.get(authorization.AUTH_CONTEXT_ENV)['user_id'],
+            self.user['id'])
+
+    def test_auth_context_override(self):
+        overridden_context = 'OVERRIDDEN_CONTEXT'
+        # this token should not be used
+        token = uuid.uuid4().hex
+        req = self._mock_request_object(token)
+        req.environ[authorization.AUTH_CONTEXT_ENV] = overridden_context
+        application = None
+        middleware.AuthContextMiddleware(application).process_request(req)
+        # make sure overridden context take precedence
+        self.assertEqual(req.environ.get(authorization.AUTH_CONTEXT_ENV),
+                         overridden_context)
+
+    def test_admin_token_auth_context(self):
+        # test to make sure AuthContextMiddleware does not attempt to build
+        # auth context if the incoming auth token is the special admin token
+        req = self._mock_request_object(CONF.admin_token)
+        application = None
+        middleware.AuthContextMiddleware(application).process_request(req)
+        self.assertDictEqual(req.environ.get(authorization.AUTH_CONTEXT_ENV),
+                             {})
