@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,20 +13,19 @@
 # under the License.
 
 import copy
-import os
-import tempfile
-import urlparse
 import uuid
 
-from keystone.common import cms
-from keystone.common.sql import migration
+from six.moves import urllib
+
+from keystone.common import sql
+from keystone.common.sql import migration_helpers
 from keystone import config
 from keystone import contrib
 from keystone.contrib import oauth1
 from keystone.contrib.oauth1 import controllers
+from keystone import exception
+from keystone.openstack.common.db.sqlalchemy import migration
 from keystone.openstack.common import importutils
-from keystone.openstack.common import jsonutils
-from keystone.policy.backends import rules
 from keystone.tests import test_v3
 
 
@@ -38,75 +35,69 @@ CONF = config.CONF
 class OAuth1Tests(test_v3.RestfulTestCase):
 
     EXTENSION_NAME = 'oauth1'
-    EXTENSION_TO_ADD = 'oauth_extension'
+    EXTENSION_TO_ADD = 'oauth1_extension'
+
+    CONSUMER_URL = '/OS-OAUTH1/consumers'
 
     def setup_database(self):
         super(OAuth1Tests, self).setup_database()
-        package_name = "%s.%s.migrate_repo" % (contrib.__name__,
-                                               self.EXTENSION_NAME)
+        package_name = '.'.join((contrib.__name__, self.EXTENSION_NAME))
         package = importutils.import_module(package_name)
-        self.repo_path = os.path.abspath(os.path.dirname(package.__file__))
-        migration.db_version_control(version=None, repo_path=self.repo_path)
-        migration.db_sync(version=None, repo_path=self.repo_path)
+        abs_path = migration_helpers.find_migrate_repo(package)
+        migration.db_version_control(sql.get_engine(), abs_path)
+        migration.db_sync(sql.get_engine(), abs_path)
 
     def setUp(self):
         super(OAuth1Tests, self).setUp()
 
         # Now that the app has been served, we can query CONF values
-        self.base_url = (CONF.public_endpoint % CONF) + "v3"
+        self.base_url = 'http://localhost/v3'
         self.controller = controllers.OAuthControllerV3()
 
     def _create_single_consumer(self):
         ref = {'description': uuid.uuid4().hex}
         resp = self.post(
-            '/OS-OAUTH1/consumers',
+            self.CONSUMER_URL,
             body={'consumer': ref})
-        return resp.result.get('consumer')
-
-    def _oauth_request(self, consumer, token=None, **kw):
-        return oauth1.Request.from_consumer_and_token(consumer=consumer,
-                                                      token=token,
-                                                      **kw)
+        return resp.result['consumer']
 
     def _create_request_token(self, consumer, project_id):
-        params = {'requested_project_id': project_id}
-        headers = {'Content-Type': 'application/json'}
-        url = '/OS-OAUTH1/request_token'
-        oreq = self._oauth_request(
-            consumer=consumer,
-            http_url=self.base_url + url,
-            http_method='POST',
-            parameters=params)
-        hmac = oauth1.SignatureMethod_HMAC_SHA1()
-        oreq.sign_request(hmac, consumer, None)
-        headers.update(oreq.to_header())
-        headers.update(params)
-        return url, headers
+        endpoint = '/OS-OAUTH1/request_token'
+        client = oauth1.Client(consumer['key'],
+                               client_secret=consumer['secret'],
+                               signature_method=oauth1.SIG_HMAC,
+                               callback_uri="oob")
+        headers = {'requested_project_id': project_id}
+        url, headers, body = client.sign(self.base_url + endpoint,
+                                         http_method='POST',
+                                         headers=headers)
+        return endpoint, headers
 
     def _create_access_token(self, consumer, token):
-        headers = {'Content-Type': 'application/json'}
-        url = '/OS-OAUTH1/access_token'
-        oreq = self._oauth_request(
-            consumer=consumer, token=token,
-            http_method='POST',
-            http_url=self.base_url + url)
-        hmac = oauth1.SignatureMethod_HMAC_SHA1()
-        oreq.sign_request(hmac, consumer, token)
-        headers.update(oreq.to_header())
-        return url, headers
+        endpoint = '/OS-OAUTH1/access_token'
+        client = oauth1.Client(consumer['key'],
+                               client_secret=consumer['secret'],
+                               resource_owner_key=token.key,
+                               resource_owner_secret=token.secret,
+                               signature_method=oauth1.SIG_HMAC,
+                               verifier=token.verifier)
+        url, headers, body = client.sign(self.base_url + endpoint,
+                                         http_method='POST')
+        headers.update({'Content-Type': 'application/json'})
+        return endpoint, headers
 
     def _get_oauth_token(self, consumer, token):
-        headers = {'Content-Type': 'application/json'}
-        body = {'auth': {'identity': {'methods': ['oauth1'], 'oauth1': {}}}}
-        url = '/auth/tokens'
-        oreq = self._oauth_request(
-            consumer=consumer, token=token,
-            http_method='POST',
-            http_url=self.base_url + url)
-        hmac = oauth1.SignatureMethod_HMAC_SHA1()
-        oreq.sign_request(hmac, consumer, token)
-        headers.update(oreq.to_header())
-        return url, headers, body
+        client = oauth1.Client(consumer['key'],
+                               client_secret=consumer['secret'],
+                               resource_owner_key=token.key,
+                               resource_owner_secret=token.secret,
+                               signature_method=oauth1.SIG_HMAC)
+        endpoint = '/auth/tokens'
+        url, headers, body = client.sign(self.base_url + endpoint,
+                                         http_method='POST')
+        headers.update({'Content-Type': 'application/json'})
+        ref = {'auth': {'identity': {'oauth1': {}, 'methods': ['oauth1']}}}
+        return endpoint, headers, ref
 
     def _authorize_request_token(self, request_id):
         return '/OS-OAUTH1/authorize/%s' % (request_id)
@@ -114,19 +105,23 @@ class OAuth1Tests(test_v3.RestfulTestCase):
 
 class ConsumerCRUDTests(OAuth1Tests):
 
-    def _consumer_create(self, description=None, description_flag=True):
+    def _consumer_create(self, description=None, description_flag=True,
+                         **kwargs):
         if description_flag:
             ref = {'description': description}
         else:
             ref = {}
+        if kwargs:
+            ref.update(kwargs)
         resp = self.post(
-            '/OS-OAUTH1/consumers',
+            self.CONSUMER_URL,
             body={'consumer': ref})
-        consumer = resp.result.get('consumer')
-        consumer_id = consumer.get('id')
+        consumer = resp.result['consumer']
+        consumer_id = consumer['id']
         self.assertEqual(consumer['description'], description)
         self.assertIsNotNone(consumer_id)
-        self.assertIsNotNone(consumer.get('secret'))
+        self.assertIsNotNone(consumer['secret'])
+        return consumer
 
     def test_consumer_create(self):
         description = uuid.uuid4().hex
@@ -138,75 +133,116 @@ class ConsumerCRUDTests(OAuth1Tests):
     def test_consumer_create_none_desc_2(self):
         self._consumer_create(description_flag=False)
 
+    def test_consumer_create_normalize_field(self):
+        # If create a consumer with a field with : or - in the name,
+        # the name is normalized by converting those chars to _.
+        field_name = 'some:weird-field'
+        field_value = uuid.uuid4().hex
+        extra_fields = {field_name: field_value}
+        consumer = self._consumer_create(**extra_fields)
+        normalized_field_name = 'some_weird_field'
+        self.assertEqual(field_value, consumer[normalized_field_name])
+
     def test_consumer_delete(self):
         consumer = self._create_single_consumer()
-        consumer_id = consumer.get('id')
-        resp = self.delete('/OS-OAUTH1/consumers/%(consumer_id)s'
-                           % {'consumer_id': consumer_id})
+        consumer_id = consumer['id']
+        resp = self.delete(self.CONSUMER_URL + '/%s' % consumer_id)
         self.assertResponseStatus(resp, 204)
 
     def test_consumer_get(self):
         consumer = self._create_single_consumer()
-        consumer_id = consumer.get('id')
-        resp = self.get('/OS-OAUTH1/consumers/%(consumer_id)s'
-                        % {'consumer_id': consumer_id})
-        self.assertEqual(resp.result.get('consumer').get('id'), consumer_id)
+        consumer_id = consumer['id']
+        resp = self.get(self.CONSUMER_URL + '/%s' % consumer_id)
+        self_url = ['http://localhost/v3', self.CONSUMER_URL,
+                    '/', consumer_id]
+        self_url = ''.join(self_url)
+        self.assertEqual(resp.result['consumer']['links']['self'], self_url)
+        self.assertEqual(resp.result['consumer']['id'], consumer_id)
 
     def test_consumer_list(self):
-        resp = self.get('/OS-OAUTH1/consumers')
-        entities = resp.result.get('consumers')
+        self._consumer_create()
+        resp = self.get(self.CONSUMER_URL)
+        entities = resp.result['consumers']
         self.assertIsNotNone(entities)
-        self.assertValidListLinks(resp.result.get('links'))
+        self_url = ['http://localhost/v3', self.CONSUMER_URL]
+        self_url = ''.join(self_url)
+        self.assertEqual(resp.result['links']['self'], self_url)
+        self.assertValidListLinks(resp.result['links'])
 
     def test_consumer_update(self):
         consumer = self._create_single_consumer()
-        original_id = consumer.get('id')
-        original_description = consumer.get('description')
-        update_description = original_description + "_new"
+        original_id = consumer['id']
+        original_description = consumer['description']
+        update_description = original_description + '_new'
 
         update_ref = {'description': update_description}
-        update_resp = self.patch('/OS-OAUTH1/consumers/%(consumer_id)s'
-                                 % {'consumer_id': original_id},
+        update_resp = self.patch(self.CONSUMER_URL + '/%s' % original_id,
                                  body={'consumer': update_ref})
-        consumer = update_resp.result.get('consumer')
-        self.assertEqual(consumer.get('description'), update_description)
-        self.assertEqual(consumer.get('id'), original_id)
+        consumer = update_resp.result['consumer']
+        self.assertEqual(consumer['description'], update_description)
+        self.assertEqual(consumer['id'], original_id)
 
     def test_consumer_update_bad_secret(self):
         consumer = self._create_single_consumer()
-        original_id = consumer.get('id')
+        original_id = consumer['id']
         update_ref = copy.deepcopy(consumer)
         update_ref['description'] = uuid.uuid4().hex
         update_ref['secret'] = uuid.uuid4().hex
-        self.patch('/OS-OAUTH1/consumers/%(consumer_id)s'
-                   % {'consumer_id': original_id},
+        self.patch(self.CONSUMER_URL + '/%s' % original_id,
                    body={'consumer': update_ref},
                    expected_status=400)
 
     def test_consumer_update_bad_id(self):
         consumer = self._create_single_consumer()
-        original_id = consumer.get('id')
-        original_description = consumer.get('description')
+        original_id = consumer['id']
+        original_description = consumer['description']
         update_description = original_description + "_new"
 
         update_ref = copy.deepcopy(consumer)
         update_ref['description'] = update_description
         update_ref['id'] = update_description
-        self.patch('/OS-OAUTH1/consumers/%(consumer_id)s'
-                   % {'consumer_id': original_id},
+        self.patch(self.CONSUMER_URL + '/%s' % original_id,
                    body={'consumer': update_ref},
                    expected_status=400)
 
+    def test_consumer_update_normalize_field(self):
+        # If update a consumer with a field with : or - in the name,
+        # the name is normalized by converting those chars to _.
+        field1_name = 'some:weird-field'
+        field1_orig_value = uuid.uuid4().hex
+
+        extra_fields = {field1_name: field1_orig_value}
+        consumer = self._consumer_create(**extra_fields)
+        consumer_id = consumer['id']
+
+        field1_new_value = uuid.uuid4().hex
+
+        field2_name = 'weird:some-field'
+        field2_value = uuid.uuid4().hex
+
+        update_ref = {field1_name: field1_new_value,
+                      field2_name: field2_value}
+
+        update_resp = self.patch(self.CONSUMER_URL + '/%s' % consumer_id,
+                                 body={'consumer': update_ref})
+        consumer = update_resp.result['consumer']
+
+        normalized_field1_name = 'some_weird_field'
+        self.assertEqual(field1_new_value, consumer[normalized_field1_name])
+
+        normalized_field2_name = 'weird_some_field'
+        self.assertEqual(field2_value, consumer[normalized_field2_name])
+
     def test_consumer_create_no_description(self):
-        resp = self.post('/OS-OAUTH1/consumers', body={'consumer': {}})
-        consumer = resp.result.get('consumer')
-        consumer_id = consumer.get('id')
-        self.assertEqual(consumer.get('description'), None)
+        resp = self.post(self.CONSUMER_URL, body={'consumer': {}})
+        consumer = resp.result['consumer']
+        consumer_id = consumer['id']
+        self.assertIsNone(consumer['description'])
         self.assertIsNotNone(consumer_id)
-        self.assertIsNotNone(consumer.get('secret'))
+        self.assertIsNotNone(consumer['secret'])
 
     def test_consumer_get_bad_id(self):
-        self.get('/OS-OAUTH1/consumers/%(consumer_id)s'
+        self.get(self.CONSUMER_URL + '/%(consumer_id)s'
                  % {'consumer_id': uuid.uuid4().hex},
                  expected_status=404)
 
@@ -215,17 +251,17 @@ class OAuthFlowTests(OAuth1Tests):
 
     def test_oauth_flow(self):
         consumer = self._create_single_consumer()
-        consumer_id = consumer.get('id')
-        consumer_secret = consumer.get('secret')
-        self.consumer = oauth1.Consumer(consumer_id, consumer_secret)
-        self.assertIsNotNone(self.consumer.key)
+        consumer_id = consumer['id']
+        consumer_secret = consumer['secret']
+        self.consumer = {'key': consumer_id, 'secret': consumer_secret}
+        self.assertIsNotNone(self.consumer['secret'])
 
         url, headers = self._create_request_token(self.consumer,
                                                   self.project_id)
         content = self.post(url, headers=headers)
-        credentials = urlparse.parse_qs(content.result)
-        request_key = credentials.get('oauth_token')[0]
-        request_secret = credentials.get('oauth_token_secret')[0]
+        credentials = urllib.parse.parse_qs(content.result)
+        request_key = credentials['oauth_token'][0]
+        request_secret = credentials['oauth_token_secret'][0]
         self.request_token = oauth1.Token(request_key, request_secret)
         self.assertIsNotNone(self.request_token.key)
 
@@ -238,17 +274,17 @@ class OAuthFlowTests(OAuth1Tests):
         url, headers = self._create_access_token(self.consumer,
                                                  self.request_token)
         content = self.post(url, headers=headers)
-        credentials = urlparse.parse_qs(content.result)
-        access_key = credentials.get('oauth_token')[0]
-        access_secret = credentials.get('oauth_token_secret')[0]
+        credentials = urllib.parse.parse_qs(content.result)
+        access_key = credentials['oauth_token'][0]
+        access_secret = credentials['oauth_token_secret'][0]
         self.access_token = oauth1.Token(access_key, access_secret)
         self.assertIsNotNone(self.access_token.key)
 
         url, headers, body = self._get_oauth_token(self.consumer,
                                                    self.access_token)
         content = self.post(url, headers=headers, body=body)
-        self.keystone_token_id = content.headers.get('X-Subject-Token')
-        self.keystone_token = content.result.get('token')
+        self.keystone_token_id = content.headers['X-Subject-Token']
+        self.keystone_token = content.result['token']
         self.assertIsNotNone(self.keystone_token_id)
 
 
@@ -262,18 +298,18 @@ class AccessTokenCRUDTests(OAuthFlowTests):
     def test_list_no_access_tokens(self):
         resp = self.get('/users/%(user_id)s/OS-OAUTH1/access_tokens'
                         % {'user_id': self.user_id})
-        entities = resp.result.get('access_tokens')
-        self.assertTrue(len(entities) == 0)
-        self.assertValidListLinks(resp.result.get('links'))
+        entities = resp.result['access_tokens']
+        self.assertEqual([], entities)
+        self.assertValidListLinks(resp.result['links'])
 
     def test_get_single_access_token(self):
         self.test_oauth_flow()
         resp = self.get('/users/%(user_id)s/OS-OAUTH1/access_tokens/%(key)s'
                         % {'user_id': self.user_id,
                            'key': self.access_token.key})
-        entity = resp.result.get('access_token')
+        entity = resp.result['access_token']
         self.assertEqual(entity['id'], self.access_token.key)
-        self.assertEqual(entity['consumer_id'], self.consumer.key)
+        self.assertEqual(entity['consumer_id'], self.consumer['key'])
 
     def test_get_access_token_dne(self):
         self.get('/users/%(user_id)s/OS-OAUTH1/access_tokens/%(key)s'
@@ -286,9 +322,9 @@ class AccessTokenCRUDTests(OAuthFlowTests):
         resp = self.get('/users/%(id)s/OS-OAUTH1/access_tokens/%(key)s/roles'
                         % {'id': self.user_id,
                            'key': self.access_token.key})
-        entities = resp.result.get('roles')
-        self.assertTrue(len(entities) > 0)
-        self.assertValidListLinks(resp.result.get('links'))
+        entities = resp.result['roles']
+        self.assertTrue(entities)
+        self.assertValidListLinks(resp.result['links'])
 
     def test_get_role_in_access_token(self):
         self.test_oauth_flow()
@@ -296,7 +332,7 @@ class AccessTokenCRUDTests(OAuthFlowTests):
                % {'id': self.user_id, 'key': self.access_token.key,
                   'role': self.role_id})
         resp = self.get(url)
-        entity = resp.result.get('role')
+        entity = resp.result['role']
         self.assertEqual(entity['id'], self.role_id)
 
     def test_get_role_in_access_token_dne(self):
@@ -311,9 +347,9 @@ class AccessTokenCRUDTests(OAuthFlowTests):
         # List access_tokens should be > 0
         resp = self.get('/users/%(user_id)s/OS-OAUTH1/access_tokens'
                         % {'user_id': self.user_id})
-        entities = resp.result.get('access_tokens')
-        self.assertTrue(len(entities) > 0)
-        self.assertValidListLinks(resp.result.get('links'))
+        entities = resp.result['access_tokens']
+        self.assertTrue(entities)
+        self.assertValidListLinks(resp.result['links'])
 
         # Delete access_token
         resp = self.delete('/users/%(user)s/OS-OAUTH1/access_tokens/%(auth)s'
@@ -324,9 +360,9 @@ class AccessTokenCRUDTests(OAuthFlowTests):
         # List access_token should be 0
         resp = self.get('/users/%(user_id)s/OS-OAUTH1/access_tokens'
                         % {'user_id': self.user_id})
-        entities = resp.result.get('access_tokens')
-        self.assertTrue(len(entities) == 0)
-        self.assertValidListLinks(resp.result.get('links'))
+        entities = resp.result['access_tokens']
+        self.assertEqual([], entities)
+        self.assertValidListLinks(resp.result['links'])
 
 
 class AuthTokenTests(OAuthFlowTests):
@@ -340,14 +376,14 @@ class AuthTokenTests(OAuthFlowTests):
 
         # now verify the oauth section
         oauth_section = r.result['token']['OS-OAUTH1']
-        self.assertEquals(oauth_section['access_token_id'],
-                          self.access_token.key)
-        self.assertEquals(oauth_section['consumer_id'], self.consumer.key)
+        self.assertEqual(oauth_section['access_token_id'],
+                         self.access_token.key)
+        self.assertEqual(oauth_section['consumer_id'], self.consumer['key'])
 
         # verify the roles section
         roles_list = r.result['token']['roles']
         # we can just verify the 0th role since we are only assigning one role
-        self.assertEquals(roles_list[0]['id'], self.role_id)
+        self.assertEqual(roles_list[0]['id'], self.role_id)
 
         # verify that the token can perform delegated tasks
         ref = self.new_user_ref(domain_id=self.domain_id)
@@ -374,7 +410,7 @@ class AuthTokenTests(OAuthFlowTests):
         self.test_oauth_flow()
 
         # Delete consumer
-        consumer_id = self.consumer.key
+        consumer_id = self.consumer['key']
         resp = self.delete('/OS-OAUTH1/consumers/%(consumer_id)s'
                            % {'consumer_id': consumer_id})
         self.assertResponseStatus(resp, 204)
@@ -382,8 +418,8 @@ class AuthTokenTests(OAuthFlowTests):
         # List access_token should be 0
         resp = self.get('/users/%(user_id)s/OS-OAUTH1/access_tokens'
                         % {'user_id': self.user_id})
-        entities = resp.result.get('access_tokens')
-        self.assertEqual(len(entities), 0)
+        entities = resp.result['access_tokens']
+        self.assertEqual([], entities)
 
         # Check Keystone Token no longer exists
         headers = {'X-Subject-Token': self.keystone_token_id,
@@ -442,144 +478,46 @@ class AuthTokenTests(OAuthFlowTests):
             method='POST',
             expected_status=403)
 
-    def test_list_keystone_tokens_by_consumer(self):
+    def test_delete_keystone_tokens_by_consumer_id(self):
         self.test_oauth_flow()
-        tokens = self.token_api.list_tokens(self.user_id,
-                                            consumer_id=self.consumer.key)
-        keystone_token_uuid = cms.cms_hash_token(self.keystone_token_id)
-        self.assertTrue(len(tokens) > 0)
-        self.assertTrue(keystone_token_uuid in tokens)
-
-    def _create_trust_get_token(self):
-        ref = self.new_trust_ref(
-            trustor_user_id=self.user_id,
-            trustee_user_id=self.user_id,
-            project_id=self.project_id,
-            impersonation=True,
-            expires=dict(minutes=1),
-            role_ids=[self.role_id])
-        del ref['id']
-
-        r = self.post('/OS-TRUST/trusts', body={'trust': ref})
-        trust = self.assertValidTrustResponse(r)
-
-        auth_data = self.build_authentication_request(
-            user_id=self.user['id'],
-            password=self.user['password'],
-            trust_id=trust['id'])
-        r = self.post('/auth/tokens', body=auth_data)
-
-        trust_token = r.headers['X-Subject-Token']
-        return trust_token
-
-    def _approve_request_token_url(self):
-        consumer = self._create_single_consumer()
-        consumer_id = consumer.get('id')
-        consumer_secret = consumer.get('secret')
-        self.consumer = oauth1.Consumer(consumer_id, consumer_secret)
-        self.assertIsNotNone(self.consumer.key)
-
-        url, headers = self._create_request_token(self.consumer,
-                                                  self.project_id)
-        content = self.post(url, headers=headers)
-        credentials = urlparse.parse_qs(content.result)
-        request_key = credentials.get('oauth_token')[0]
-        request_secret = credentials.get('oauth_token_secret')[0]
-        self.request_token = oauth1.Token(request_key, request_secret)
-        self.assertIsNotNone(self.request_token.key)
-
-        url = self._authorize_request_token(request_key)
-
-        return url
-
-    def test_oauth_token_cannot_create_new_trust(self):
-        self.test_oauth_flow()
-        ref = self.new_trust_ref(
-            trustor_user_id=self.user_id,
-            trustee_user_id=self.user_id,
-            project_id=self.project_id,
-            impersonation=True,
-            expires=dict(minutes=1),
-            role_ids=[self.role_id])
-        del ref['id']
-
-        self.post('/OS-TRUST/trusts',
-                  body={'trust': ref},
-                  token=self.keystone_token_id,
-                  expected_status=403)
-
-    def test_oauth_token_cannot_authorize_request_token(self):
-        self.test_oauth_flow()
-        url = self._approve_request_token_url()
-        body = {'roles': [{'id': self.role_id}]}
-        self.put(url, body=body, token=self.keystone_token_id,
-                 expected_status=403)
-
-    def test_oauth_token_cannot_list_request_tokens(self):
-        self._set_policy({"identity:list_access_tokens": [],
-                          "identity:create_consumer": [],
-                          "identity:authorize_request_token": []})
-        self.test_oauth_flow()
-        url = '/users/%s/OS-OAUTH1/access_tokens' % self.user_id
-        self.get(url, token=self.keystone_token_id,
-                 expected_status=403)
-
-    def _set_policy(self, new_policy):
-        _unused, self.tmpfilename = tempfile.mkstemp()
-        rules.reset()
-        self.opt(policy_file=self.tmpfilename)
-        with open(self.tmpfilename, "w") as policyfile:
-            policyfile.write(jsonutils.dumps(new_policy))
-        self.addCleanup(os.remove, self.tmpfilename)
-
-    def test_trust_token_cannot_authorize_request_token(self):
-        trust_token = self._create_trust_get_token()
-        url = self._approve_request_token_url()
-        body = {'roles': [{'id': self.role_id}]}
-        self.put(url, body=body, token=trust_token, expected_status=403)
-
-    def test_trust_token_cannot_list_request_tokens(self):
-        self._set_policy({"identity:list_access_tokens": [],
-                          "identity:create_trust": []})
-        trust_token = self._create_trust_get_token()
-        url = '/users/%s/OS-OAUTH1/access_tokens' % self.user_id
-        self.get(url, token=trust_token, expected_status=403)
+        self.token_api.get_token(self.keystone_token_id)
+        self.token_api.delete_tokens(self.user_id,
+                                     consumer_id=self.consumer['key'])
+        self.assertRaises(exception.TokenNotFound, self.token_api.get_token,
+                          self.keystone_token_id)
 
 
 class MaliciousOAuth1Tests(OAuth1Tests):
 
     def test_bad_consumer_secret(self):
         consumer = self._create_single_consumer()
-        consumer_id = consumer.get('id')
-        consumer = oauth1.Consumer(consumer_id, "bad_secret")
-        url, headers = self._create_request_token(consumer,
-                                                  self.project_id)
-        self.post(url, headers=headers, expected_status=500)
+        consumer_id = consumer['id']
+        consumer = {'key': consumer_id, 'secret': uuid.uuid4().hex}
+        url, headers = self._create_request_token(consumer, self.project_id)
+        self.post(url, headers=headers, expected_status=401)
 
     def test_bad_request_token_key(self):
         consumer = self._create_single_consumer()
-        consumer_id = consumer.get('id')
-        consumer_secret = consumer.get('secret')
-        consumer = oauth1.Consumer(consumer_id, consumer_secret)
-        url, headers = self._create_request_token(consumer,
-                                                  self.project_id)
+        consumer_id = consumer['id']
+        consumer_secret = consumer['secret']
+        consumer = {'key': consumer_id, 'secret': consumer_secret}
+        url, headers = self._create_request_token(consumer, self.project_id)
         self.post(url, headers=headers)
-        url = self._authorize_request_token("bad_key")
+        url = self._authorize_request_token(uuid.uuid4().hex)
         body = {'roles': [{'id': self.role_id}]}
         self.put(url, body=body, expected_status=404)
 
     def test_bad_verifier(self):
         consumer = self._create_single_consumer()
-        consumer_id = consumer.get('id')
-        consumer_secret = consumer.get('secret')
-        consumer = oauth1.Consumer(consumer_id, consumer_secret)
+        consumer_id = consumer['id']
+        consumer_secret = consumer['secret']
+        consumer = {'key': consumer_id, 'secret': consumer_secret}
 
-        url, headers = self._create_request_token(consumer,
-                                                  self.project_id)
+        url, headers = self._create_request_token(consumer, self.project_id)
         content = self.post(url, headers=headers)
-        credentials = urlparse.parse_qs(content.result)
-        request_key = credentials.get('oauth_token')[0]
-        request_secret = credentials.get('oauth_token_secret')[0]
+        credentials = urllib.parse.parse_qs(content.result)
+        request_key = credentials['oauth_token'][0]
+        request_secret = credentials['oauth_token_secret'][0]
         request_token = oauth1.Token(request_key, request_secret)
 
         url = self._authorize_request_token(request_key)
@@ -588,26 +526,23 @@ class MaliciousOAuth1Tests(OAuth1Tests):
         verifier = resp.result['token']['oauth_verifier']
         self.assertIsNotNone(verifier)
 
-        request_token.set_verifier("bad verifier")
-        url, headers = self._create_access_token(consumer,
-                                                 request_token)
+        request_token.set_verifier(uuid.uuid4().hex)
+        url, headers = self._create_access_token(consumer, request_token)
         self.post(url, headers=headers, expected_status=401)
 
     def test_bad_authorizing_roles(self):
         consumer = self._create_single_consumer()
-        consumer_id = consumer.get('id')
-        consumer_secret = consumer.get('secret')
-        consumer = oauth1.Consumer(consumer_id, consumer_secret)
+        consumer_id = consumer['id']
+        consumer_secret = consumer['secret']
+        consumer = {'key': consumer_id, 'secret': consumer_secret}
 
-        url, headers = self._create_request_token(consumer,
-                                                  self.project_id)
+        url, headers = self._create_request_token(consumer, self.project_id)
         content = self.post(url, headers=headers)
-        credentials = urlparse.parse_qs(content.result)
-        request_key = credentials.get('oauth_token')[0]
+        credentials = urllib.parse.parse_qs(content.result)
+        request_key = credentials['oauth_token'][0]
 
-        self.identity_api.remove_role_from_user_and_project(self.user_id,
-                                                            self.project_id,
-                                                            self.role_id)
+        self.assignment_api.remove_role_from_user_and_project(
+            self.user_id, self.project_id, self.role_id)
         url = self._authorize_request_token(request_key)
         body = {'roles': [{'id': self.role_id}]}
         self.admin_request(path=url, method='PUT',
@@ -617,17 +552,17 @@ class MaliciousOAuth1Tests(OAuth1Tests):
         CONF.oauth1.request_token_duration = -1
 
         consumer = self._create_single_consumer()
-        consumer_id = consumer.get('id')
-        consumer_secret = consumer.get('secret')
-        self.consumer = oauth1.Consumer(consumer_id, consumer_secret)
-        self.assertIsNotNone(self.consumer.key)
+        consumer_id = consumer['id']
+        consumer_secret = consumer['secret']
+        self.consumer = {'key': consumer_id, 'secret': consumer_secret}
+        self.assertIsNotNone(self.consumer['key'])
 
         url, headers = self._create_request_token(self.consumer,
                                                   self.project_id)
         content = self.post(url, headers=headers)
-        credentials = urlparse.parse_qs(content.result)
-        request_key = credentials.get('oauth_token')[0]
-        request_secret = credentials.get('oauth_token_secret')[0]
+        credentials = urllib.parse.parse_qs(content.result)
+        request_key = credentials['oauth_token'][0]
+        request_secret = credentials['oauth_token_secret'][0]
         self.request_token = oauth1.Token(request_key, request_secret)
         self.assertIsNotNone(self.request_token.key)
 
@@ -638,17 +573,17 @@ class MaliciousOAuth1Tests(OAuth1Tests):
     def test_expired_creating_keystone_token(self):
         CONF.oauth1.access_token_duration = -1
         consumer = self._create_single_consumer()
-        consumer_id = consumer.get('id')
-        consumer_secret = consumer.get('secret')
-        self.consumer = oauth1.Consumer(consumer_id, consumer_secret)
-        self.assertIsNotNone(self.consumer.key)
+        consumer_id = consumer['id']
+        consumer_secret = consumer['secret']
+        self.consumer = {'key': consumer_id, 'secret': consumer_secret}
+        self.assertIsNotNone(self.consumer['key'])
 
         url, headers = self._create_request_token(self.consumer,
                                                   self.project_id)
         content = self.post(url, headers=headers)
-        credentials = urlparse.parse_qs(content.result)
-        request_key = credentials.get('oauth_token')[0]
-        request_secret = credentials.get('oauth_token_secret')[0]
+        credentials = urllib.parse.parse_qs(content.result)
+        request_key = credentials['oauth_token'][0]
+        request_secret = credentials['oauth_token_secret'][0]
         self.request_token = oauth1.Token(request_key, request_secret)
         self.assertIsNotNone(self.request_token.key)
 
@@ -661,9 +596,9 @@ class MaliciousOAuth1Tests(OAuth1Tests):
         url, headers = self._create_access_token(self.consumer,
                                                  self.request_token)
         content = self.post(url, headers=headers)
-        credentials = urlparse.parse_qs(content.result)
-        access_key = credentials.get('oauth_token')[0]
-        access_secret = credentials.get('oauth_token_secret')[0]
+        credentials = urllib.parse.parse_qs(content.result)
+        access_key = credentials['oauth_token'][0]
+        access_secret = credentials['oauth_token_secret'][0]
         self.access_token = oauth1.Token(access_key, access_secret)
         self.assertIsNotNone(self.access_token.key)
 
