@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,14 +12,20 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import gettext
+import socket
 import uuid
 
 from babel import localedata
-import gettext
+import mock
+import webob
 
+from keystone.common import environment
 from keystone.common import wsgi
 from keystone import exception
+from keystone.openstack.common.fixture import moxstubout
 from keystone.openstack.common import gettextutils
+from keystone.openstack.common.gettextutils import _
 from keystone.openstack.common import jsonutils
 from keystone import tests
 
@@ -37,7 +41,7 @@ class BaseWSGITest(tests.TestCase):
         super(BaseWSGITest, self).setUp()
 
     def _make_request(self, url='/'):
-        req = wsgi.Request.blank(url)
+        req = webob.Request.blank(url)
         args = {'action': 'index', 'controller': None}
         req.environ['wsgiorg.routing_args'] = [None, args]
         return req
@@ -104,13 +108,20 @@ class ApplicationTest(BaseWSGITest):
 
         app = FakeApp.factory({}, testkey="test")
         self.assertIn("testkey", app.kwargs)
-        self.assertEquals("test", app.kwargs["testkey"])
+        self.assertEqual("test", app.kwargs["testkey"])
 
     def test_render_exception(self):
         e = exception.Unauthorized(message=u'\u7f51\u7edc')
         resp = wsgi.render_exception(e)
         self.assertEqual(resp.status_int, 401)
 
+    def test_render_exception_host(self):
+        e = exception.Unauthorized(message=u'\u7f51\u7edc')
+        context = {'host_url': 'http://%s:5000' % uuid.uuid4().hex}
+        resp = wsgi.render_exception(e, context=context)
+
+        self.assertEqual(resp.status_int, 401)
+
 
 class ExtensionRouterTest(BaseWSGITest):
     def test_extensionrouter_local_config(self):
@@ -121,7 +132,7 @@ class ExtensionRouterTest(BaseWSGITest):
         factory = FakeRouter.factory({}, testkey="test")
         app = factory(self.app)
         self.assertIn("testkey", app.kwargs)
-        self.assertEquals("test", app.kwargs["testkey"])
+        self.assertEqual("test", app.kwargs["testkey"])
 
 
 class MiddlewareTest(BaseWSGITest):
@@ -152,7 +163,7 @@ class MiddlewareTest(BaseWSGITest):
         req = self._make_request()
         req.environ['REMOTE_ADDR'] = '127.0.0.1'
         resp = FakeMiddleware(self.app)(req)
-        self.assertEquals(resp.status_int, exception.Unauthorized.code)
+        self.assertEqual(resp.status_int, exception.Unauthorized.code)
 
     def test_middleware_type_error(self):
         class FakeMiddleware(wsgi.Middleware):
@@ -163,17 +174,29 @@ class MiddlewareTest(BaseWSGITest):
         req.environ['REMOTE_ADDR'] = '127.0.0.1'
         resp = FakeMiddleware(self.app)(req)
         # This is a validationerror type
-        self.assertEquals(resp.status_int, exception.ValidationError.code)
+        self.assertEqual(resp.status_int, exception.ValidationError.code)
 
     def test_middleware_exception_error(self):
+
+        exception_str = 'EXCEPTIONERROR'
+
         class FakeMiddleware(wsgi.Middleware):
             def process_response(self, request, response):
-                raise exception.UnexpectedError("EXCEPTIONERROR")
+                raise exception.UnexpectedError(exception_str)
 
-        req = self._make_request()
-        resp = FakeMiddleware(self.app)(req)
-        self.assertEquals(resp.status_int, exception.UnexpectedError.code)
-        self.assertIn("EXCEPTIONERROR", resp.body)
+        def do_request():
+            req = self._make_request()
+            resp = FakeMiddleware(self.app)(req)
+            self.assertEqual(resp.status_int, exception.UnexpectedError.code)
+            return resp
+
+        # Exception data should not be in the message when debug is False
+        self.config_fixture.config(debug=False)
+        self.assertNotIn(exception_str, do_request().body)
+
+        # Exception data should be in the message when debug is True
+        self.config_fixture.config(debug=True)
+        self.assertIn(exception_str, do_request().body)
 
     def test_middleware_local_config(self):
         class FakeMiddleware(wsgi.Middleware):
@@ -183,50 +206,18 @@ class MiddlewareTest(BaseWSGITest):
         factory = FakeMiddleware.factory({}, testkey="test")
         app = factory(self.app)
         self.assertIn("testkey", app.kwargs)
-        self.assertEquals("test", app.kwargs["testkey"])
-
-
-class WSGIFunctionTest(tests.TestCase):
-    def test_mask_password(self):
-        message = ("test = 'password': 'aaaaaa', 'param1': 'value1', "
-                   "\"new_password\": 'bbbbbb'")
-        self.assertEqual(wsgi.mask_password(message, True),
-                         u"test = 'password': '***', 'param1': 'value1', "
-                         "\"new_password\": '***'")
-
-        message = "test = 'password'  :   'aaaaaa'"
-        self.assertEqual(wsgi.mask_password(message, False, '111'),
-                         "test = 'password'  :   '111'")
-
-        message = u"test = u'password' : u'aaaaaa'"
-        self.assertEqual(wsgi.mask_password(message, True),
-                         u"test = u'password' : u'***'")
-
-        message = 'test = "password" : "aaaaaaaaa"'
-        self.assertEqual(wsgi.mask_password(message),
-                         'test = "password" : "***"')
-
-        message = 'test = "original_password" : "aaaaaaaaa"'
-        self.assertEqual(wsgi.mask_password(message),
-                         'test = "original_password" : "***"')
-
-        message = 'test = "original_password" : ""'
-        self.assertEqual(wsgi.mask_password(message),
-                         'test = "original_password" : "***"')
-
-        message = 'test = "param1" : "value"'
-        self.assertEqual(wsgi.mask_password(message),
-                         'test = "param1" : "value"')
+        self.assertEqual("test", app.kwargs["testkey"])
 
 
 class LocalizedResponseTest(tests.TestCase):
     def setUp(self):
         super(LocalizedResponseTest, self).setUp()
-        gettextutils._AVAILABLE_LANGUAGES.clear()
 
-    def tearDown(self):
         gettextutils._AVAILABLE_LANGUAGES.clear()
-        super(LocalizedResponseTest, self).tearDown()
+        self.addCleanup(gettextutils._AVAILABLE_LANGUAGES.clear)
+
+        fixture = self.useFixture(moxstubout.MoxStubout())
+        self.stubs = fixture.stubs
 
     def _set_expected_languages(self, all_locales=[], avail_locales=None):
         # Override localedata.locale_identifiers to return some locales.
@@ -250,8 +241,8 @@ class LocalizedResponseTest(tests.TestCase):
 
     def test_request_match_default(self):
         # The default language if no Accept-Language is provided is None
-        req = wsgi.Request.blank('/')
-        self.assertIsNone(req.best_match_language())
+        req = webob.Request.blank('/')
+        self.assertIsNone(wsgi.best_match_language(req))
 
     def test_request_match_language_expected(self):
         # If Accept-Language is a supported language, best_match_language()
@@ -259,8 +250,8 @@ class LocalizedResponseTest(tests.TestCase):
 
         self._set_expected_languages(all_locales=['it'])
 
-        req = wsgi.Request.blank('/', headers={'Accept-Language': 'it'})
-        self.assertEquals(req.best_match_language(), 'it')
+        req = webob.Request.blank('/', headers={'Accept-Language': 'it'})
+        self.assertEqual(wsgi.best_match_language(req), 'it')
 
     def test_request_match_language_unexpected(self):
         # If Accept-Language is a language we do not support,
@@ -268,36 +259,8 @@ class LocalizedResponseTest(tests.TestCase):
 
         self._set_expected_languages(all_locales=['it'])
 
-        req = wsgi.Request.blank('/', headers={'Accept-Language': 'zh'})
-        self.assertIsNone(req.best_match_language())
-
-    def test_localized_message(self):
-        # If the accept-language header is set on the request, the localized
-        # message is returned by calling get_localized_message.
-
-        LANG_ID = uuid.uuid4().hex
-        ORIGINAL_TEXT = uuid.uuid4().hex
-        TRANSLATED_TEXT = uuid.uuid4().hex
-
-        self._set_expected_languages(all_locales=[LANG_ID])
-
-        def fake_get_localized_message(message, user_locale):
-            if (user_locale == LANG_ID and
-                    message == ORIGINAL_TEXT):
-                return TRANSLATED_TEXT
-
-        self.stubs.Set(gettextutils, 'get_localized_message',
-                       fake_get_localized_message)
-
-        error = exception.NotFound(message=ORIGINAL_TEXT)
-        resp = wsgi.render_exception(error, user_locale=LANG_ID)
-        result = jsonutils.loads(resp.body)
-
-        exp = {'error': {'message': TRANSLATED_TEXT,
-                         'code': 404,
-                         'title': 'Not Found'}}
-
-        self.assertEqual(exp, result)
+        req = webob.Request.blank('/', headers={'Accept-Language': 'zh'})
+        self.assertIsNone(wsgi.best_match_language(req))
 
     def test_static_translated_string_is_Message(self):
         # Statically created message strings are Message objects so that they
@@ -310,3 +273,68 @@ class LocalizedResponseTest(tests.TestCase):
         # are lazy-translated.
         self.assertIsInstance(_('The resource could not be found.'),
                               gettextutils.Message)
+
+
+class ServerTest(tests.TestCase):
+
+    def setUp(self):
+        super(ServerTest, self).setUp()
+        environment.use_eventlet()
+        self.host = '127.0.0.1'
+        self.port = '1234'
+
+    @mock.patch('eventlet.listen')
+    @mock.patch('socket.getaddrinfo')
+    def test_keepalive_unset(self, mock_getaddrinfo, mock_listen):
+        mock_getaddrinfo.return_value = [(1, 2, 3, 4, 5)]
+        mock_sock = mock.Mock()
+        mock_sock.setsockopt = mock.Mock()
+
+        mock_listen.return_value = mock_sock
+        server = environment.Server(mock.MagicMock(), host=self.host,
+                                    port=self.port)
+        server.start()
+        self.assertTrue(mock_listen.called)
+        self.assertFalse(mock_sock.setsockopt.called)
+
+    @mock.patch('eventlet.listen')
+    @mock.patch('socket.getaddrinfo')
+    def test_keepalive_set(self, mock_getaddrinfo, mock_listen):
+        mock_getaddrinfo.return_value = [(1, 2, 3, 4, 5)]
+        mock_sock = mock.Mock()
+        mock_sock.setsockopt = mock.Mock()
+
+        mock_listen.return_value = mock_sock
+        server = environment.Server(mock.MagicMock(), host=self.host,
+                                    port=self.port, keepalive=True)
+        server.start()
+        mock_sock.setsockopt.assert_called_once_with(socket.SOL_SOCKET,
+                                                     socket.SO_KEEPALIVE,
+                                                     1)
+        self.assertTrue(mock_listen.called)
+
+    @mock.patch('eventlet.listen')
+    @mock.patch('socket.getaddrinfo')
+    def test_keepalive_and_keepidle_set(self, mock_getaddrinfo, mock_listen):
+        mock_getaddrinfo.return_value = [(1, 2, 3, 4, 5)]
+        mock_sock = mock.Mock()
+        mock_sock.setsockopt = mock.Mock()
+
+        mock_listen.return_value = mock_sock
+        server = environment.Server(mock.MagicMock(), host=self.host,
+                                    port=self.port, keepalive=True,
+                                    keepidle=1)
+        server.start()
+
+        # keepidle isn't available in the OS X version of eventlet
+        if hasattr(socket, 'TCP_KEEPIDLE'):
+            self.assertEqual(mock_sock.setsockopt.call_count, 2)
+
+            # Test the last set of call args i.e. for the keepidle
+            mock_sock.setsockopt.assert_called_with(socket.IPPROTO_TCP,
+                                                    socket.TCP_KEEPIDLE,
+                                                    1)
+        else:
+            self.assertEqual(mock_sock.setsockopt.call_count, 1)
+
+        self.assertTrue(mock_listen.called)
