@@ -11,33 +11,35 @@
 # under the License.
 
 
-from sqlalchemy.orm import declarative_base
+from sqlalchemy.ext import declarative
+import testtools
 
 from keystone.common import sql
-from keystone.tests import unit
-from keystone.tests.unit import utils
-
-ModelBase = declarative_base()
+from keystone.tests import utils
 
 
-class TestModel(ModelBase, sql.ModelDictMixin):  # type: ignore
+ModelBase = declarative.declarative_base()
+
+
+class TestModel(ModelBase, sql.ModelDictMixin):
     __tablename__ = 'testmodel'
     id = sql.Column(sql.String(64), primary_key=True)
     text = sql.Column(sql.String(64), nullable=False)
 
 
-class TestModelDictMixin(unit.BaseTestCase):
+class TestModelDictMixin(testtools.TestCase):
+
     def test_creating_a_model_instance_from_a_dict(self):
         d = {'id': utils.new_uuid(), 'text': utils.new_uuid()}
         m = TestModel.from_dict(d)
-        self.assertEqual(d['id'], m.id)
-        self.assertEqual(d['text'], m.text)
+        self.assertEqual(m.id, d['id'])
+        self.assertEqual(m.text, d['text'])
 
     def test_creating_a_dict_from_a_model_instance(self):
         m = TestModel(id=utils.new_uuid(), text=utils.new_uuid())
         d = m.to_dict()
-        self.assertEqual(d['id'], m.id)
-        self.assertEqual(d['text'], m.text)
+        self.assertEqual(m.id, d['id'])
+        self.assertEqual(m.text, d['text'])
 
     def test_creating_a_model_instance_from_an_invalid_dict(self):
         d = {'id': utils.new_uuid(), 'text': utils.new_uuid(), 'extra': None}
@@ -47,6 +49,4 @@ class TestModelDictMixin(unit.BaseTestCase):
         expected = {'id': utils.new_uuid(), 'text': utils.new_uuid()}
         m = TestModel(id=expected['id'], text=expected['text'])
         m.extra = 'this should not be in the dictionary'
-        # NOTE(notmorgan): This is currently explicitly harmless as this does
-        # not actually use SQL-Alchemy.
-        self.assertEqual(expected, m.to_dict())
+        self.assertEqual(m.to_dict(), expected)
