@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,22 +12,25 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import datetime
 import json
 
-from keystone.common import cms
+from keystoneclient.common import cms
+import six
+
 from keystone.common import controller
 from keystone.common import dependency
 from keystone.common import wsgi
 from keystone import config
 from keystone import exception
-from keystone.openstack.common import log as logging
+from keystone.openstack.common.gettextutils import _
+from keystone.openstack.common import log
 from keystone.openstack.common import timeutils
 from keystone.token import core
 
 
 CONF = config.CONF
-LOG = logging.getLogger(__name__)
-DEFAULT_DOMAIN_ID = CONF.identity.default_domain_id
+LOG = log.getLogger(__name__)
 
 
 class ExternalAuthNotApplicable(Exception):
@@ -37,20 +38,25 @@ class ExternalAuthNotApplicable(Exception):
     pass
 
 
-@dependency.requires('token_provider_api')
+@dependency.requires('assignment_api', 'catalog_api', 'identity_api',
+                     'token_api', 'token_provider_api', 'trust_api')
 class Auth(controller.V2Controller):
+
+    @controller.v2_deprecated
     def ca_cert(self, context, auth=None):
         ca_file = open(CONF.signing.ca_certs, 'r')
         data = ca_file.read()
         ca_file.close()
         return data
 
+    @controller.v2_deprecated
     def signing_cert(self, context, auth=None):
         cert_file = open(CONF.signing.certfile, 'r')
         data = cert_file.read()
         cert_file.close()
         return data
 
+    @controller.v2_deprecated
     def authenticate(self, context, auth=None):
         """Authenticate credentials and return a token.
 
@@ -77,8 +83,6 @@ class Auth(controller.V2Controller):
             raise exception.ValidationError(attribute='auth',
                                             target='request body')
 
-        auth_token_data = None
-
         if "token" in auth:
             # Try to authenticate using a token
             auth_info = self._authenticate_token(
@@ -100,7 +104,7 @@ class Auth(controller.V2Controller):
         # The user_ref is encoded into the auth_token_data which is returned as
         # part of the token data. The token provider doesn't care about the
         # format.
-        user_ref = self.identity_api.v3_to_v2_user(user_ref)
+        user_ref = self.v3_to_v2_user(user_ref)
         if tenant_ref:
             tenant_ref = self.filter_domain_id(tenant_ref)
         auth_token_data = self._get_auth_token_data(user_ref,
@@ -120,18 +124,11 @@ class Auth(controller.V2Controller):
 
         roles_ref = []
         for role_id in metadata_ref.get('roles', []):
-            role_ref = self.identity_api.get_role(role_id)
+            role_ref = self.assignment_api.get_role(role_id)
             roles_ref.append(dict(name=role_ref['name']))
 
         (token_id, token_data) = self.token_provider_api.issue_v2_token(
             auth_token_data, roles_ref=roles_ref, catalog_ref=catalog_ref)
-
-        # NOTE(nkinder): We only log this message in Havana.  In Icehouse and
-        # later, we emit a CADF notification upon successful token issuance
-        # instead.
-        msg = _('Issued token for user %(u_id)s')
-        msg = msg % {'u_id': user_ref['id']}
-        LOG.info(msg)
         return token_data
 
     def _authenticate_token(self, context, auth):
@@ -167,8 +164,6 @@ class Auth(controller.V2Controller):
 
         user_ref = old_token_ref['user']
         user_id = user_ref['id']
-        tenant_id = self._get_project_id_from_auth(auth)
-
         if not CONF.trust.enabled and 'trust_id' in auth:
             raise exception.Forbidden('Trusts are disabled.')
         elif CONF.trust.enabled and 'trust_id' in auth:
@@ -177,9 +172,6 @@ class Auth(controller.V2Controller):
                 raise exception.Forbidden()
             if user_id != trust_ref['trustee_user_id']:
                 raise exception.Forbidden()
-            if (trust_ref['project_id'] and
-                    tenant_id != trust_ref['project_id']):
-                raise exception.Forbidden()
             if ('expires' in trust_ref) and (trust_ref['expires']):
                 expiry = trust_ref['expires']
                 if expiry < timeutils.parse_isotime(timeutils.isotime()):
@@ -193,6 +185,8 @@ class Auth(controller.V2Controller):
                 trust_ref['trustee_user_id'])
             if not trustee_user_ref['enabled']:
                 raise exception.Forbidden()()
+            self.trust_api.consume_use(auth['trust_id'])
+
             if trust_ref['impersonation'] is True:
                 current_user_ref = trustor_user_ref
             else:
@@ -202,6 +196,7 @@ class Auth(controller.V2Controller):
             current_user_ref = self.identity_api.get_user(user_id)
 
         metadata_ref = {}
+        tenant_id = self._get_project_id_from_auth(auth)
         tenant_ref, metadata_ref['roles'] = self._get_project_roles_and_ref(
             user_id, tenant_id)
 
@@ -224,7 +219,7 @@ class Auth(controller.V2Controller):
             metadata_ref['trustee_user_id'] = trust_ref['trustee_user_id']
             metadata_ref['trust_id'] = trust_id
 
-        bind = old_token_ref.get('bind', None)
+        bind = old_token_ref.get('bind')
 
         return (current_user_ref, tenant_ref, metadata_ref, expiry, bind)
 
@@ -252,7 +247,7 @@ class Auth(controller.V2Controller):
                 attribute='username or userId',
                 target='passwordCredentials')
 
-        user_id = auth['passwordCredentials'].get('userId', None)
+        user_id = auth['passwordCredentials'].get('userId')
         if user_id and len(user_id) > CONF.max_param_size:
             raise exception.ValidationSizeError(attribute='userId',
                                                 size=CONF.max_param_size)
@@ -265,17 +260,18 @@ class Auth(controller.V2Controller):
         if username:
             try:
                 user_ref = self.identity_api.get_user_by_name(
-                    username, DEFAULT_DOMAIN_ID)
+                    username, CONF.identity.default_domain_id)
                 user_id = user_ref['id']
             except exception.UserNotFound as e:
                 raise exception.Unauthorized(e)
 
         try:
             user_ref = self.identity_api.authenticate(
+                context,
                 user_id=user_id,
                 password=password)
         except AssertionError as e:
-            raise exception.Unauthorized(e)
+            raise exception.Unauthorized(e.args[0])
 
         metadata_ref = {}
         tenant_id = self._get_project_id_from_auth(auth)
@@ -290,7 +286,7 @@ class Auth(controller.V2Controller):
 
         Returns auth_token_data, (user_ref, tenant_ref, metadata_ref)
         """
-        if 'REMOTE_USER' not in context:
+        if 'REMOTE_USER' not in context.get('environment', {}):
             raise ExternalAuthNotApplicable()
 
         #NOTE(jamielennox): xml and json differ and get confused about what
@@ -298,10 +294,10 @@ class Auth(controller.V2Controller):
         if not auth:
             auth = {}
 
-        username = context['REMOTE_USER']
+        username = context['environment']['REMOTE_USER']
         try:
             user_ref = self.identity_api.get_user_by_name(
-                username, DEFAULT_DOMAIN_ID)
+                username, CONF.identity.default_domain_id)
             user_id = user_ref['id']
         except exception.UserNotFound as e:
             raise exception.Unauthorized(e)
@@ -314,7 +310,8 @@ class Auth(controller.V2Controller):
         expiry = core.default_expire_time()
         bind = None
         if ('kerberos' in CONF.token.bind and
-                context.get('AUTH_TYPE', '').lower() == 'negotiate'):
+                context['environment'].
+                get('AUTH_TYPE', '').lower() == 'negotiate'):
             bind = {'kerberos': username}
 
         return (user_ref, tenant_ref, metadata_ref, expiry, bind)
@@ -330,44 +327,25 @@ class Auth(controller.V2Controller):
 
         Returns a valid tenant_id if it exists, or None if not specified.
         """
-        tenant_id = auth.get('tenantId', None)
+        tenant_id = auth.get('tenantId')
         if tenant_id and len(tenant_id) > CONF.max_param_size:
             raise exception.ValidationSizeError(attribute='tenantId',
                                                 size=CONF.max_param_size)
 
-        tenant_name = auth.get('tenantName', None)
+        tenant_name = auth.get('tenantName')
         if tenant_name and len(tenant_name) > CONF.max_param_size:
             raise exception.ValidationSizeError(attribute='tenantName',
                                                 size=CONF.max_param_size)
 
         if tenant_name:
             try:
-                tenant_ref = self.identity_api.get_project_by_name(
-                    tenant_name, DEFAULT_DOMAIN_ID)
+                tenant_ref = self.assignment_api.get_project_by_name(
+                    tenant_name, CONF.identity.default_domain_id)
                 tenant_id = tenant_ref['id']
             except exception.ProjectNotFound as e:
                 raise exception.Unauthorized(e)
         return tenant_id
 
-    def _get_domain_id_from_auth(self, auth):
-        """Extract domain information from v3 auth dict.
-
-        Returns a valid domain_id if it exists, or None if not specified.
-        """
-        # FIXME(henry-nash): This is a placeholder that needs to be
-        # only called in the v3 context, and the auth.get calls
-        # converted to the v3 format
-        domain_id = auth.get('domainId', None)
-        domain_name = auth.get('domainName', None)
-        if domain_name:
-            try:
-                domain_ref = self.identity_api._get_domain_by_name(
-                    domain_name)
-                domain_id = domain_ref['id']
-            except exception.DomainNotFound as e:
-                raise exception.Unauthorized(e)
-        return domain_id
-
     def _get_project_roles_and_ref(self, user_id, tenant_id):
         """Returns the project roles for this user, and the project ref."""
 
@@ -375,8 +353,8 @@ class Auth(controller.V2Controller):
         role_list = []
         if tenant_id:
             try:
-                tenant_ref = self.identity_api.get_project(tenant_id)
-                role_list = self.identity_api.get_roles_for_user_and_project(
+                tenant_ref = self.assignment_api.get_project(tenant_id)
+                role_list = self.assignment_api.get_roles_for_user_and_project(
                     user_id, tenant_id)
             except exception.ProjectNotFound:
                 pass
@@ -405,6 +383,7 @@ class Auth(controller.V2Controller):
                     _('Token does not belong to specified tenant.'))
         return data
 
+    @controller.v2_deprecated
     @controller.protected()
     def validate_token_head(self, context, token_id):
         """Check that a token is valid.
@@ -414,9 +393,11 @@ class Auth(controller.V2Controller):
         Identical to ``validate_token``, except does not return a response.
 
         """
+        # TODO(ayoung) validate against revocation API
         belongs_to = context['query_string'].get('belongsTo')
         self.token_provider_api.check_v2_token(token_id, belongs_to)
 
+    @controller.v2_deprecated
     @controller.protected()
     def validate_token(self, context, token_id):
         """Check that a token is valid.
@@ -427,21 +408,26 @@ class Auth(controller.V2Controller):
 
         """
         belongs_to = context['query_string'].get('belongsTo')
+        # TODO(ayoung) validate against revocation API
         return self.token_provider_api.validate_v2_token(token_id, belongs_to)
 
+    @controller.v2_deprecated
     def delete_token(self, context, token_id):
         """Delete a token, effectively invalidating it for authz."""
         # TODO(termie): this stuff should probably be moved to middleware
         self.assert_admin(context)
-        self.token_api.delete_token(token_id)
+        self.token_provider_api.revoke_token(token_id)
 
+    @controller.v2_deprecated
     @controller.protected()
     def revocation_list(self, context, auth=None):
+        if not CONF.token.revoke_by_id:
+            raise exception.Gone()
         tokens = self.token_api.list_revoked_tokens()
 
         for t in tokens:
             expires = t['expires']
-            if not (expires and isinstance(expires, unicode)):
+            if expires and isinstance(expires, datetime.datetime):
                     t['expires'] = timeutils.isotime(expires)
         data = {'revoked': tokens}
         json_data = json.dumps(data)
@@ -451,6 +437,7 @@ class Auth(controller.V2Controller):
 
         return {'signed': signed_text}
 
+    @controller.v2_deprecated
     def endpoints(self, context, token_id):
         """Return a list of endpoints available to the token."""
         self.assert_admin(context)
@@ -490,8 +477,8 @@ class Auth(controller.V2Controller):
             return {}
 
         endpoints = []
-        for region_name, region_ref in catalog_ref.iteritems():
-            for service_type, service_ref in region_ref.iteritems():
+        for region_name, region_ref in six.iteritems(catalog_ref):
+            for service_type, service_ref in six.iteritems(region_ref):
                 endpoints.append({
                     'id': service_ref.get('id'),
                     'name': service_ref.get('name'),
