@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -18,20 +16,22 @@
 
 import json
 
-from keystone.common import cms
+from keystoneclient.common import cms
+
 from keystone.common import environment
 from keystone import config
 from keystone import exception
-from keystone.openstack.common import log as logging
-from keystone.token.providers import uuid
+from keystone.openstack.common.gettextutils import _
+from keystone.openstack.common import log
+from keystone.token.providers import common
 
 
 CONF = config.CONF
 
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 
-class Provider(uuid.Provider):
+class Provider(common.BaseProvider):
     def _get_token_id(self, token_data):
         try:
             token_id = cms.cms_sign_token(json.dumps(token_data),
@@ -39,6 +39,6 @@ class Provider(uuid.Provider):
                                           CONF.signing.keyfile)
             return token_id
         except environment.subprocess.CalledProcessError:
-            LOG.exception('Unable to sign token')
+            LOG.exception(_('Unable to sign token'))
             raise exception.UnexpectedError(_(
                 'Unable to sign token.'))
