@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,16 +14,19 @@
 
 import uuid
 
+import six
+
+from keystone import assignment
 from keystone.common import controller
 from keystone.common import dependency
 from keystone import config
 from keystone import exception
-from keystone import identity
-from keystone.openstack.common import log as logging
+from keystone.openstack.common.gettextutils import _
+from keystone.openstack.common import log
 from keystone.openstack.common import timeutils
 
 
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 CONF = config.CONF
 
 
@@ -34,10 +35,9 @@ def _trustor_only(context, trust, user_id):
         raise exception.Forbidden()
 
 
-def _admin_trustor_trustee_only(context, trust, user_id):
-    if (user_id != trust.get('trustor_user_id') and
-            user_id != trust.get('trustor_user_id') and
-            context['is_admin']):
+def _trustor_trustee_only(trust, user_id):
+    if (user_id != trust.get('trustee_user_id') and
+            user_id != trust.get('trustor_user_id')):
                 raise exception.Forbidden()
 
 
@@ -46,25 +46,20 @@ def _admin_trustor_only(context, trust, user_id):
         raise exception.Forbidden()
 
 
-@dependency.requires('identity_api', 'trust_api', 'token_api')
+@dependency.requires('assignment_api', 'identity_api', 'trust_api',
+                     'token_api')
 class TrustV3(controller.V3Controller):
     collection_name = "trusts"
     member_name = "trust"
 
     @classmethod
-    def base_url(cls, path=None):
-        endpoint = CONF.public_endpoint % CONF
+    def base_url(cls, context, path=None):
+        """Construct a path and pass it to V3Controller.base_url method."""
 
-        # allow a missing trailing slash in the config
-        if endpoint[-1] != '/':
-            endpoint += '/'
-
-        url = endpoint + 'v3/OS-TRUST'
-
-        if path:
-            return url + path
-        else:
-            return url + '/' + cls.collection_name
+        # NOTE(stevemar): Overriding path to /OS-TRUST/trusts so that
+        # V3Controller.base_url handles setting the self link correctly.
+        path = '/OS-TRUST/' + cls.collection_name
+        return super(TrustV3, cls).base_url(context, path=path)
 
     def _get_user_id(self, context):
         if 'token_id' in context:
@@ -79,17 +74,12 @@ class TrustV3(controller.V3Controller):
         trust = self.trust_api.get_trust(trust_id)
         if not trust:
             raise exception.TrustNotFound(trust_id)
-        _admin_trustor_trustee_only(context, trust, user_id)
-        if not trust:
-            raise exception.TrustNotFound(trust_id=trust_id)
-        if (user_id != trust['trustor_user_id'] and
-                user_id != trust['trustee_user_id']):
-            raise exception.Forbidden()
+        _trustor_trustee_only(trust, user_id)
         self._fill_in_roles(context, trust,
-                            self.identity_api.list_roles())
+                            self.assignment_api.list_roles())
         return TrustV3.wrap_member(context, trust)
 
-    def _fill_in_roles(self, context, trust, global_roles):
+    def _fill_in_roles(self, context, trust, all_roles):
         if trust.get('expires_at') is not None:
             trust['expires_at'] = (timeutils.isotime
                                    (trust['expires_at'],
@@ -99,33 +89,31 @@ class TrustV3(controller.V3Controller):
             trust['roles'] = []
         trust_full_roles = []
         for trust_role in trust['roles']:
-            if isinstance(trust_role, basestring):
+            if isinstance(trust_role, six.string_types):
                 trust_role = {'id': trust_role}
-            matching_roles = [x for x in global_roles
+            matching_roles = [x for x in all_roles
                               if x['id'] == trust_role['id']]
             if matching_roles:
-                full_role = identity.controllers.RoleV3.wrap_member(
+                full_role = assignment.controllers.RoleV3.wrap_member(
                     context, matching_roles[0])['role']
                 trust_full_roles.append(full_role)
         trust['roles'] = trust_full_roles
         trust['roles_links'] = {
-            'self': (self.base_url() + "/%s/roles" % trust['id']),
+            'self': (self.base_url(context) + "/%s/roles" % trust['id']),
             'next': None,
             'previous': None}
 
-    def _clean_role_list(self, context, trust, global_roles):
+    def _clean_role_list(self, context, trust, all_roles):
         trust_roles = []
-        global_role_names = dict((r['name'], r)
-                                 for r in
-                                 global_roles)
+        all_role_names = dict((r['name'], r) for r in all_roles)
         for role in trust.get('roles', []):
             if 'id' in role:
                 trust_roles.append({'id': role['id']})
             elif 'name' in role:
                 rolename = role['name']
-                if rolename in global_role_names:
+                if rolename in all_role_names:
                     trust_roles.append({'id':
-                                        global_role_names[rolename]['id']})
+                                        all_role_names[rolename]['id']})
                 else:
                     raise exception.RoleNotFound("role %s is not defined" %
                                                  rolename)
@@ -147,37 +135,40 @@ class TrustV3(controller.V3Controller):
         if not trust:
             raise exception.ValidationError(attribute='trust',
                                             target='request')
+        if trust.get('project_id') and not trust.get('roles'):
+            raise exception.Forbidden(
+                _('At least one role should be specified.'))
         try:
             user_id = self._get_user_id(context)
             _trustor_only(context, trust, user_id)
             #confirm that the trustee exists
-            trustee_ref = self.identity_api.get_user(trust['trustee_user_id'])
-            if not trustee_ref:
-                raise exception.UserNotFound(user_id=trust['trustee_user_id'])
-            global_roles = self.identity_api.list_roles()
-            clean_roles = self._clean_role_list(context, trust, global_roles)
+            self.identity_api.get_user(trust['trustee_user_id'])
+            all_roles = self.assignment_api.list_roles()
+            clean_roles = self._clean_role_list(context, trust, all_roles)
             if trust.get('project_id'):
-                user_roles = self.identity_api.get_roles_for_user_and_project(
-                    user_id, trust['project_id'])
+                user_role = self.assignment_api.get_roles_for_user_and_project(
+                    user_id,
+                    trust['project_id'])
             else:
-                user_roles = []
+                user_role = []
             for trust_role in clean_roles:
-                matching_roles = [x for x in user_roles
+                matching_roles = [x for x in user_role
                                   if x == trust_role['id']]
                 if not matching_roles:
                     raise exception.RoleNotFound(role_id=trust_role['id'])
             if trust.get('expires_at') is not None:
                 if not trust['expires_at'].endswith('Z'):
                     trust['expires_at'] += 'Z'
-                trust['expires_at'] = (timeutils.parse_isotime
-                                       (trust['expires_at']))
-            new_trust = self.trust_api.create_trust(
-                trust_id=uuid.uuid4().hex,
-                trust=trust,
-                roles=clean_roles)
-            self._fill_in_roles(context,
-                                new_trust,
-                                global_roles)
+                try:
+                    trust['expires_at'] = (timeutils.parse_isotime
+                                           (trust['expires_at']))
+                except ValueError:
+                    raise exception.ValidationTimeStampError()
+            trust_id = uuid.uuid4().hex
+            new_trust = self.trust_api.create_trust(trust_id,
+                                                    trust,
+                                                    clean_roles)
+            self._fill_in_roles(context, new_trust, all_roles)
             return TrustV3.wrap_member(context, new_trust)
         except KeyError as e:
             raise exception.ValidationError(attribute=e.args[0],
@@ -233,7 +224,7 @@ class TrustV3(controller.V3Controller):
         if not trust:
             raise exception.TrustNotFound(trust_id)
         user_id = self._get_user_id(context)
-        _admin_trustor_trustee_only(context, trust, user_id)
+        _trustor_trustee_only(trust, user_id)
         return {'roles': trust['roles'],
                 'links': trust['roles_links']}
 
@@ -244,31 +235,13 @@ class TrustV3(controller.V3Controller):
         if not trust:
             raise exception.TrustNotFound(trust_id)
         user_id = self._get_user_id(context)
-        _admin_trustor_trustee_only(context, trust, user_id)
-        matching_roles = [x for x in trust['roles']
-                          if x['id'] == role_id]
-        if not matching_roles:
+        _trustor_trustee_only(trust, user_id)
+        if not any(role['id'] == role_id for role in trust['roles']):
             raise exception.RoleNotFound(role_id=role_id)
 
     @controller.protected()
     def get_role_for_trust(self, context, trust_id, role_id):
-        """Checks if a role has been assigned to a trust."""
-        trust = self.trust_api.get_trust(trust_id)
-        if not trust:
-            raise exception.TrustNotFound(trust_id)
-
-        user_id = self._get_user_id(context)
-        _admin_trustor_trustee_only(context, trust, user_id)
-        matching_roles = [x for x in trust['roles']
-                          if x['id'] == role_id]
-        if not matching_roles:
-            raise exception.RoleNotFound(role_id=role_id)
-        global_roles = self.identity_api.list_roles()
-        matching_roles = [x for x in global_roles
-                          if x['id'] == role_id]
-        if matching_roles:
-            full_role = (identity.controllers.
-                         RoleV3.wrap_member(context, matching_roles[0]))
-            return full_role
-        else:
-            raise exception.RoleNotFound(role_id=role_id)
+        """Get a role that has been assigned to a trust."""
+        self.check_role_for_trust(context, trust_id, role_id)
+        role = self.assignment_api.get_role(role_id)
+        return assignment.controllers.RoleV3.wrap_member(context, role)
