@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2012 OpenStack LLC
+# Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -16,7 +14,6 @@
 """WSGI Routers for the Identity service."""
 
 from keystone.trust import controllers
-from keystone.common import router
 
 
 def append_v3_routers(mapper, routers):
